package fi.helsinki.cs.seco.onki.config;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.RDFList;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.vocabulary.RDFS;

public class OntologyServiceConfiguration extends ServiceConfiguration {

	public boolean getIsActive() {
		return this.getProperty("isActive").equals(this.onkiConfig.getOnkiConfResource("true"));
	}
	
	public String getShortName() {
		return this.getPropertyAsString("shortName");
	}
	
	public List<String> getDataSources() {
		return this.getDataSources("dataSource");
	}
	
	public List<String> getConfigSources() {
		return this.getDataSources("configSource");
	}
	
	public Map<String,String> getLabel() {
		return this.getMultilingualLiterals(RDFS.label);
	}

	public Map<String,String> getCopyright() {
		return this.getMultilingualLiterals("copyright");
	}
	
	protected List<String> getDataSources(String id) {
		RDFNode dataSource = this.getProperty(id);
		if (dataSource == null)
			return null;

		List<RDFNode> dataSources = null; 
		if (dataSource.canAs(RDFList.class)) {
			dataSources = ((RDFList)dataSource.as(RDFList.class)).asJavaList();
		} else {
			dataSources = new LinkedList<RDFNode>();
			dataSources.add(dataSource);
		}
		
		List<String> resultList = new LinkedList<String>();
		
		for (RDFNode node: dataSources) {
			String source = null;
			
			if (node.isURIResource()) {
				source = ((Resource)node.as(Resource.class)).getURI();
			
			// if the datasource is relative, add the directory defined in ServerConfiguration if available
			
			} else if (this.hasParentConfiguration(OnkiServerConfiguration.class)) {
				DataSourceConfiguration dsConf = ((OnkiServerConfiguration)this.parentConfig).getDataSource(id);
				if (dsConf != null && dsConf.getClass() != null)
					source = dsConf.getDir() + ((Literal)node.as(Literal.class)).getString();
			} else {
				System.err.println("WARNING: Do not know how to handle OntologyServiceConfiguration's "+id+": "+node);
			}
			
			if (source != null)
				resultList.add(source);
		}
		return resultList;
	}
}
