package fi.helsinki.cs.seco.onki.index;

import java.util.Set;

/**
 * An alphabetical index that can be used for fast retrieval of items (ontological concepts) based
 * on their label properties.
 * 
 * @author oisuomin
 */

public interface IAlphabeticalIndex {
	
	/**
	 * Initialize the index with a set of items.
	 * @param items set of items to store in the index
	 */
	
	public void initialize(Iterable<IndexItem> items);
	
	public void initialize(Iterable<IndexItem> items, Set<String> allowedNamespaces);
	
	/**
	 * Query the index for items that have a label (prefLabel or altLabel) matching the given prefix.
	 * @param language language of labels to search
	 * @param prefix label prefix to look for
	 * @return list of results, ordered alphabetically by the matching label
	 */
	
	public IndexResults search(String language, String prefix);
	
	public Iterable<String> getKnownLangs();
	
	public Iterable<String> getKnownTypes();
}