package fi.helsinki.cs.seco.onki.index;

import fi.helsinki.cs.seco.onki.utils.Encoder;

/**
 * Represents a result returned by the alphabetical index.
 * 
 * @author oisuomin
 */

public class IndexResult {
	private String uriNamespace;
	private String uriNamespacePrefix;
	private String uriLocalname;
	private String prefLabel;
	private String altLabel;

	public IndexResult(String uriNamespace, String uriNamespacePrefix, String uriLocalname,
			String prefLabel, String altLabel) {
		this.uriNamespace = uriNamespace;
		this.uriNamespacePrefix = uriNamespacePrefix;
		this.uriLocalname = uriLocalname;
		this.prefLabel = prefLabel;
		this.altLabel = altLabel;
	}
	
    /**
	 * Getters and setters.
	 */	
	
	public String getUriNamespace() {
		return uriNamespace;
	}
	
	public String getUriNamespacePrefix() {
		return uriNamespacePrefix;
	}	
	
	public String getUriLocalname() {
		return uriLocalname;
	}
	
	public String getUri() {
		return uriNamespace+uriLocalname;
	}

	public String getPrefLabel() {
		return prefLabel;
	}

    /**
	 * Get prefLabel escaped for XHTML presentation.
	 */
	
	public String getPrefLabelEscaped() {
		return Encoder.escapeXml(prefLabel);
	}
	
	/**
	 * 
	 * @return the matching altLabel, or null if the match was on the prefLabel
	 */
	
	public String getAltLabel() {
		return altLabel;
	}
	
    /**
	 * Get altLabel escaped for XHTML presentation.
	 */
	
	public String getAltLabelEscaped() {
		return Encoder.escapeXml(altLabel);
	}
	
	public String toString() {
		return "IndexResult(\"" + uriNamespace + uriLocalname + "\", \"" + prefLabel + "\", \"" + altLabel + "\")";
	}
}