package fi.helsinki.cs.seco.onki.logic;

import java.util.List;

/**
 * Represents a concept in a concept hierarchy.
 */

public class OnkiTreeResource {

	private OnkiResource ores;
	private OnkiResource oresparent;
	// treeindent contains information for printing the concept as part of a indented tree
	private List<Boolean> treeindent;	
	
	public OnkiTreeResource(OnkiResource ores, OnkiResource oresparent, List<Boolean> treeindent) {
		this.ores = ores;
		this.oresparent = oresparent;
		this.treeindent = treeindent;
	}
	
    /**
	 * Get the OnkiResource representation of the concept. 
	 */
	
	public OnkiResource getOnkiResource() {
		return this.ores;
	}
	
    /**
	 * Get the OnkiResource representation of the parent of the concept.
	 */
	
	public OnkiResource getOnkiResourceParent() {
		return this.oresparent;
	}
	
    /**
	 * Get the tree indent information, used to print the branches of the tree.
	 */
	
	public List<Boolean> getTreeindent() {
		return this.treeindent;
	}
	
    /**
	 * Get the indent size of the concept.
	 */
	
	public int getTreeindentLevel() {
		if (this.treeindent != null)
			return this.treeindent.size();
		return -1;
	}
	
    /**
	 * Get the tree indent information in a specific indent index.
	 */
	
	public boolean getIndentLevel(int i) {
		if (i >= 0 && this.treeindent != null && i < this.treeindent.size())
			return this.treeindent.get(i).booleanValue();
		return false;
	}
}