package fi.helsinki.cs.seco.onki.logic.propertyfiddlers;

import java.util.List;
import java.util.LinkedList;

import com.hp.hpl.jena.rdf.model.NodeIterator;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Seq;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResourceFactory;

import fi.helsinki.cs.seco.onki.logic.IPropertyFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.logic.OnkiStatement;

/**
 * Generate relations to from concepts to altLabels defined as lists in SKOS vocabularies.
 */

public class SkosAltLabelsAsListPropertyFiddler implements IPropertyFiddler {
	
	private static final String NS_SKOS = "http://www.w3.org/2008/05/skos#";
	private static final Property SKOS_altLabel = ResourceFactory.createProperty(NS_SKOS+"altLabel");

	// not needed currently
	public List<OnkiStatement> findBackward(OnkiResource ores) {
		return null;
	}

	/** 
	 * Find relations to nonpreferred terms.
	 */
	
	public List<OnkiStatement> findForward(OnkiResource ores, Statement stmt) {
		if (!ores.getResource().isResource())
			return null;
		if (!stmt.getPredicate().equals(SKOS_altLabel))
			return null;

		List<OnkiStatement> reslist = new LinkedList<OnkiStatement>();
		RDFNode object = stmt.getObject();
			try {
				//if (object.canAs(java.lang.Class.forName("com.hp.hpl.jena.rdf.model.Seq"))) {
				if (object.canAs((Class<Seq>)java.lang.Class.forName("com.hp.hpl.jena.rdf.model.Seq"))) {
					//NodeIterator iter = ((Seq)object.as(java.lang.Class.forName("com.hp.hpl.jena.rdf.model.Seq"))).iterator();
					NodeIterator iter = ((Seq)object.as((Class<Seq>)java.lang.Class.forName("com.hp.hpl.jena.rdf.model.Seq"))).iterator();
					while (iter.hasNext()) {
						RDFNode node = iter.nextNode();
						if (node.isLiteral())
							reslist.add(new OnkiStatement(ores, SKOS_altLabel, node, ores.getLangPreferences(), OnkiStatement.TYPE_INDIRECT));
					}
				}
				else
					return null;
			} catch (ClassNotFoundException e) {
				System.err.println("Exception catched in SkosAltLabelsAsListPropertyFiddler when instantiating class object of com.hp.hpl.jena.rdf.model.Seq: "+e);
			}
		return reslist;
	}
}