package fi.helsinki.cs.seco.onki.service;

import java.io.Serializable;
import java.text.Collator;
import java.util.Collection;
import java.util.Locale;

/**
 * Represents an element in the ontology directory.
 * An element may be either 1) alphabet, 2) group or 3) top concept,
 * and it consists of concepts.
 */

public class OnkiDirectoryElement implements Comparable<OnkiDirectoryElement>, Serializable {
	
	private static final long serialVersionUID = 1L;
	
	private String uri;
	private String label;
	private int size;
	private Collection<OnkiResourceSerializable> members;
	
	public OnkiDirectoryElement(String uri, String label, int size, Collection<OnkiResourceSerializable> members) {
		this.uri = uri;
		this.label = label;
		this.size = size;
		this.members = members;
	}
	
	public String getUri() {
		return uri;
	}
	
	public void setUri(String uri) {
		this.uri = uri;
	}
	
	public String getLabel() {
		return label;
	}
	
	public void setLabel(String label) {
		this.label = label;
	}
	
	public int getSize() {
		return size;
	}
	
	public void setSize(int size) {
		this.size = size;
	}
	
	public Collection<OnkiResourceSerializable> getMembers() {
		return members;
	}
	
	public void setMembers(Collection<OnkiResourceSerializable> members) {
		this.members = members;
	}
	
	public int compareTo(OnkiDirectoryElement arg) {
		if (arg == null)
			return -1;
		// to handle accents etc. (e.g. "ś" == "s")
    	Collator fiCollator = Collator.getInstance(new Locale("fi"));
    	fiCollator.setStrength(Collator.PRIMARY);
    	return fiCollator.compare(this.getLabel(), arg.getLabel());
	}
}