package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;  

import fi.helsinki.cs.seco.onki.service.OnkiHierarchyRelation;

/**
 * Represents the concept hierarchy of a concept.
 */

public class GetConceptHierarchyResource extends BaseResource {
	private String uri;
	private String lang;
	private int parents = -1;
	private int children = 1;
	private int siblings = 1;
	
	Collection<OnkiHierarchyRelation> tree;
	
	public GetConceptHierarchyResource(Context context, Request request, Response response) {  
		super(context, request, response);  
			   
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("u"))
				this.uri = param.getValue();
			else if (param.getName().equals("l"))
				this.lang = param.getValue();
			else if (param.getName().equals("p")) {
				if (param.getValue().equals("*"))
					this.parents = -1;
				else
					this.parents = new Integer(param.getValue());
			}
			else if (param.getName().equals("c")) {
				if (param.getValue().equals("*"))
					this.children = -1;
				else
					this.children = new Integer(param.getValue());
			}
			else if (param.getName().equals("s")) {
				if (param.getValue().equals("*"))
					this.siblings = -1;
				else
					this.siblings = new Integer(param.getValue());
			}
		}

		if (this.uri != null)
			this.tree = getConceptHierarchy(this.uri, this.lang, this.parents, this.children, this.siblings);
		   
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {  
		JSONArray array = new JSONArray();
		if (this.tree != null) {
			for (OnkiHierarchyRelation relation : this.tree) {
				JSONObject object = new JSONObject();
				try {
					object.put("conceptUri", relation.getConceptUri());
					object.put("label", relation.getLabel());
					object.put("parentUri", relation.getParentUri());
					object.put("indent", relation.getIndent());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}

		JsonRepresentation representation = new JsonRepresentation(array);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}