package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;
import java.util.Map;

import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;  

import fi.helsinki.cs.seco.onki.service.IBaseStatement;
import fi.helsinki.cs.seco.onki.service.OnkiHierarchyRelation;

import fi.helsinki.cs.seco.onki.logic.OnkiConceptMapping;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;

/**
 * Represents the full presentation of a concept.
 * Wraps together methods used to get relevant information about a concept
 * by the ONKI2 browser, reducing the amount of HTTP requests.
 */

public class GetFullPresentationResource extends BaseResource {
	String uri;
	String lang;
	String propertyLang;
	
	boolean hasGrandChildren;
	Collection<IBaseStatement> labels;
	Collection<IBaseStatement> altLabels;
	Collection<IBaseStatement> equivalentConcepts;
	Collection<IBaseStatement> properties;
	Collection<OnkiHierarchyRelation> parentTree;
	Collection<OnkiHierarchyRelation> childTree;
	Collection<OnkiHierarchyRelation> siblingTree;
	String polygon;
	Map <OnkiResource, Collection<OnkiConceptMapping>> mappings;
	
	private JSONObject obj;
	
	// OLD
	/*
	private static final int PARENTS = 0;
	private static final int CHILDREN = 1;
	private static final int SIBLINGS = 2;
	*/
	
	public GetFullPresentationResource(Context context, Request request, Response response) {  
		super(context, request, response);  
			   
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("u"))
				this.uri = param.getValue();
			else if (param.getName().equals("l"))
				this.lang = param.getValue();
			else if (param.getName().equals("pl"))
				this.propertyLang = param.getValue();
		}
		
		if (this.uri != null)
			this.obj = this.getFullPresentation(this.uri, this.lang, this.propertyLang);
		
		// OLD
		/*
		if (this.uri != null && this.containsResource(this.uri)) {
			// children & hasGrandChildren
			// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
			//this.childTree = getConceptHierarchy(this.uri, this.lang, 0, 1, 0);
			this.childTree = getConceptHierarchy(this.uri, this.propertyLang, 0, 1, 0);
			// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
			this.hasGrandChildren = childTree.size() != 
								//getConceptHierarchy(this.uri, this.lang, 0, 2, 0).size();
								getConceptHierarchy(this.uri, this.propertyLang, 0, 2, 0).size();
			//getLabels in all languages
			this.labels = getLabels(this.uri, null, this.propertyLang, true);
			//getAltLabels in all languages
			this.altLabels = getAltLabels(this.uri, null, this.propertyLang, true);
			//getEquivalentConcepts
			// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
			//this.equivalentConcepts = getEquivalentConcepts(this.uri, this.lang, true);
			this.equivalentConcepts = getEquivalentConcepts(this.uri, this.propertyLang, true);
			// properties sorted and filtered
			// TODO: getProperties should support 2 languages: 1 for property labels and 1 for object labels
			//this.properties = getProperties(this.uri, null, this.lang, true);
			this.properties = getProperties(this.uri, null, this.propertyLang, true);
			// parents
			// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
			//this.parentTree = getConceptHierarchy(this.uri, this.lang, getHierarchySize(PARENTS), 0, 0);
			this.parentTree = getConceptHierarchy(this.uri, this.propertyLang, getHierarchySize(PARENTS), 0, 0);
			// siblings
			// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
			//this.siblingTree = getConceptHierarchy(this.uri, this.lang, 0, 0, getHierarchySize(SIBLINGS));
			this.siblingTree = getConceptHierarchy(this.uri, this.propertyLang, 0, 0, getHierarchySize(SIBLINGS));
			if (getHierarchySize(CHILDREN) != 1)
				// TODO: temporarily use propertyLang instead of lang; when getProperties supports 2 languages, fix this
				//this.childTree = getConceptHierarchy(this.uri, this.lang, 0, getHierarchySize(CHILDREN), 0);
				this.childTree = getConceptHierarchy(this.uri, this.propertyLang, 0, getHierarchySize(CHILDREN), 0);
			this.polygon = getPolygon(this.uri);
			this.mappings = getConceptMappings(this.uri, this.propertyLang);
		}
		*/
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */
	@Override  
	public Representation represent(Variant variant) throws ResourceException {
		
		// OLD
		/*
		JSONObject obj = new JSONObject();
		
		JSONArray propertyArray = new JSONArray();
		if (this.properties != null) {
			try {
				JSONObject prop = new JSONObject();
				JSONArray objects = new JSONArray();
				for (IBaseStatement stmt : this.properties) {
					JSONObject object = new JSONObject();
					if (stmt instanceof ResourceStatement) {
						object.put("uri", ((ResourceStatement) stmt).getUri());
						object.put("label", ((ResourceStatement) stmt).getLabel());
					}
					else if (stmt instanceof LiteralStatement) {
						object.put("value", ((LiteralStatement) stmt).getValue());
						object.put("lang", ((LiteralStatement) stmt).getLang());
					}
					if (!prop.has("propertyUri")) { // first statement
						prop.put("propertyUri", stmt.getPredicateUri());
						prop.put("propertyLabel", stmt.getPredicateLabel());
					}
					if (prop.getString("propertyUri").equals(stmt.getPredicateUri())) { // same predicate as in previous statement of first statement
						objects.put(object);
					}
					else { // some other predicate
						prop.put("objects", objects);
						propertyArray.put(prop);
						prop = new JSONObject();
						prop.put("propertyUri", stmt.getPredicateUri());
						prop.put("propertyLabel", stmt.getPredicateLabel());
						objects = new JSONArray();
						objects.put(object);
					}
				}
				// last statement
				if (objects.length() > 0) {
					prop.put("objects", objects);
					propertyArray.put(prop);
				}
				
				obj.put("hasGrandChildren", this.hasGrandChildren);
				obj.put("getLabels", this.statementsIntoArray(this.labels));
				obj.put("getAltLabels", this.statementsIntoArray(this.altLabels));
				obj.put("getEquivalents", this.statementsIntoArray(this.equivalentConcepts));
				obj.put("getProperties", propertyArray);
				obj.put("getParents", this.treeIntoArray(this.parentTree));
				obj.put("getChildren", this.treeIntoArray(this.childTree));
				obj.put("getSiblings", this.treeIntoArray(this.siblingTree));
				
				if (this.polygon == null)
					obj.put("getPolygon", JSONObject.NULL);
				else
					obj.put("getPolygon", this.polygon);

				JSONArray mappings = new JSONArray();
				if (this.mappings != null) {
					for (Entry<OnkiResource, Collection<OnkiConceptMapping>> mapping : this.mappings.entrySet()) {
						JSONObject mappingObj = new JSONObject();
						mappingObj.put("propertyUri", mapping.getKey().getUri());
						mappingObj.put("propertyLabel", mapping.getKey().getLabel());
						JSONArray values = new JSONArray();
						for (OnkiConceptMapping value : mapping.getValue()) {
							JSONObject valueObj = new JSONObject();
							valueObj.put("targetUri", value.getTarget().getUri());
							valueObj.put("targetLabel", value.getTarget().getLabel());
							valueObj.put("overlaps", value.getOverlapsRounded());
							valueObj.put("overlappedBy", value.getOverlappedByRounded());
							values.put(valueObj);
						}
						mappingObj.put("targets", values);
						mappings.put(mappingObj);
					}
				}
				obj.put("getMappings", mappings);
			}
			catch (JSONException e) {
				e.printStackTrace();
			}
		}
		 */
		
		JsonRepresentation representation = new JsonRepresentation(obj);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
	
	// OLD
	/*
	 * Helper to make a JSON array from collection of statements.
	 */
	/*
	private JSONArray statementsIntoArray(Collection<IBaseStatement> stmts) {
		JSONArray array = new JSONArray();
		if (stmts != null) {
			for (IBaseStatement stmt : stmts) {
				JSONObject object = new JSONObject();
				try {
					if (stmt instanceof LiteralStatement) {
						object.put("label", ((LiteralStatement)stmt).getValue());
						object.put("lang", ((LiteralStatement)stmt).getLang());
					}
					else if (stmt instanceof ResourceStatement) {
						object.put("label", ((ResourceStatement)stmt).getLabel());
						object.put("uri", ((ResourceStatement)stmt).getUri());
					}
					object.put("propertyUri", stmt.getPredicateUri());
					object.put("propertyLabel", stmt.getPredicateLabel());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}
		return array;
	}
	*/
	
	// OLD
	/*
	 * Helper to make a JSON array from concept hierarchy tree.
	 */
	/*
	private JSONArray treeIntoArray(Collection<OnkiHierarchyRelation> tree) {
		JSONArray array = new JSONArray();
		if (tree != null) {
			for (OnkiHierarchyRelation relation : tree) {
				JSONObject object = new JSONObject();
				try {
					object.put("conceptUri", relation.getConceptUri());
					object.put("label", relation.getLabel());
					object.put("parentUri", relation.getParentUri());
					object.put("indent", relation.getIndent());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}
		return array;
	}
	*/
}