package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;  

import fi.helsinki.cs.seco.onki.service.IBaseStatement;

/**
 * Represents labels of a concept.
 */

public class GetLabelsResource extends BaseResource {
	String uri;
	String lang;
	String propertyLang;
	Collection<IBaseStatement> labels;
	
	public GetLabelsResource(Context context, Request request, Response response) {  
		super(context, request, response);  
			   
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("u"))
				this.uri = param.getValue();
			else if (param.getName().equals("l"))
				this.lang = param.getValue();
			else if (param.getName().equals("pl"))
				this.propertyLang = param.getValue();
		}
		 
		if (this.uri != null)
			this.labels = getLabels(this.uri, this.lang, this.propertyLang, false);
		   
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {  
		JSONArray array = new JSONArray();
		if (this.labels != null) {
			for (IBaseStatement stmt : this.labels) {
				JSONObject object = new JSONObject();
				try {
					object.put("label", ((LiteralStatement)stmt).getValue());
					object.put("lang", ((LiteralStatement)stmt).getLang());
					object.put("propertyUri", stmt.getPredicateUri());
					object.put("propertyLabel", stmt.getPredicateLabel());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}

		JsonRepresentation representation = new JsonRepresentation(array);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}