package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;
import java.util.HashMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;

/**
 * Represents metadata about an ontology.
 */

public class GetMetadataResource extends BaseResource {
	HashMap<String, String> metadata;
	String lang;
	
	public GetMetadataResource(Context context, Request request, Response response) {  
		super(context, request, response);  
		
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("l"))
				this.lang = param.getValue();
		}
		
		this.metadata = getMetadata();
		
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {  
		JSONObject object = new JSONObject();
		if (this.metadata != null) {
			for (String metadataKey : this.metadata.keySet()) {
				try {
					object.put(metadataKey, this.metadata.get(metadataKey));
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
			}
			//aakkoset, joilla alkavia käsitteitä ontologia sisältää
			JSONArray alphabets = new JSONArray();
			Collection<Character> alphabetsCollection = getAvailableAlphabets(lang);
			if (alphabetsCollection != null)
				for (Character alphabet : alphabetsCollection)
					alphabets.put(alphabet);
			try {
				object.put("conceptLabelStartingAlphabets", alphabets);
			}
			catch (JSONException e) {
				;
			}
		}

		JsonRepresentation representation = new JsonRepresentation(object);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}