package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;
import java.util.LinkedList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;  

import fi.helsinki.cs.seco.onki.service.IBaseStatement;

/**
 * Represents properties of a concept.
 */

public class GetPropertiesResource extends BaseResource {
	String uri;
	Collection<String> propertyUris;
	String lang;
	String propertyLang;
	Collection<IBaseStatement> properties;
	boolean filter = false;
	
	public GetPropertiesResource(Context context, Request request, Response response) {  
		super(context, request, response);  
			   
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("u"))
				this.uri = param.getValue();
			else if (param.getName().equals("p")) {
				if (this.propertyUris == null)
					this.propertyUris = new LinkedList<String>();
				this.propertyUris.add(param.getValue());
			}
			else if (param.getName().equals("l"))
				this.lang = param.getValue();
			else if (param.getName().equals("pl"))
				this.propertyLang = param.getValue();
			else if (param.getName().equals("f"))
				// TODO: handle errors
				this.filter = new Boolean(param.getValue());
		}
		 
		if (this.uri != null)
			this.properties = getProperties(this.uri, this.propertyUris, this.lang, this.propertyLang, filter);

		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {  
		JSONArray array = new JSONArray();
		if (this.properties != null) {
			try {
				JSONObject prop = new JSONObject();
				JSONArray objects = new JSONArray();
				for (IBaseStatement stmt : this.properties) {
					JSONObject object = new JSONObject();
					if (stmt instanceof ResourceStatement) {
						object.put("uri", ((ResourceStatement) stmt).getUri());
						object.put("label", ((ResourceStatement) stmt).getLabel());
					}
					else if (stmt instanceof LiteralStatement) {
						object.put("value", ((LiteralStatement) stmt).getValue());
						object.put("lang", ((LiteralStatement) stmt).getLang());
					}
					if (!prop.has("propertyUri")) { // first statement
						prop.put("propertyUri", stmt.getPredicateUri());
						prop.put("propertyLabel", stmt.getPredicateLabel());
					}
					// TODO: merging literal properties does not always work,
					//		 because the statements are sorted primarily by the langs
					// TODO: the merging should be generalized into e.g. RestApplication
					//       so that also other REST methods could use it 
					if (prop.getString("propertyUri").equals(stmt.getPredicateUri())) { // same predicate as in previous statement of first statement
						objects.put(object);
					}
					else { // some other predicate
						prop.put("objects", objects);
						array.put(prop);
						prop = new JSONObject();
						prop.put("propertyUri", stmt.getPredicateUri());
						prop.put("propertyLabel", stmt.getPredicateLabel());
						objects = new JSONArray();
						objects.put(object);
					}
				}
				// last statement
				if (objects.length() > 0) {
					prop.put("objects", objects);
					array.put(prop);
				}
			}
			catch (JSONException e) {
				e.printStackTrace();
			}	
		}

		JsonRepresentation representation = new JsonRepresentation(array);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}