package fi.helsinki.cs.seco.onki.servlet;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.velocity.Template;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.servlet.VelocityServlet;

/**
 * Servlet for handling the error pages.
 *
 * @author vpkomula
 */

public class OnkiErrorServlet extends VelocityServlet {

    private Log logger = LogFactory.getLog(OnkiErrorServlet.class);
    
    /**
     * Loads Velocity configuration on init.
     */
    
    @Override
	protected Properties loadConfiguration(ServletConfig config) throws IOException, FileNotFoundException {

        String path = config.getServletContext().getRealPath("/");
        if (path == null) path = "/";

        if (!path.endsWith("/") || !path.endsWith("\\")) path += "/";

        //	Configure Velocity
        Properties p = new Properties();
        p.setProperty(Velocity.FILE_RESOURCE_LOADER_PATH, path + "templates/");
        p.setProperty(Velocity.COUNTER_INITIAL_VALUE, "0");

        return p;
    }

    /**
     * Handles the request, generates the error page.
     */
    
    @Override
	public Template handleRequest(HttpServletRequest request, HttpServletResponse response, Context ctx) {

        Template templ = null;
        Integer statusCode = (Integer) request.getAttribute("javax.servlet.error.status_code");
        ctx.put("statusCode", statusCode);

        try {
            templ = getTemplate(OnkiServlet.ONKI_ERROR_TEMPLATE);
        } catch (ParseErrorException pee) {
            logger.error(getClass() + " : parse error for template : " + pee);
        } catch (ResourceNotFoundException rnfe) {
            logger.error(getClass() + " : template not found : " + rnfe);
        } catch (Exception e) {
            logger.error(getClass() + " : Error : " + e);
        }
        return templ;
    }
}