dojo.provide("semweb.graph.DomDrawer");

dojo.require("prototype.prototype");
dojo.require("MochiKit.MochiKit");
dojo.require("semweb.graphics");
dojo.require("semweb.graph.Graph");

semweb.graph.DomDrawer = Class.create();

semweb.graph.DomDrawer.prototype = {
  
  setOptions: function(options) {
    this.options = {
      radius: 20,
      arrowHeadLength: null,
      arrowAngle: Math.PI/10,
      mecd: 10, // minimum edge-click distance
      edgeColor: 'blue'
    }
    Object.extend(this.options, options || {});
  },

  initialize: function(element, graph, options) {
    this.element = element;
    this.graph = graph;
    this.lines = $H();
	this.setOptions(options);
  },
  
  rotate: function(point, length, angle) {
    var dx = length * Math.cos(angle);
    var dy = length * Math.sin(angle);
    return [point[0]+dx, point[1]+dy];
  },

  draw: function() {
//    this.lines.each(function (line) { line[1].hide(); });
//    this.lines = $H();
//    this.ctx.clearRect(0, 0, this.element.width, this.element.height);
    var nodes = this.graph.nodes();
    for (var i = 0; i < nodes.length; i++) {
      this.drawNode(nodes[i]);
    }
  },
  
  animate: function(iterator, count, timeout) {
    iterator.layout(10);
    this.draw();
    if (count>0) semweb.callLater(timeout,this.animate.bind(this),iterator,count-1,timeout);
  },
  
  drawNode: function(node) {
    node.radius = this.options.radius;
    // Draw the text (or if the value is an element, move it)
    if(typeof node.value == 'string') {
    } else {
      node.value.style.position = 'absolute';
      var tmp = node.value.getDimensions();
      if (tmp.width>tmp.height) node.radius = tmp.width;
      else node.radius=tmp.height;
      var pos = MochiKit.DOM.elementPosition(this.element);
      node.value.style.left     = pos.x + node.x - tmp.width/2 + 'px';
      node.value.style.top      = pos.y + node.y - tmp.height/2 + 'px';    
    }
    var nodes = node.getEdgesOut();
    for (var i = 0; i < nodes.length; i++) {
      this.drawEdge(node,nodes[i]);
    }
  },
  
  drawEdge: function(node1,node2) {
    var img = this.lines[""+node1.id+"->"+node2.id];
    if (img == undefined) {
      img = Element.extend(MochiKit.DOM.IMG());
      img.style.position= 'absolute';
      this.element.appendChild(img);
      this.lines[""+node1.id+"->"+node2.id]=img;
    }
    var pos = MochiKit.DOM.elementPosition(this.element);
    semweb.graphics.drawLine(img,pos.x+node1.x,pos.y+node1.y,pos.x+node2.x,pos.y+node2.y,this.options.lineimgpath);
//    this.drawArrowHead(theta, source[0], source[1], target[0], target[1]);
  },

  drawArrowHead: function(theta, startx, starty, endx, endy) {
	var length = this.options.arrowHeadLength ? this.options.arrowHeadLength : this.options.radius;
    var top = this.rotate([endx, endy], length, theta + this.options.arrowAngle);
    var bottom = this.rotate([endx, endy], length, theta - this.options.arrowAngle);
    this.ctx.beginPath();
    this.ctx.moveTo(endx, endy);
    this.ctx.lineTo(top[0], top[1]);
    this.ctx.lineTo(bottom[0], bottom[1]);
    this.ctx.fill();
  },

  elementAt: function(point) {
	var node = undefined;
	var edge = undefined;
	var minnd = Infinity;
	var mined = Infinity;
	var nodes = this.graph.nodes();
	for (var i = 0; i < nodes.length; i++) {
      var n = nodes[i];
      var dx = point[0] - n.x;
      var dy = point[1] - n.y;
      var d = Math.sqrt(dx * dx + dy * dy);
      if(d < minnd && d <= n.radius) {
        minnd = d;
        node = n;
      }
      for (var j = 0; j< nodes.length; j++) {
        var n2 = nodes[j];
        var x0 = point[0];
        var y0 = point[1];

        var num = Math.abs((n2.x-n.x)*(n1.y-y0)-(n.x-x0)*(n2.y-n1.y));
        var den = Math.sqrt((n2.x-n.x)*(n2.x-n.x)+(n2.y-n1.y)*(n2.y-n1.y));
        var d = num/den;
        if(d < mined && d <= this.options.mecd) {
          mined = d;
          edge = [n1, n2];
        }
      }
    }
    return [node, edge];
  }
};
