package fi.seco.semweb.util.sql;

import java.sql.DatabaseMetaData;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class NonExceptingMetaData implements DatabaseMetaData {

	private DatabaseMetaData metadata;
	private static final Log log = LogFactory.getLog(NonExceptingMetaData.class);

	public NonExceptingMetaData(DatabaseMetaData metadata) {
		this.metadata=metadata;
	}

	public boolean allProceduresAreCallable() {
		try {
			return metadata.allProceduresAreCallable();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean allTablesAreSelectable() {
		try {
			return metadata.allTablesAreSelectable();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean dataDefinitionCausesTransactionCommit() {
		try {
			return metadata.dataDefinitionCausesTransactionCommit();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean dataDefinitionIgnoredInTransactions() {
		try {
			return metadata.dataDefinitionIgnoredInTransactions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean deletesAreDetected(int type) {
		try {
			return metadata.deletesAreDetected(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean doesMaxRowSizeIncludeBlobs() {
		try {
			return metadata.doesMaxRowSizeIncludeBlobs();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public NonExceptingResultSet getAttributes(String catalog, String schemaPattern, String typeNamePattern, String attributeNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getAttributes(catalog, schemaPattern, typeNamePattern, attributeNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getBestRowIdentifier(String catalog, String schema, String table, int scope, boolean nullable) {
		try {
			return new NonExceptingResultSet(metadata.getBestRowIdentifier(catalog, schema, table, scope, nullable));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getCatalogs() {
		try {
			return new NonExceptingResultSet(metadata.getCatalogs());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getCatalogSeparator() {
		try {
			return metadata.getCatalogSeparator();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getCatalogTerm() {
		try {
			return metadata.getCatalogTerm();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getColumnPrivileges(String catalog, String schema, String table, String columnNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getColumnPrivileges(catalog, schema, table, columnNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getColumns(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getColumns(catalog, schemaPattern, tableNamePattern, columnNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingConnection getConnection() {
		try {
			return new NonExceptingConnection(metadata.getConnection());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getCrossReference(String primaryCatalog, String primarySchema, String primaryTable, String foreignCatalog, String foreignSchema, String foreignTable) {
		try {
			return new NonExceptingResultSet(metadata.getCrossReference(primaryCatalog, primarySchema, primaryTable, foreignCatalog, foreignSchema, foreignTable));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getDatabaseMajorVersion() {
		try {
			return metadata.getDatabaseMajorVersion();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getDatabaseMinorVersion() {
		try {
			return metadata.getDatabaseMinorVersion();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public String getDatabaseProductName() {
		try {
			return metadata.getDatabaseProductName();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getDatabaseProductVersion() {
		try {
			return metadata.getDatabaseProductVersion();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getDefaultTransactionIsolation() {
		try {
			return metadata.getDefaultTransactionIsolation();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getDriverMajorVersion() {
		return metadata.getDriverMajorVersion();
	}

	public int getDriverMinorVersion() {
		return metadata.getDriverMinorVersion();
	}

	public String getDriverName() {
		try {
			return metadata.getDriverName();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getDriverVersion() {
		try {
			return metadata.getDriverVersion();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getExportedKeys(String catalog, String schema, String table) {
		try {
			return new NonExceptingResultSet(metadata.getExportedKeys(catalog, schema, table));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getExtraNameCharacters() {
		try {
			return metadata.getExtraNameCharacters();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getIdentifierQuoteString() {
		try {
			return metadata.getIdentifierQuoteString();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getImportedKeys(String catalog, String schema, String table) {
		try {
			return new NonExceptingResultSet(metadata.getImportedKeys(catalog, schema, table));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getIndexInfo(String catalog, String schema, String table, boolean unique, boolean approximate) {
		try {
			return new NonExceptingResultSet(metadata.getIndexInfo(catalog, schema, table, unique, approximate));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getJDBCMajorVersion() {
		try {
			return metadata.getJDBCMajorVersion();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getJDBCMinorVersion() {
		try {
			return metadata.getJDBCMinorVersion();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxBinaryLiteralLength() {
		try {
			return metadata.getMaxBinaryLiteralLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxCatalogNameLength() {
		try {
			return metadata.getMaxCatalogNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxCharLiteralLength() {
		try {
			return metadata.getMaxCharLiteralLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnNameLength() {
		try {
			return metadata.getMaxColumnNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnsInGroupBy() {
		try {
			return metadata.getMaxColumnsInGroupBy();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnsInIndex() {
		try {
			return metadata.getMaxColumnsInIndex();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnsInOrderBy() {
		try {
			return metadata.getMaxColumnsInOrderBy();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnsInSelect() {
		try {
			return metadata.getMaxColumnsInSelect();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxColumnsInTable() {
		try {
			return metadata.getMaxColumnsInTable();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxConnections() {
		try {
			return metadata.getMaxConnections();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxCursorNameLength() {
		try {
			return metadata.getMaxCursorNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxIndexLength() {
		try {
			return metadata.getMaxIndexLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxProcedureNameLength() {
		try {
			return metadata.getMaxProcedureNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxRowSize() {
		try {
			return metadata.getMaxRowSize();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxSchemaNameLength() {
		try {
			return metadata.getMaxSchemaNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxStatementLength() {
		try {
			return metadata.getMaxStatementLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxStatements() {
		try {
			return metadata.getMaxStatements();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxTableNameLength() {
		try {
			return metadata.getMaxTableNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxTablesInSelect() {
		try {
			return metadata.getMaxTablesInSelect();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxUserNameLength() {
		try {
			return metadata.getMaxUserNameLength();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public String getNumericFunctions() {
		try {
			return metadata.getNumericFunctions();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getPrimaryKeys(String catalog, String schema, String table) {
		try {
			return new NonExceptingResultSet(metadata.getPrimaryKeys(catalog, schema, table));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getProcedureColumns(String catalog, String schemaPattern, String procedureNamePattern, String columnNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getProcedureColumns(catalog, schemaPattern, procedureNamePattern, columnNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getProcedures(String catalog, String schemaPattern, String procedureNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getProcedures(catalog, schemaPattern, procedureNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getProcedureTerm() {
		try {
			return metadata.getProcedureTerm();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getResultSetHoldability() {
		try {
			return metadata.getResultSetHoldability();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public NonExceptingResultSet getSchemas() {
		try {
			return new NonExceptingResultSet(metadata.getSchemas());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getSchemaTerm() {
		try {
			return metadata.getSchemaTerm();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getSearchStringEscape() {
		try {
			return metadata.getSearchStringEscape();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getSQLKeywords() {
		try {
			return metadata.getSQLKeywords();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getSQLStateType() {
		try {
			return metadata.getSQLStateType();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public String getStringFunctions() {
		try {
			return metadata.getStringFunctions();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getSuperTables(String catalog, String schemaPattern, String tableNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getSuperTables(catalog, schemaPattern, tableNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getSuperTypes(String catalog, String schemaPattern, String typeNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getSuperTypes(catalog, schemaPattern, typeNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getSystemFunctions() {
		try {
			return metadata.getSystemFunctions();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getTablePrivileges(String catalog, String schemaPattern, String tableNamePattern) {
		try {
			return new NonExceptingResultSet(metadata.getTablePrivileges(catalog, schemaPattern, tableNamePattern));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getTables(String catalog, String schemaPattern, String tableNamePattern, String[] types) {
		try {
			return new NonExceptingResultSet(metadata.getTables(catalog, schemaPattern, tableNamePattern, types));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getTableTypes() {
		try {
			return new NonExceptingResultSet(metadata.getTableTypes());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getTimeDateFunctions() {
		try {
			return metadata.getTimeDateFunctions();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getTypeInfo() {
		try {
			return new NonExceptingResultSet(metadata.getTypeInfo());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getUDTs(String catalog, String schemaPattern, String typeNamePattern, int[] types) {
		try {
			return new NonExceptingResultSet(metadata.getUDTs(catalog, schemaPattern, typeNamePattern, types));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getURL() {
		try {
			return metadata.getURL();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getUserName() {
		try {
			return metadata.getUserName();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet getVersionColumns(String catalog, String schema, String table) {
		try {
			return new NonExceptingResultSet(metadata.getVersionColumns(catalog, schema, table));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public boolean insertsAreDetected(int type) {
		try {
			return metadata.insertsAreDetected(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean isCatalogAtStart() {
		try {
			return metadata.isCatalogAtStart();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean isReadOnly() {
		try {
			return metadata.isReadOnly();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean locatorsUpdateCopy() {
		try {
			return metadata.locatorsUpdateCopy();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean nullPlusNonNullIsNull() {
		try {
			return metadata.nullPlusNonNullIsNull();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean nullsAreSortedAtEnd() {
		try {
			return metadata.nullsAreSortedAtEnd();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean nullsAreSortedAtStart() {
		try {
			return metadata.nullsAreSortedAtStart();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean nullsAreSortedHigh() {
		try {
			return metadata.nullsAreSortedHigh();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean nullsAreSortedLow() {
		try {
			return metadata.nullsAreSortedLow();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean othersDeletesAreVisible(int type) {
		try {
			return metadata.othersDeletesAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean othersInsertsAreVisible(int type) {
		try {
			return metadata.othersInsertsAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean othersUpdatesAreVisible(int type) {
		try {
			return metadata.othersUpdatesAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean ownDeletesAreVisible(int type) {
		try {
			return metadata.ownDeletesAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean ownInsertsAreVisible(int type) {
		try {
			return metadata.ownInsertsAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean ownUpdatesAreVisible(int type) {
		try {
			return metadata.ownUpdatesAreVisible(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesLowerCaseIdentifiers() {
		try {
			return metadata.storesLowerCaseIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesLowerCaseQuotedIdentifiers() {
		try {
			return metadata.storesLowerCaseQuotedIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesMixedCaseIdentifiers() {
		try {
			return metadata.storesMixedCaseIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesMixedCaseQuotedIdentifiers() {
		try {
			return metadata.storesMixedCaseQuotedIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesUpperCaseIdentifiers() {
		try {
			return metadata.storesUpperCaseIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean storesUpperCaseQuotedIdentifiers() {
		try {
			return metadata.storesUpperCaseQuotedIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsAlterTableWithAddColumn() {
		try {
			return metadata.supportsAlterTableWithAddColumn();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsAlterTableWithDropColumn() {
		try {
			return metadata.supportsAlterTableWithDropColumn();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsANSI92EntryLevelSQL() {
		try {
			return metadata.supportsANSI92EntryLevelSQL();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsANSI92FullSQL() {
		try {
			return metadata.supportsANSI92FullSQL();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsANSI92IntermediateSQL() {
		try {
			return metadata.supportsANSI92IntermediateSQL();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsBatchUpdates() {
		try {
			return metadata.supportsBatchUpdates();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCatalogsInDataManipulation() {
		try {
			return metadata.supportsCatalogsInDataManipulation();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCatalogsInIndexDefinitions() {
		try {
			return metadata.supportsCatalogsInIndexDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCatalogsInPrivilegeDefinitions() {
		try {
			return metadata.supportsCatalogsInPrivilegeDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCatalogsInProcedureCalls() {
		try {
			return metadata.supportsCatalogsInProcedureCalls();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCatalogsInTableDefinitions() {
		try {
			return metadata.supportsCatalogsInTableDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsColumnAliasing() {
		try {
			return metadata.supportsColumnAliasing();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsConvert() {
		try {
			return metadata.supportsConvert();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsConvert(int fromType, int toType) {
		try {
			return metadata.supportsConvert(fromType, toType);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCoreSQLGrammar() {
		try {
			return metadata.supportsCoreSQLGrammar();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsCorrelatedSubqueries() {
		try {
			return metadata.supportsCorrelatedSubqueries();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsDataDefinitionAndDataManipulationTransactions() {
		try {
			return metadata.supportsDataDefinitionAndDataManipulationTransactions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsDataManipulationTransactionsOnly() {
		try {
			return metadata.supportsDataManipulationTransactionsOnly();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsDifferentTableCorrelationNames() {
		try {
			return metadata.supportsDifferentTableCorrelationNames();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsExpressionsInOrderBy() {
		try {
			return metadata.supportsExpressionsInOrderBy();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsExtendedSQLGrammar() {
		try {
			return metadata.supportsExtendedSQLGrammar();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsFullOuterJoins() {
		try {
			return metadata.supportsFullOuterJoins();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsGetGeneratedKeys() {
		try {
			return metadata.supportsGetGeneratedKeys();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsGroupBy() {
		try {
			return metadata.supportsGroupBy();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsGroupByBeyondSelect() {
		try {
			return metadata.supportsGroupByBeyondSelect();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsGroupByUnrelated() {
		try {
			return metadata.supportsGroupByUnrelated();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsIntegrityEnhancementFacility() {
		try {
			return metadata.supportsIntegrityEnhancementFacility();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsLikeEscapeClause() {
		try {
			return metadata.supportsLikeEscapeClause();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsLimitedOuterJoins() {
		try {
			return metadata.supportsLimitedOuterJoins();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMinimumSQLGrammar() {
		try {
			return metadata.supportsMinimumSQLGrammar();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMixedCaseIdentifiers() {
		try {
			return metadata.supportsMixedCaseIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMixedCaseQuotedIdentifiers() {
		try {
			return metadata.supportsMixedCaseQuotedIdentifiers();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMultipleOpenResults() {
		try {
			return metadata.supportsMultipleOpenResults();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMultipleResultSets() {
		try {
			return metadata.supportsMultipleResultSets();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsMultipleTransactions() {
		try {
			return metadata.supportsMultipleTransactions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsNamedParameters() {
		try {
			return metadata.supportsNamedParameters();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsNonNullableColumns() {
		try {
			return metadata.supportsNonNullableColumns();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOpenCursorsAcrossCommit() {
		try {
			return metadata.supportsOpenCursorsAcrossCommit();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOpenCursorsAcrossRollback() {
		try {
			return metadata.supportsOpenCursorsAcrossRollback();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOpenStatementsAcrossCommit() {
		try {
			return metadata.supportsOpenStatementsAcrossCommit();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOpenStatementsAcrossRollback() {
		try {
			return metadata.supportsOpenStatementsAcrossRollback();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOrderByUnrelated() {
		try {
			return metadata.supportsOrderByUnrelated();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsOuterJoins() {
		try {
			return metadata.supportsOuterJoins();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsPositionedDelete() {
		try {
			return metadata.supportsPositionedDelete();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsPositionedUpdate() {
		try {
			return metadata.supportsPositionedUpdate();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsResultSetConcurrency(int type, int concurrency) {
		try {
			return metadata.supportsResultSetConcurrency(type, concurrency);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsResultSetHoldability(int holdability) {
		try {
			return metadata.supportsResultSetHoldability(holdability);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsResultSetType(int type) {
		try {
			return metadata.supportsResultSetType(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSavepoints() {
		try {
			return metadata.supportsSavepoints();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSchemasInDataManipulation() {
		try {
			return metadata.supportsSchemasInDataManipulation();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSchemasInIndexDefinitions() {
		try {
			return metadata.supportsSchemasInIndexDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSchemasInPrivilegeDefinitions() {
		try {
			return metadata.supportsSchemasInPrivilegeDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSchemasInProcedureCalls() {
		try {
			return metadata.supportsSchemasInProcedureCalls();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSchemasInTableDefinitions() {
		try {
			return metadata.supportsSchemasInTableDefinitions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSelectForUpdate() {
		try {
			return metadata.supportsSelectForUpdate();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsStatementPooling() {
		try {
			return metadata.supportsStatementPooling();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsStoredProcedures() {
		try {
			return metadata.supportsStoredProcedures();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSubqueriesInComparisons() {
		try {
			return metadata.supportsSubqueriesInComparisons();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSubqueriesInExists() {
		try {
			return metadata.supportsSubqueriesInExists();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSubqueriesInIns() {
		try {
			return metadata.supportsSubqueriesInIns();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsSubqueriesInQuantifieds() {
		try {
			return metadata.supportsSubqueriesInQuantifieds();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsTableCorrelationNames() {
		try {
			return metadata.supportsTableCorrelationNames();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsTransactionIsolationLevel(int level) {
		try {
			return metadata.supportsTransactionIsolationLevel(level);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsTransactions() {
		try {
			return metadata.supportsTransactions();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsUnion() {
		try {
			return metadata.supportsUnion();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean supportsUnionAll() {
		try {
			return metadata.supportsUnionAll();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean updatesAreDetected(int type) {
		try {
			return metadata.updatesAreDetected(type);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean usesLocalFilePerTable() {
		try {
			return metadata.usesLocalFilePerTable();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean usesLocalFiles() {
		try {
			return metadata.usesLocalFiles();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

}
