package fi.seco.semweb.util.sql;

import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class NonExceptingStatement implements Statement {

	protected Statement stmt;
	private static Log log = LogFactory.getLog(NonExceptingStatement.class);

	public NonExceptingStatement(Statement stmt) {
		this.stmt=stmt;
	}

	public void addBatch(String sql) {
		try {
			stmt.addBatch(sql);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void cancel() {
		try {
			stmt.cancel();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void clearBatch() {
		try {
			stmt.clearBatch();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void clearWarnings() {
		try {
			stmt.clearWarnings();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void close() {
		try {
			stmt.close();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean execute(String sql, int autoGeneratedKeys) {
		try {
			return stmt.execute(sql, autoGeneratedKeys);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean execute(String sql, int[] columnIndexes) {
		try {
			return stmt.execute(sql, columnIndexes);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean execute(String sql, String[] columnNames) {
		try {
			return stmt.execute(sql, columnNames);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean execute(String sql) {
		try {
			return stmt.execute(sql);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public int[] executeBatch() {
		try {
			return stmt.executeBatch();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingResultSet executeQuery(String sql) {
		try {
			return new NonExceptingResultSet(stmt.executeQuery(sql));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int executeUpdate(String sql, int autoGeneratedKeys) {
		try {
			return stmt.executeUpdate(sql, autoGeneratedKeys);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int executeUpdate(String sql, int[] columnIndexes) {
		try {
			return stmt.executeUpdate(sql, columnIndexes);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int executeUpdate(String sql, String[] columnNames) {
		try {
			return stmt.executeUpdate(sql, columnNames);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int executeUpdate(String sql) {
		try {
			return stmt.executeUpdate(sql);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public NonExceptingConnection getConnection() {
		try {
			return new NonExceptingConnection(stmt.getConnection());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getFetchDirection() {
		try {
			return stmt.getFetchDirection();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getFetchSize() {
		try {
			return stmt.getFetchSize();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public NonExceptingResultSet getGeneratedKeys() {
		try {
			return new NonExceptingResultSet(stmt.getGeneratedKeys());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getMaxFieldSize() {
		try {
			return stmt.getMaxFieldSize();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getMaxRows() {
		try {
			return stmt.getMaxRows();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public boolean getMoreResults() {
		try {
			return stmt.getMoreResults();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean getMoreResults(int current) {
		try {
			return stmt.getMoreResults(current);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public int getQueryTimeout() {
		try {
			return stmt.getQueryTimeout();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public NonExceptingResultSet getResultSet() {
		try {
			return new NonExceptingResultSet(stmt.getResultSet());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getResultSetConcurrency() {
		try {
			return stmt.getResultSetConcurrency();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getResultSetHoldability() {
		try {
			return stmt.getResultSetHoldability();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getResultSetType() {
		try {
			return stmt.getResultSetType();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getUpdateCount() {
		try {
			return stmt.getUpdateCount();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public SQLWarning getWarnings() {
		try {
			return stmt.getWarnings();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void setCursorName(String name) {
		try {
			stmt.setCursorName(name);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setEscapeProcessing(boolean enable) {
		try {
			stmt.setEscapeProcessing(enable);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setFetchDirection(int direction) {
		try {
			stmt.setFetchDirection(direction);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setFetchSize(int rows) {
		try {
			stmt.setFetchSize(rows);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setMaxFieldSize(int max) {
		try {
			stmt.setMaxFieldSize(max);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setMaxRows(int max) {
		try {
			stmt.setMaxRows(max);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setQueryTimeout(int seconds) {
		try {
			stmt.setQueryTimeout(seconds);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

}
