package fi.seco.semweb.util.sql;

import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.NClob;
import java.sql.SQLClientInfoException;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Struct;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class NonExceptingConnection implements Connection {

	private Connection conn;
	private static final Log log = LogFactory.getLog(NonExceptingConnection.class);

	public NonExceptingConnection() {} // Needed for HiveMind

	public NonExceptingConnection(Connection conn) {
		this.conn=conn;
	}

	public void clearWarnings() {
		try {
			conn.clearWarnings();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void close() {
		try {
			conn.close();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void commit() {
		try {
			conn.commit();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public NonExceptingStatement createStatement() {
		try {
			return new NonExceptingStatement(conn.createStatement());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingStatement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) {
		try {
			return new NonExceptingStatement(conn.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingStatement createStatement(int resultSetType, int resultSetConcurrency) {
		try {
			return new NonExceptingStatement(conn.createStatement(resultSetType, resultSetConcurrency));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public boolean getAutoCommit() {
		try {
			return conn.getAutoCommit();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public String getCatalog() {
		try {
			return conn.getCatalog();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getHoldability() {
		try {
			return conn.getHoldability();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public NonExceptingMetaData getMetaData() {
		try {
			return new NonExceptingMetaData(conn.getMetaData());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getTransactionIsolation() {
		try {
			return conn.getTransactionIsolation();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	@SuppressWarnings("unchecked")
	public Map getTypeMap() {
		try {
			return conn.getTypeMap();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public SQLWarning getWarnings() {
		try {
			return conn.getWarnings();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public boolean isClosed() {
		try {
			return conn.isClosed();
		} catch (SQLException e) {
			log.error("",e);
			return true;
		}
	}

	public boolean isReadOnly() {
		try {
			return conn.isReadOnly();
		} catch (SQLException e) {
			log.error("",e);
			return true;
		}
	}

	public String nativeSQL(String sql) {
		try {
			return conn.nativeSQL(sql);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) {
		try {
			return conn.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) {
		try {
			return conn.prepareCall(sql, resultSetType, resultSetConcurrency);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public CallableStatement prepareCall(String sql) {
		try {
			return conn.prepareCall(sql);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql, resultSetType, resultSetConcurrency));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql, int autoGeneratedKeys) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql, autoGeneratedKeys));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql, int[] columnIndexes) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql, columnIndexes));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql, String[] columnNames) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql, columnNames));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NonExceptingPreparedStatement prepareStatement(String sql) {
		try {
			return new NonExceptingPreparedStatement(conn.prepareStatement(sql));
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void releaseSavepoint(Savepoint savepoint) {
		try {
			conn.releaseSavepoint(savepoint);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void rollback() {
		try {
			conn.rollback();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void rollback(Savepoint savepoint) {
		try {
			conn.rollback(savepoint);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setAutoCommit(boolean autoCommit) {
		try {
			conn.setAutoCommit(autoCommit);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setCatalog(String catalog) {
		try {
			conn.setCatalog(catalog);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setHoldability(int holdability) {
		try {
			conn.setHoldability(holdability);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setReadOnly(boolean readOnly) {
		try {
			conn.setReadOnly(readOnly);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public Savepoint setSavepoint() {
		try {
			return conn.setSavepoint();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Savepoint setSavepoint(String name) {
		try {
			return conn.setSavepoint(name);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void setTransactionIsolation(int level) {
		try {
			conn.setTransactionIsolation(level);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	@SuppressWarnings("unchecked")
	public void setTypeMap(Map arg0) {
		try {
			conn.setTypeMap(arg0);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	@Override
	public String toString() {
		return conn.toString();
	}

	public Connection getOriginalConnection() {
		return conn;
	}

	public Array createArrayOf(String arg0, Object[] arg1) {
		try {
			return conn.createArrayOf(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Blob createBlob() {
		try {
			return conn.createBlob();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Clob createClob() {
		try {
			return conn.createClob();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public NClob createNClob() {
		try {
			return conn.createNClob();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public SQLXML createSQLXML() {
		try {
			return conn.createSQLXML();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Struct createStruct(String arg0, Object[] arg1) {
		try {
			return conn.createStruct(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Properties getClientInfo() {
		try {
			return conn.getClientInfo();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getClientInfo(String arg0) {
		try {
			return conn.getClientInfo(arg0);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public boolean isValid(int arg0) {
		try {
			return conn.isValid(arg0);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
}

	@SuppressWarnings("unchecked")
	public boolean isWrapperFor(Class arg0) {
		try {
			return conn.isWrapperFor(arg0);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void setClientInfo(Properties arg0) {
		try {
			conn.setClientInfo(arg0);
		} catch (SQLClientInfoException e) {
			log.error("",e);
		}
	}

	public void setClientInfo(String arg0, String arg1) {
		try {
			conn.setClientInfo(arg0, arg1);
		} catch (SQLClientInfoException e) {
			log.error("",e);
		}
	}

	public <T> T unwrap(Class<T> arg0) {
		try {
			return conn.unwrap(arg0);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}

	}
}
