package fi.seco.semweb.util.sql;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class NonExceptingPreparedStatement extends NonExceptingStatement implements PreparedStatement {
	
	private static final Log log = LogFactory.getLog(NonExceptingPreparedStatement.class);

	public NonExceptingPreparedStatement(PreparedStatement stmt) {
		super(stmt);
	}

	public NonExceptingResultSet executeQuery() {
		try {
			return new NonExceptingResultSet(((PreparedStatement)stmt).executeQuery());
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int executeUpdate() {
		try {
			return ((PreparedStatement)stmt).executeUpdate();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public void setNull(int parameterIndex, int sqlType) {
		try {
			((PreparedStatement)stmt).setNull(parameterIndex, sqlType);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBoolean(int parameterIndex, boolean x) {
		try {
			((PreparedStatement)stmt).setBoolean(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setByte(int parameterIndex, byte x) {
		try {
			((PreparedStatement)stmt).setByte(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setShort(int parameterIndex, short x) {
		try {
			((PreparedStatement)stmt).setShort(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setInt(int parameterIndex, int x) {
		try {
			((PreparedStatement)stmt).setInt(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setLong(int parameterIndex, long x) {
		try {
			((PreparedStatement)stmt).setLong(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setFloat(int parameterIndex, float x) {
		try {
			((PreparedStatement)stmt).setFloat(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setDouble(int parameterIndex, double x) {
		try {
			((PreparedStatement)stmt).setDouble(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBigDecimal(int parameterIndex, BigDecimal x) {
		try {
			((PreparedStatement)stmt).setBigDecimal(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setString(int parameterIndex, String x) {
		try {
			((PreparedStatement)stmt).setString(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBytes(int parameterIndex, byte[] x) {
		try {
			((PreparedStatement)stmt).setBytes( parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setDate(int parameterIndex, Date x) {
		try {
			((PreparedStatement)stmt).setDate( parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setTime(int parameterIndex, Time x) {
		try {
			((PreparedStatement)stmt).setTime( parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setTimestamp(int parameterIndex, Timestamp x) {
		try {
			((PreparedStatement)stmt).setTimestamp( parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setAsciiStream(int parameterIndex, InputStream x, int length) {
		try {
			((PreparedStatement)stmt).setAsciiStream(parameterIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	@Deprecated
	public void setUnicodeStream(int parameterIndex, InputStream x, int length) {
		try {
			((PreparedStatement)stmt).setUnicodeStream(parameterIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBinaryStream(int parameterIndex, InputStream x, int length) {
		try {
			((PreparedStatement)stmt).setBinaryStream(parameterIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void clearParameters() {
		try {
			((PreparedStatement)stmt).clearParameters();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) {
		try {
			((PreparedStatement)stmt).setObject(parameterIndex, x, targetSqlType, scale);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setObject(int parameterIndex, Object x, int targetSqlType) {
		try {
			((PreparedStatement)stmt).setObject(parameterIndex, x, targetSqlType);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setObject(int parameterIndex, Object x) {
		try {
			((PreparedStatement)stmt).setObject(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean execute() {
		try {
			return ((PreparedStatement)stmt).execute();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void addBatch() {
		try {
			((PreparedStatement)stmt).addBatch();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setCharacterStream(int parameterIndex, Reader reader, int length) {
		try {
			((PreparedStatement)stmt).setCharacterStream(parameterIndex, reader, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setRef(int i, Ref x) {
		try {
			((PreparedStatement)stmt).setRef(i, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBlob(int i, Blob x) {
		try {
			((PreparedStatement)stmt).setBlob(i, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setClob(int i, Clob x) {
		try {
			((PreparedStatement)stmt).setClob(i, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setArray(int i, Array x) {
		try {
			((PreparedStatement)stmt).setArray(i, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public ResultSetMetaData getMetaData() {
		try {
			return ((PreparedStatement)stmt).getMetaData();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void setDate(int parameterIndex, Date x, Calendar cal) {
		try {
			((PreparedStatement)stmt).setDate(parameterIndex, x, cal);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setTime(int parameterIndex, Time x, Calendar cal) {
		try {
			((PreparedStatement)stmt).setTime(parameterIndex, x, cal);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) {
		try {
			((PreparedStatement)stmt).setTimestamp(parameterIndex, x, cal);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNull(int paramIndex, int sqlType, String typeName) {
		try {
			((PreparedStatement)stmt).setNull(paramIndex, sqlType, typeName);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setURL(int parameterIndex, URL x) {
		try {
			((PreparedStatement)stmt).setURL(parameterIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public ParameterMetaData getParameterMetaData() {
		try {
			return ((PreparedStatement)stmt).getParameterMetaData();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void setAsciiStream(int arg0, InputStream arg1, long arg2) {
		try {
			((PreparedStatement) stmt).setAsciiStream(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setAsciiStream(int arg0, InputStream arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setAsciiStream(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBinaryStream(int arg0, InputStream arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setBinaryStream(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBinaryStream(int arg0, InputStream arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setBinaryStream(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBlob(int arg0, InputStream arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setBlob(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setBlob(int arg0, InputStream arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setBlob(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setCharacterStream(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setCharacterStream(int arg0, Reader arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setCharacterStream(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setClob(int arg0, Reader arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setClob(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setClob(int arg0, Reader arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setClob(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setNCharacterStream(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNCharacterStream(int arg0, Reader arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setNCharacterStream(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNClob(int arg0, NClob arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setNClob(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNClob(int arg0, Reader arg1, long arg2) throws SQLException {
		try {
			((PreparedStatement) stmt).setNClob(arg0, arg1, arg2);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNClob(int arg0, Reader arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setNClob(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setNString(int arg0, String arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setNString(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setRowId(int arg0, RowId arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setRowId(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setSQLXML(int arg0, SQLXML arg1) throws SQLException {
		try {
			((PreparedStatement) stmt).setSQLXML(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

}
