package fi.seco.semweb.service.indexing.tree;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.search.HitCollector;
import org.apache.lucene.search.Query;

import fi.seco.semweb.service.indexing.tree.model.ISearchableInputNode;
import fi.seco.semweb.service.indexing.tree.model.IndexedNode;
import fi.seco.semweb.util.lucene.LuceneDirectory;
import fi.seco.semweb.util.sql.NonExceptingConnection;

public class SearchableTreeIndexService<T> extends TreeIndexService<T> {

	@SuppressWarnings("unused")
	private static Log log = LogFactory.getLog(SearchableTreeIndexService.class);

	private LuceneDirectory ld;

	public SearchableTreeIndexService(String lucenePath, String tablePrefix, NonExceptingConnection sqlconn) {
		super(tablePrefix,sqlconn);
		this.ld=new LuceneDirectory(lucenePath);
	}

	public void indexTreeS(ISearchableInputNode<T> tree) {
		clearTreeIndex();
		ld.clearIndex();
		int lastId = 0;
		Document d = new Document();
		for (Field f : tree.getIndexFields()) d.add(f);
		ld.getLuceneWriter().addDocument(d);
		for (ISearchableInputNode<T> c : tree.getSubNodes())
			lastId = processNodeToIndexS(c, 0, lastId + 1);
		addToTree.setInt(1, 0);
		addToTree.setInt(2, lastId);
		addToTree.setNull(3, Types.INTEGER);
		addToTree.setString(4, tree.getId());
		addToTree.setObject(5, tree.getInfo());
		addToTree.executeUpdate();
		nodeMap = null;
		ld.optimize();
	}

	private int processNodeToIndexS(ISearchableInputNode<T> c, int parentId, int myId) {
		Document d = new Document();
		for (Field f : c.getIndexFields()) d.add(f);
		ld.getLuceneWriter().addDocument(d);
		int lastId = myId;
		for (ISearchableInputNode<T> c2 : c.getSubNodes())
			lastId = processNodeToIndexS(c2, myId, lastId + 1);
		addToTree.setInt(1, myId);
		addToTree.setInt(2, lastId);
		addToTree.setInt(3, parentId);
		addToTree.setString(4, c.getId());
		addToTree.setObject(5, c.getInfo());
		addToTree.executeUpdate();
		return lastId;
	}

	public Collection<IndexedNode<T>> match(Query q) {
		final Collection<IndexedNode<T>> ret = new ArrayList<IndexedNode<T>>();
		ld.getLuceneSearcher().search(q, new HitCollector() {

			@Override
			public void collect(int docid, float score) {
				ret.add(getNodeById(docid));
			}

		});
		return ret;
	}
}
