package fi.seco.semweb.service.indexing.tree.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;

import fi.seco.semweb.util.LString;

public class SearchableCategoryInfo implements Serializable {

	private static final Log log = LogFactory.getLog(SearchableCategoryInfo.class);
	private static final long serialVersionUID = 2L;
	private LString qlabels = new LString();
	private List<LString> labels = new ArrayList<LString>();
	private boolean showEvenIfNoHits;

	public SearchableCategoryInfo(Resource r, List<Property> properties, boolean showEvenIfNoHits) {
		this.showEvenIfNoHits=showEvenIfNoHits;
		for (Property p : properties)
			if (r.hasProperty(p)) labels.add(new LString(r,p));
		ListIterator<LString> l = labels.listIterator(labels.size());
			while (l.hasPrevious()) qlabels.addAll(l.previous());
		if (labels.size()==0) {
			log.warn("Resource "+r+" has no usable labels!");
			labels.add(new LString(r.getURI()));
			qlabels.add(null,r.getURI());
		}
	}

	public boolean isShowEvenIfNoHits() {
		return showEvenIfNoHits;
	}

	public String getLabel(String lang) {
		return qlabels.getMatchAny(lang);
	}

	public String getLabel(String match, String lang) {
		match = match.toLowerCase();
		for (LString l : labels) {
			String t = l.getMatch(lang);
			if (t!=null && t.toLowerCase().contains(match)) return t;
		}
		for (LString l : labels) for (String ll : l.getAll())
			if (ll!=null && ll.toLowerCase().contains(match)) return ll;
		return null;
	}

	public List<LString> getLabels() {
		return labels;
	}
}
