package fi.seco.semweb.service.indexing.tree.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.lucene.document.Field;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;

import fi.seco.semweb.util.JenaHelper;
import fi.seco.semweb.util.LString;
import fi.seco.semweb.util.iterator.AMappingIterator;

public class SearchableOntodellaCategory extends AOntodellaCategory implements ISearchableInputNode<SearchableCategoryInfo> {

	private List<Property> properties;
	private Model dataModel;

	public SearchableOntodellaCategory(Resource resource, Model dataModel,List<Property> properties) {
		super(resource);
		this.dataModel=dataModel;
		this.properties=properties;
	}

	public SearchableCategoryInfo getInfo() {
		return new SearchableCategoryInfo((Resource) resource.getProperty(projectionOf).getResource().inModel(dataModel),properties,false);
	}

	public Iterable<? extends ISearchableInputNode<SearchableCategoryInfo>> getSubNodes() {
		if (!resource.hasProperty(subCategories)) return Collections.emptyList();
		return new AMappingIterator<RDFNode, ISearchableInputNode<SearchableCategoryInfo>>(JenaHelper.getListItems(resource, subCategories)) {

			@Override
			public ISearchableInputNode<SearchableCategoryInfo> map(RDFNode src) {
				return new SearchableOntodellaCategory((Resource)src,dataModel,properties);
			}

		};
	}

	public Collection<Field> getIndexFields() {
		Collection<Field> ret = new ArrayList<Field>();
		SearchableCategoryInfo i = getInfo();
		for (LString labels : i.getLabels())
			for (String label : labels.getAll()) ret.add(new Field("label",label,Field.Store.NO,Field.Index.TOKENIZED));
		return ret;
	}

}
