package fi.seco.semweb.service.lucenegator;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.highlight.WeightedTerm;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;

public class LucenegatorTest {
	private static final String[] catids = { "A", "B" };
	
	private static List<WeightedTerm> documentCategories(int doc) {
		List<WeightedTerm> l = new LinkedList<WeightedTerm>();
		for (int i = 0; i < catids.length; ++i) {
			float weight = (((doc >> (i * 3)) & 7) - 3) / 4.0f;
			System.out.println(doc + ": " + catids[i] + " weight " + weight);
			if (weight <= 0) {
				continue;
			}
			WeightedTerm wt = new WeightedTerm(weight, catids[i]);
			l.add(wt);
		}
		return l;
	}
	
	public static Directory createIndex() {
		System.out.println("Constructing index...");
		Directory d = new RAMDirectory();
		try {
			IndexWriter iw = new IndexWriter(d, new StandardAnalyzer(), true);
			CategorySimilarity.install(iw, "catid");
			CategoryWriter cw = new CategoryWriter("catid", 8);
			for (int i = 0; i < 64; ++i) {
				Document doc = new Document();
				doc.add(new Field("id", ""+i, Field.Store.YES, Field.Index.NO));
				cw.setCategories(doc, documentCategories(i));
				iw.addDocument(doc);
			}
			System.out.println("Done. Index contains " + iw.docCount() + "documents.");
			iw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return d;
	}
	
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
		Directory d = createIndex();
		try {
			System.out.println("Searching...");
			IndexSearcher is = new IndexSearcher(d);
			Query q1 = new CategoryQuery(new Term("catid", "A"));
			Query q2 = new CategoryQuery(new Term("catid", "B"));
			BooleanQuery q = new BooleanQuery();
			q.add(q1, BooleanClause.Occur.MUST);
			q.add(q2, BooleanClause.Occur.MUST);
			Hits hits = is.search(q);
			for (int i = 0; i < hits.length(); ++i) {
				System.out.println(hits.doc(i).get("id") + " score: " + hits.score(i));
				System.out.println(is.explain(q, hits.id(i)));
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
