package fi.seco.semweb.service.poka;
/**
 * Geneerinen dokumenttieristäjä:
 * sisältää inputin asettajan,
 * käsitteiden etsijän (yksi- tai monisanaiset) sekä
 * muita perustoiminnallisuuksia
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.htmlparser.util.ParserException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import fi.seco.semweb.service.poka.frame.Token;
import fi.seco.semweb.service.poka.tokenIndexer.IndexingFailedException;
import fi.seco.semweb.service.poka.tokenIndexer.TokenIndexer;
import fi.seco.semweb.service.poka.util.FoundResources;
import fi.seco.semweb.service.poka.util.TermoQuery;
import fi.seco.semweb.service.poka.util.TrieIndex;

//entinen PokaDom
public class DocumentExtractor {
	private static final Log log = LogFactory.getLog(DocumentExtractor.class);
	
	public static final String DEFAULTNAMESPACE = "http://www.seco.hut.fi/ns/2005/10/alm#";
	public static final String VERBNAMESPACE = "http://www.seco.hut.fi/ns/2006/02/alm#";
	
	protected String shortForm;// =null; //pätkitty muoto termille (jos on löytynyt)
	protected boolean hasShort;// = false; //löytyykö lyhyt muoto käsitteelle?
	
	protected String parsedHTML;
	
	protected Document doc; //DOM-doc
	
	public DocumentExtractor() {

		shortForm = null; //termin lyhyt muoto
		hasShort = false; //ja lyhyen muodon olemassaolo 

	}

	
	public void setDocument(String file) {	
		String pokadoc = new PokaDoctor(file).getResult();
		this.setInput(pokadoc);		
		
	}
		
	public void setInputString(String input) {
		String pokadoc = new PokaDoctor(input, false).getResult();
		this.setInput(pokadoc);		
	}
	
	public Document cloneDocument() 
			throws ParserConfigurationException, 
					SAXException, 
					IOException  {
		
		log.debug("clonedoc!");
		Document cloneDoc;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance(); 
		DocumentBuilder db = factory.newDocumentBuilder();
		
		cloneDoc = db.parse(this.getDocumentAsInputSource());
		
		return cloneDoc;
	}
	
/**
 * 
 * @param documentUrl
 */
	
	private void setInput(String parsed) {
 		this.doc = null;
	    try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance(); 
			DocumentBuilder db = factory.newDocumentBuilder();
			log.debug("Parsitaan: ");
			doc = db.parse(new InputSource(new StringReader(parsed)));
			log.debug("Parsinta ok!");
			

		} catch(NullPointerException e) {
			System.out.println("Error reading file!"+e);
		} catch(IOException e) {
			System.out.println("IO-error (db.parse): "+e);
		} catch(FactoryConfigurationError e) {
			System.out.println("Factory config error!"+e);
		} catch (ParserConfigurationException e) {
			System.out.println("Parser config error!"+e);
		} catch(SAXException e) {
			System.out.println("SaxException: "+e);
		}
		

	}
	/**
	 * Iteroidaan solmusta eteenpäin, tarkistetaan
	 * muodostavatko peräkkäiset tokenit käsitteen
	 * Jos käsite löytyy, se merkataan suoraan
	 * appendConceptilla dokumenttiin.
	 * 
	 * 
	 * @param token solmu, josta lähdetään liikkeelle
	 * @return kuinka monta solmua hypätään yli
	 * 
	 */
	protected int checkLongTerm(String nodeType, 
							    Node item, 
							    String [] nodeNames, 
							    int counter, 
							    TrieIndex wordIndex, 
							    TermoQuery kb, 
							    boolean upper,
							    FoundResources instBase) {

		ArrayList <Node> conceptTokens = new ArrayList <Node>();
		int increment = 0;
		String preceding = null;
//		Node precedingNode = null;		
		String nodetypesRegex = "";

		for(String types:nodeNames) {
			nodetypesRegex += types+"|";
		}			
		
		nodetypesRegex = nodetypesRegex.substring(0,nodetypesRegex.length()-1);
		
		String temp = item.getChildNodes().item(1).getTextContent()+" ";
		
		conceptTokens.add(item);
		item = this.nextNode(item);
		
		if(item == null) {
			log.debug("Termi '"+temp+"' lauseen lopussa, siirrytään yksittäiseen");
		}
		while(item != null) {

			temp += item.getChildNodes().item(1).getTextContent();  			
       		log.debug("Etsitään termiä: '"+temp+"'");
			
			if(item.getNodeName().matches(nodetypesRegex)) {
				log.debug("\t--noden tyyppi mätsää, kasvatetaan laskuria--");
				increment++;
			}/* else { //TODO: keksi jotain parempaa: kys. versio hylkää monisanaisen käsitteen, vaikka preceding olisikin käsite.
				log.debug("\t--node väärää tyyppiä, ("+item.getNodeName()+") LOPETETAAN!");
				return counter;
			}*/
       		
       		if(wordIndex.containsPrefix(temp)) {
           		log.debug("Sana '"+temp+"' saattaa jatkua...");
           		preceding = temp;
           		List <String> l = wordIndex.matchByPrefix(temp);
           		for(String s:l) {
           			log.debug("\t--> '"+s+"'");
           		}
           		temp += " ";
           		conceptTokens.add(item);
           		item = this.nextNode(item);          		
           	} else {   		
         		log.debug("\t--> Termiä ei löydy, tarkastetaan edellinen...");
           		if(preceding != null) {
           			if(wordIndex.contains(preceding)) {

    					HashSet <String> foundUris = wordIndex.getUris(preceding);

//    					TODO: virheiden käsittely
    					           				
           				String concepts = "";
           				if(upper) {
           					concepts = this.upperClasses(foundUris, kb); //wordIndex.getUris(preceding)
           				} else {
           					for(String s: wordIndex.getUris(preceding)) {
    							concepts += s +" | ";
    						}
           					//concepts = wordIndex.getConceptString(temp);
           				}
           				log.debug("\tMonisanainen käsite: "+ concepts);
    					
    					int last = conceptTokens.size()-1;
    					
    					Node foundToken = conceptTokens.get(last);
    					
    					int sentence = Integer.parseInt(foundToken.getParentNode().getAttributes().getNamedItem("no").getTextContent());
    					int token = Integer.parseInt(foundToken.getAttributes().getNamedItem("no").getTextContent());
    					log.debug("Monisanaisen viimeinen token: lause "+sentence+", token "+token);
    					
    					this.appendConcept(nodeType, foundToken, concepts, true);	
   					

    					

    					for(String s: foundUris) { //jokaista uria kohti lisätään sijainti
    						instBase.addURI(s, sentence, token, true);
    					}
    					
    					for(int i=0; i<last;i++) {
    						foundToken = conceptTokens.get(i);
    						
    						this.appendConcept(nodeType, foundToken, ""+i, true);
    						
//    						sentence = Integer.parseInt(foundToken.getParentNode().getAttributes().getNamedItem("no").getTextContent());
        					token = Integer.parseInt(foundToken.getAttributes().getNamedItem("no").getTextContent());
        					log.debug("Monisanaisen aiempi token: lause "+sentence+", token "+token);
        					for(String s: foundUris) { //jokaista täsmäävää uria kohti lisätään tokenin sijaintitieto
        						instBase.addURI(s, sentence, token, false);
        					}	
    					}   					
//TODO: tarkista viat!
    					return counter+increment+2;
           			}
           		} else {
           			log.debug("\t--> Preceding oli null...");
           		}	
        		return counter;
           	}	      		
   		}
		return counter;
	}
	
	/**
	 * Palauttaa seuraavan sisaren, jos ei löydy, palautetaan null (3)
	 */
	protected Node nextNode(Node n) {
		try { 
			n = n.getNextSibling();
		} catch(NullPointerException e) { //tulee jostain syystä, kaiketi kun tavara loppuu...
			return null;
		}
					
		if(n != null) { //jos solmussa on tavaraa, 
			while(n.getNodeName().equals("#text")) { //poistetaan #text
				n = n.getNextSibling();
				if(n == null) {
					return null;
				}
			}
		}else {
			return null;
		}
		return n;
	}
	
/**
 * Lisää käsitteet dokumenttiin
 * @param nodeType lisättävän solmun tunniste, esim 'C' (=CONCEPT)
 * @param nodeNames solmut, joista käsitettä etsitään, esim BS, AD, UP
 * @param multiword true, jos halutaan etsiä monisanaisia käsitteitä
 * @param upper true, jos solmun tietoihin halutaan lisätä yläkäsite
 * @param wordIndex indeksi, josta sanoja halutaan etsiä
 * @param kb tietämyskanta, josta haetaan tieto yläkäsitteestä
 * @param instBase resurssikanta, jonne löydetyt resurssit t tallennetaan
 * UUSI:
 * @param split jos true, etsitään pilkotulle sanalle vastinetta:
 *  lippu#tanko --> tanko
 * 
 */

	public void addConcepts(String nodeType, 
							String [] nodeNames, 
							boolean multiword, 
							boolean upper, 
							TrieIndex wordIndex, 
							TermoQuery kb, 
							FoundResources instBase,
							boolean split) {
  	    
		String expression = "";
		for(String s: nodeNames) {
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);
		
		log.debug("XPATH / etsitään käsitettä solmuista "+expression);
		
	    NodeList nodes;
		String temp = "";
		XPath xpath = XPathFactory.newInstance().newXPath();

		try {

			//ota docin käsitesolmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			
	        for(int i = 0; i < nodes.getLength(); i++){
	        	Node item = nodes.item(i);
				
	        	if(multiword) {
	        		log.debug("i ennen  : "+ i);
	        		i = this.checkLongTerm(nodeType, item, nodeNames, i, wordIndex, kb, upper, instBase);
	        		log.debug("i jälkeen: "+ i);
	        	}
	        	
	        	item = nodes.item(i); //mahd. iteroitu käsite
	        	if(item == null) {
	        		log.debug("iteroitavat solmut lopussa");
	        		break;
	        	}
	        	temp = item.getChildNodes().item(1).getTextContent();
	        	log.debug("Etsitään vastaavuutta yksittäiselle tokenille: '"+temp+"'");
	           	String concepts = "";
	           		
				if(this.containsURI(temp, wordIndex, split)) { //vanha osa PokaDomista
					
					//jos käsitemuoto on lyhyt
					if(this.hasShortForm()) {
						temp = this.getShort();
						log.debug("\t--> lyhyt vastine löytyy");
					} else {
						log.debug("\t--> pitkä vastine löytyy");
					}
					HashSet <String> foundUris = wordIndex.getUris(temp);

					//TODO: virheiden käsittely
					int sentence = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getTextContent());
					int token = Integer.parseInt(item.getAttributes().getNamedItem("no").getTextContent());		
					
					for(String s: foundUris) {
						//System.out.println("ARVOT: "+sentence+", "+token+", "+s);
						instBase.addURI(s, sentence, token, true);
					}
					if(upper) {
						concepts = this.upperClasses(foundUris, kb);
					} else {
						for(String s: foundUris) { //wordIndex.getUris(temp)
							concepts += s +" | ";
						}
//						HUOM: ei saa poistaa, käytetään OpasExtractorissa!
//						TODO: ehkä vois trimmata vikan '|' pois ja pistää metodit kauniimmiksi...
//						--> Muutoksia OpasExtractorin metodiin addInstanceUris
//						concepts = concepts.substring(0, concepts.length()-3);//viim. ' | ' pois

						//concepts = wordIndex.getConceptString(temp);
					}
					log.debug("\tKäsite: "+ concepts);
						
					this.appendConcept(nodeType, item, concepts, true);//true, SOLMULLA LEMMA, käsitteen lisäys
				}
	
	        }
         		
		} catch (XPathExpressionException e) {
	    		System.out.println(e);
	    }

	}
	
/**
 * Etsii käsitettä Triestä, jos ei löydy,
 * etsii uudelleen pätkity(i)llä käsitte(e/i)llä, 
 * palauttaa true jos käsite löytyy.
 * 
 * Palauttaa true myös silloin, jos käsitteen lyhyt
 * muoto löytyy: esim. kuulakärki#kynä ei löydy, mutta kynä löytyy
 * palautetaan true
 * 
 * Lyhyen muodon olemassaolo kysytään metodilla hasShortForm()
 * 
 * @param term etsittävä sana
 * @return true jos käsite tai siitä erotettu lyhyt muoto löytyy
 */

	public boolean containsURI(String term, TrieIndex indexTrie, boolean split) {
		int index;
		String temp;
		this.hasShort = false;
		//jos termille ei löytynyt vastinetta...
		
		if(!indexTrie.contains(term)) {

			
			if(split) { //jos halutaan tarkistaa lyhyt muoto
			
				if((index = term.lastIndexOf('#')) != -1) { //...ja termi muodostuu useammasta osasta
					temp = term.substring(index+1); //otetaan termiksi loppuosa

					if(indexTrie.contains(temp)) {	//jos lyhyt muoto on olemassa
						this.shortForm = temp;
						this.hasShort = true;										
						return true;
					}
				}
			} 
//11.05.06: kikkailulisäys: jos token monisanainen, etsitään viimeiselle
//sanalle vastinetta. Esim: <text>Suomalaisen Kirjallisuuden Seura</text>
//tapaus symmetrinen ylläolevan tarkistuksen kanssa
			if((index = term.lastIndexOf(' ')) != -1) { //...ja termi muodostuu useammasta osasta
				temp = term.substring(index+1); //otetaan termiksi loppuosa
				//jos lyhyt muoto on olemassa
				if(indexTrie.contains(temp)) {
					this.shortForm = temp;
					this.hasShort = true;										
					return true;
				}
			}
			return false;
		}
		return true; //käsite löytyy
	}
	
		
	protected String getShort() {
		return shortForm;
	}
	
	protected boolean hasShortForm() {
		return hasShort;
	}
	
	/**
	 * Termin lisäys terminologiaan
	 *
	 */
	public boolean addLabel(String uri, String label, TrieIndex index) {
		return index.addLabel(uri, label);	 
	}
	/**
	 * Termin poisto
	 */
	public boolean removeLabel(String uri, String label, TrieIndex index) {
		return index.removeLabel(uri, label);
	}
	
	
//	--------------------------------------	

/**
 * Palauttaa xml-documentin Stringinä, UTF-8-enkoodauksella
 */	
	public String getDocument() {
 		try {
			ByteArrayOutputStream out;
			Transformer transformer;
			TransformerFactory tFactory;

			tFactory = TransformerFactory.newInstance();
			transformer = tFactory.newTransformer();

			transformer.setOutputProperty(javax.xml.transform.OutputKeys.ENCODING,"UTF-8");

			out = new ByteArrayOutputStream();
			transformer.transform(new DOMSource(doc.getDocumentElement()), new StreamResult(out));
			if (out != null) {
				return new String(out.toByteArray());
			}
			else {
				return null;
			}
		} catch (TransformerException te) {
			te.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Palauttaa xml-documentin InputSourcena
	 */	
	public InputSource getDocumentAsInputSource() {
 		try {
			ByteArrayOutputStream out;
			Transformer transformer;
			TransformerFactory tFactory;

			tFactory = TransformerFactory.newInstance();
			transformer = tFactory.newTransformer();

			transformer.setOutputProperty(javax.xml.transform.OutputKeys.ENCODING,"ISO-8859-1");

			out = new ByteArrayOutputStream();
			transformer.transform(new DOMSource(doc.getDocumentElement()), new StreamResult(out));
			if (out != null) {
				return new InputSource(new ByteArrayInputStream(out.toByteArray()));
			}
			else {
				return null;
			}
		} catch (TransformerException te) {
			te.printStackTrace();
			return null;
		}
	}
		
/**
 * Lisätään käsite puuhun, 
 * parametreina Node item, käsiteltävä solmu
 *         sekä lisättävä tekstisisältö
 *         
 *         TODO: miettimisen arvoinen asia: kannattaako alkaa käpistelemään
 *               uusia tietorakenteita päällekkäisten käsitteiden suhteen 
 *               vaiko suoraan lisätä appendConceptiin toimintamalli, jossa
 *               info-solmua kasvatetaan? HUOM: nimet merkataan muualla:
 *               päällekkäisyydessä syntyy ongelmia, täytyy tietää minkä
 *               ontologian / työkalun käsite on kyseessä (nimillä id,
 *               ontologioilla uri). 
 *            
 *               TODO: hasLemma taitaa olla vanha jäänne?
 *
 */
	protected Node appendConcept(String nodetype, Node item, String desc, boolean hasLemma) {
		Element concept;

		Element info;

		//luo solmu C
		concept = (Element) doc.createElement(nodetype);
			
		//itemin (tokenin) numeron lisäys.
		concept.setAttribute("no", item.getAttributes().getNamedItem("no").getTextContent());
			
		//liitä alkuperäinen T C:n lapseksi
		Node text = item.getFirstChild();
		concept.appendChild(text);
	 		
		//liitä info-solmu
		info = (Element) doc.createElement("I");
		info.setTextContent(desc);
		concept.appendChild(info);


		while(item.hasChildNodes()) {
			concept.appendChild(item.getFirstChild());
		}
			
			
			
		//poista item, korvaa concept-solmulla
		item.getParentNode().replaceChild(concept, item);
		return concept; //lisätty 3.8., regexissä tarvitaan
	}
	

	
	
	/**
	 * iteroi annetun listan käsitteet, hakee jokaiselle
	 * käsitteelle yläluokan
	 */

	protected String upperClasses(HashSet <String> matching, TermoQuery kb) {
		Iterator <String> i = matching.iterator();
		String conc;
		String result = "";
		while(i.hasNext()) {
			conc = i.next();
				//lisätään instanssit PokaInstanceen
//			this.inst.addURI(conc);
				//listataan käsite ilman URIn alkuosaa + yläkäsite
			int start = conc.indexOf("#");
			if(start==-1) { //YSO sisältää resurssin http://yso.fi
				start = 0;
			}
			
			result += conc.substring(start, conc.length()) +
				": " + kb.getUpper(conc) +" |";
					
		}
		if(result.length() >= 2) {
			result = result.substring(0,result.length()-2);
		}
		return result;
	}

/**
 * Palauttaa käsiteltävänä olevan Poka-dokumentin dom-dokkarina
 * @return käsittelyssä oleva dom-dokkari
 */
	public Document getXMLDoc() {
		return this.doc;
	}
	
	
	public boolean writeToFile(String filename, String text) throws IOException {
		log.info("Kirjoitetaan tiedostoon '"+filename+"'...");
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		pw.write(text);
		pw.flush();
		log.info("...tiedosto valmis!");
		return true;
	}
	
	/**
	 * Palauttaa kyseisen lauseen perustekstiesityksenä
	 * @param sentence lauseen numero (1-n)
	 * @return tekstiesitys lauseesta
	 */
	public String getSentenceAsText(int sentence) {
       String expression = "/A/S["+sentence+"]/*";
       StringBuffer result = new StringBuffer();
	        
	   NodeList nodes;
	   XPath xpath = XPathFactory.newInstance().newXPath();

	   NodeList children;
							
	   try {
		   //ota docin käsitesolmut NodeListiin
		   nodes = (NodeList) xpath.evaluate(expression, this.getXMLDoc(), XPathConstants.NODESET);
							
		   for(int i = 0; i < nodes.getLength(); i++){
			   Node item = nodes.item(i); 
	           		
	           if(item.getNodeName().equals("P")) {
	           		return "";
	           }
	           children = item.getChildNodes();
	           item = children.item(0);
	           result.append(item.getTextContent()+" ");
	           		           		   		
		   }
	        	

	   } catch (XPathExpressionException e) {
	   		System.out.println(e);
	   }

	   return result.toString();
		    	    		
	}
/**
 * Palauttaa koko dokumentin tekstinä
 * @return 
 */
	public String getDocumentAsText() {
		int count = this.sentenceCount();
		if(count==-1) {
			return "";
		}
		StringBuffer sb = new StringBuffer();
		for(int i=1; i<count; i++) {
			sb.append(this.getSentenceAsText(i));
		}
		return sb.toString();
	}
	
	
	/**
	 * lauselaskuri 
	 * @return
	 */	
	public int sentenceCount() {
		String expression = "/A/S";
		int result = -1;
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		 
		try {
			nodes = (NodeList) xpath.evaluate(expression, this.getXMLDoc(), XPathConstants.NODESET);
			result = nodes.getLength();
		} catch (XPathExpressionException e) {
			System.out.println("XPATHEXCEPTION: " +e);
			System.exit(0);
		}
		return result;
	}
//----------------------------
	
	/**
	 * Palauttaa kyseisen lauseen xml-rakenteen
	 * @param sentence lauseen numero (1-n)
	 * @return lauseen rakenteen
	 */
	public String getSentence(int sentence) {
       String expression = "/A/S["+sentence+"]";
      	        
	   Node node;
	   XPath xpath = XPathFactory.newInstance().newXPath();

	   try {
		   //ota docin käsitesolmut NodeListiin
		   node = (Node) xpath.evaluate(expression, this.getXMLDoc(), XPathConstants.NODE);				

           ByteArrayOutputStream out;
   		   Transformer transformer;
   		   TransformerFactory tFactory;

   		   tFactory = TransformerFactory.newInstance();
   		   transformer = tFactory.newTransformer();

   		   transformer.setOutputProperty(javax.xml.transform.OutputKeys.ENCODING,"ISO-8859-1");
   		   transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
   		   
    	   out = new ByteArrayOutputStream();
   		   transformer.transform(new DOMSource(node), new StreamResult(out));

   		   if (out != null) {
   			   return new String(out.toByteArray());
		
   		   } else {
   			   return null;
   		   }
           
	   } catch (XPathExpressionException e) {
	   		System.out.println(e);
	   } catch (TransformerException te) {
		te.printStackTrace();
	   }      
	   return null;
		    	    		
	}
	
/**
 * Uusinta uutta 11.9.06: lämmätään dokumentiin
 * sisään sanojen sijainti dokumentin esitysmuodossa
 * 
 * @param text tekstidokumenttijoka syötettiin inputStringinä
 * @return true, kun onnistui
 * 
 */

	public boolean addTokenIndex(String text) throws
										IndexingFailedException {
		TokenIndexer tt = new TokenIndexer(text, false);
		
		log.info("Lisätään indeksi dommiin");
		this.doc = tt.parseWithDom(this.doc);
		log.info("ok!");

		return true;
		
	}
/**
 * HTML-indeksin lisäys PokaDokkariin
 * TODO / FIXME: poista urlOrStr parametrista,
 * voidaan antaa suoraan DocumentExtractorista.
 * --> allaoleva typerää toisteisuutta.
 * 
 * @param urlOrStr
 * @param doc jos true, url on linkki, muuten sisältö
 * @return
 * @throws IndexingFailedException
 */
	public boolean addHTMLIndex(String urlOrStr, boolean doc) { 
						// throws 	IndexingFailedException, ParserException {
		log.info("Lisätään html-indeksi dommin");
		
		try {
			TokenIndexer tt = new TokenIndexer(urlOrStr, doc, this.doc);
			this.doc = tt.getDoc();
			log.info("ok!");
			//htmlparserin parsima html
			this.parsedHTML = tt.getParsedHTML();
		} catch (IndexingFailedException e) {
			log.error("HTML-indeksointivirhe: "+e);
			return false;
		} catch(ParserException e) {
			log.error("HTML-parsintavirhe: "+e);
			return false;
		}
		return true;
	}
	public String getParsedHTML() {
		return this.parsedHTML;
	}
	
/*
 * 
 * TODO: POIS JOS TOINEN TOIMII

	public boolean addTokenIndex(ArrayList <TokenIndexer.Token> tokens) {
		
		String expression = "/A/S/*"; //analysis sentence all
	    NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		int count = 0;
		try {

			//ota docin käsitesolmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			
 
 			System.out.println(this.getDocument());
			if(tokens.size() != nodes.getLength()) {
				System.err.println("KOKO ERI, palataan!");
				System.err.println("Tokens: "+tokens.size()+", nodeja: "+nodes.getLength());
				return false;
			}
			
			
	        for(int i = 0; i < nodes.getLength(); i++){
	        	
	        	System.out.println("KIERROS: "+i);
	        	Node item = nodes.item(i);
	        	
	        	if(item.getNodeName().equals("P")) {
	        		System.out.println("OHITETAAN PARA"+item.getTextContent());
	        		continue;
	        	} else {
	        		TokenIndexer.Token t = tokens.get(count);
		        	
		        	System.out.println("node: '"+item.getFirstChild().getTextContent()+"'");
		        	System.out.println("tokn: '"+t.getStr()+"'");
		        	
		        	Element coordinate;
		        	Element start;
		        	Element end;
		    		Element info;

		    		//luo solmu C
		    		coordinate = (Element) doc.createElement("CR"); //COORDINATE
		    		start = (Element) doc.createElement("start");
		    		end =   (Element) doc.createElement("end");

		    		//TODO: String vai intti, tyypitetty xml-arvo?
		    		start.setTextContent(""+t.getStart());
		    		end.setTextContent(""+t.getEnd());
		    		
		    		coordinate.appendChild(start);
		    		coordinate.appendChild(end);
		    		
		    		//itemin (tokenin) numeron lisäys.

		    		item.appendChild(coordinate);	
	        		
	        		count++;
	        	}
	        
	        }
	              
		} catch (XPathExpressionException e) {
    		System.out.println(e);
		}
		
		System.out.println("tokeneita: "+tokens.size());
		System.out.println("nodet:   : "+count);
		return true;
	}
*/
	
	
/*	
	public static void main(String [] args) {
		
		DocumentExtractor ex = new DocumentExtractor();
		String hup = new PokaDoctor("/home/alm/aaparser/text/Gallenkallela.html").getResult();
		ex.setDocument(hup);
	}
*/	

	
}
