package fi.seco.semweb.service.poka;

/**
 * Peruseristin, DocumentExtractorin aliluokka:
 * tarkentaa toiminnallisuuksia olemassaoleviin
 * ontologioihin. Härpäke etsii nimiä, paikkoja, regexejä 
 * sekä verbejä.
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.spi.Configurator;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.name.NameExtractor;
import fi.seco.semweb.service.poka.name.NameIndex.Name;
import fi.seco.semweb.service.poka.util.DocToHTML;
import fi.seco.semweb.service.poka.util.PlaceQuery;
import fi.seco.semweb.service.poka.util.FoundResources;
import fi.seco.semweb.service.poka.util.TrieIndex;
import fi.seco.semweb.service.poka.util.VerbQuery;
import fi.seco.semweb.service.poka.util.YsoQuery;

public class GeneralExtractor extends DocumentExtractor {
	private static final Log log = LogFactory.getLog(GeneralExtractor.class);
	
	private static final int YSOKB = 1;
	private static final int PLACEKB = 2;
	private static final int VERBKB = 3;
	private static final int REGEXKB = 4;
	
	protected YsoQuery yso; //YSOn Jena-kyselyrajapinta
	protected VerbQuery verbs;
	protected PlaceQuery places;
	
	private NameExtractor ne; //nimen tunnistin
	
	public TrieIndex termExist;
	public TrieIndex verbExist;
	public TrieIndex placeExist;
	
	public static final String CONCEPT = "C"; //CONCEPT -- käsitesolmu
	public static final String REGEX = "R"; //REGEX -- regexillä löydetty käsite
	public static final String NAME = "N"; //NAME -- tulee myöhemmin nimien tunnistukseen
	public static final String VERB = "V";
	public static final String PLACE = "PL";

	public static final String [] VERBNODES = {"VB"};
	public static final String [] NUMERALNODES = {"NM"}; //=regex
	public static final String [] CONCEPTNODES = {"AD","BS","UP"};
	public static final String [] UPNODES = {"UP"};

	
	protected FoundResources ysoInstances; //tänne kootaan URIt
	protected FoundResources placeInstances; //tänne kootaan URIt
	protected FoundResources verbInstances;
	protected FoundResources regexInstances;

	private HashMap<String, String> regexToC;
	
	public GeneralExtractor() {
		super();
		yso = new YsoQuery(Configuration.getNeuOntology(), "UTF-8");
		verbs = new VerbQuery(Configuration.getVerbs(), "ISO-8859-1");
		places = new PlaceQuery(Configuration.getPlaces(), "UTF-8");
		
		this.ysoInstances   = new FoundResources();
		this.verbInstances  = new FoundResources();
		this.placeInstances = new FoundResources();
		this.regexInstances = new FoundResources();
		
		regexToC = new HashMap <String, String>();
		ne = new NameExtractor();
		
		log.info("Luodaan trie-rakenteet...");
		try {
			termExist = new TrieIndex(Configuration.getNeulemmas(), 
					                  "ISO-8859-1", 
					                  DocumentExtractor.DEFAULTNAMESPACE);	
			
			placeExist = new TrieIndex(Configuration.getPlaceLemmas(),
									  "ISO-8859-1",
									  DocumentExtractor.DEFAULTNAMESPACE);
			
			verbExist = new TrieIndex(Configuration.getVerbs(), 
									"ISO-8859-1", 
									DocumentExtractor.VERBNAMESPACE);
			
		} catch(Exception e) {
			System.out.println("TrieQuery error: " + e);
			//System.exit(0);
		}
		log.info("...ok, termistön toisistaan eroavien labeleiden lkm: " +termExist.size());
		log.info("       paikkojen toisistaan eroavien labeleiden lkm: " +placeExist.size());		
		log.info("       verbistön toisistaan eroavien labeleiden lkm: " +verbExist.size());

	}
	
	public void setDocument(String documentUrl) {
		log.debug("Asetetaan dokumentti '"+documentUrl+"'");
		super.setDocument(documentUrl);
		log.debug("Asetetaan nimentunnistin dokumentille");
		this.ne.setDocument(doc);
		this.clearPokaInstance();
	}
	
	public void setInputString(String input) {
		log.debug("Asetetaan merkkijono");
		super.setInputString(input);
		log.debug("Asetetaan nimentunnistin dokumentille");
		this.ne.setDocument(doc);
		this.clearPokaInstance();
	}
	
	
	/**
	 * Lisää paikkasolmut dokumenttiin
	 * @param multiword jos true, etsitään myös monisanaisia yksikköjä
	 * @param upper jos true, lisätään solmuun tieto yläkäsitteestä
	 */
	public void addPlaces(String [] nodeNames, boolean multiword, boolean upper) {
 	    this.addConcepts(GeneralExtractor.PLACE, nodeNames, multiword, upper, this.placeExist, this.places, this.placeInstances, false); 	    
	}
	
	public void addVerbs(boolean multiword, boolean upper) {
		this.addConcepts(GeneralExtractor.VERB, GeneralExtractor.VERBNODES, multiword, upper, this.verbExist,this.yso, this.verbInstances, true);
	}
	
	public void addYsoConcepts(String [] nodeNames, boolean multiword, boolean upper) {
		this.addConcepts(GeneralExtractor.CONCEPT, nodeNames, multiword, upper, this.termExist, this.yso, this.ysoInstances, true);
	}
	
	
	
//	TODO: otettu PokaConceptista. Voisi sijaita vakituisesti täällä	
	public String getHTML() { 
		String result;
		System.out.print("Luodaan HTML...");
		result =  DocToHTML.convertHTML(this.getDocument(),"/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/stylesheets/pokadoc.xsl");
		System.out.println("...HTML ok!");
		return result;
	}
	
	
/**
 *  Palauta käsitesolmut
 */
	public String getConceptNodes(String [] nodeNames) {
		String expression = "";
		if(nodeNames == null) {
			expression = "/A/S/C";
		} else {
			for(String s: nodeNames) {
				expression += "/A/S/"+s+"|";
			}
			expression = expression.substring(0,expression.length()-1);
		}
		
	
		NodeList nodes;
		StringBuffer result = new StringBuffer();
		XPath xpath = XPathFactory.newInstance().newXPath();

		try {
			//ota docin käsitesolmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);

			for(int i = 0; i < nodes.getLength(); i++){

				Node item = nodes.item(i);
						
		        NodeList n  = item.getChildNodes();
		        for(int ii=0; ii< n.getLength()-1;ii++) {
		        	result.append(n.item(ii).getNodeName()+": "+n.item(ii).getTextContent());
		        }
		        result.append("\n");
		           		
		     }
		} catch (XPathExpressionException e) {
		    	 	System.out.println(e);
		}
		return result.toString();
	}
	
	
	
	
	

//	--------------------------------------
//	 REGEX
//	--------------------------------------
	public void addRegex(String regexp, String uri) {
		this.regexToC.put(regexp, uri);
	}
	public void removeRegex(String regexp) {
		this.regexToC.remove(regexp);
	}
//	TODO: yhdistä muihin jos siltä tuntuu	
	public void addRegexConcepts(String [] nodeNames, boolean upper) {
		Iterator <String> i = regexToC.keySet().iterator();
		String tempRegex;

		String expression = "";
		for(String s: nodeNames) {
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);
        NodeList nodes;
		String temp, currURI;
		String concepts;
		XPath xpath = XPathFactory.newInstance().newXPath();
		while(i.hasNext()) {
		
			tempRegex = i.next();
			log.debug("regex on: '"+tempRegex+"'");
			
			try {
				currURI = this.regexToC.get(tempRegex);
				log.debug("currURI: "+currURI);
				if(upper) {
					concepts = currURI.substring(	
						currURI.indexOf("#"), currURI.length())+": "+
                            yso.getUpper(currURI);			
				} else {
					concepts = currURI.substring(	
						currURI.indexOf("#"), currURI.length());
					
					log.debug("concepts: '"+concepts+"'");
				}
					 
				nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
        		for(int ii = 0; ii < nodes.getLength(); ii++){
          			Node item = nodes.item(ii);
					temp = item.getFirstChild().getTextContent(); //tekstimuoto
		
					log.debug("temp: '"+temp+"'");
					
					if(temp.matches(tempRegex)) {
						//false --> solmulla ei lemmaa
						item = this.appendConcept(REGEX, item, concepts, false);
						
						log.debug("item "+item.getNodeName());
						log.debug("parentnode local: " +(null==item.getParentNode()));
						
						//TODO: mietinnän paikka: lyödäänkö regexit ysoinstansseihin (V: ei.)
						int sentence = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getTextContent());
						int token = Integer.parseInt(item.getAttributes().getNamedItem("no").getTextContent());
						this.regexInstances.addURI(currURI, sentence, token, true);						
					}
				}
			} catch (XPathExpressionException e) {
	    		System.out.println(e);
		    }
		}

	}
	
//	REGEX LOPPUU
	public ArrayList<Name> getNames() {
		return this.ne.index.getNameObjects();
	}
		
/**
 * Palauttaa dokumentista löydetyt yso-instanssit
 * @return yso-instanssit
 */	
	public String getYSOUris() {
		return this.ysoInstances.toString();
	}
/**
 * Samat kuin getYSOURIs, mutta palauttaa Stringit settinä
 * @return
 */
	public Set<String> getYSOURISet() {
		return this.ysoInstances.getUris();
	}
		
/**
 * Paikka-urien palautus Stringinä
 * @return
 */
	public String getPlaceUris() {
		return this.placeInstances.toString();
	}
/**
 * Paikka-urit settinä
 * @return
 */
	
	public Set<String> getPlaceURISet() {
		return this.placeInstances.getUris();
	}
		
/**
  * Verbi-urien palautus
  * @return
  */
	public String getVerbUris() {
		return this.verbInstances.toString();
	}
/**
 * Verbit settinä
 * @return
 */	
	public Set<String> getVerbURISet() {
		return this.verbInstances.getUris();
	}
	
/**
 * Palauttaa uria vastaavat käsitteet Stringinä
 * 
 * @param uri yso-käsitteen uri
 * @return vastaavat labelit Stringinä
 */		
	public String getYsoLabel(String uri) {
		return this.termExist.getLabel(uri);
	}
/**
 * Palauttaa Yson urit hashSettinä. Käytetään D-eristimessä
 * @return urit HashSetissä
 */	
	public HashSet<String> getYsoUris() {
		return this.termExist.getTermURIs();
	}
/**
 * Palauttaa YSO-mallin kyselyrajapintaan käärittynä
 * @return YsoQuery-olio
 */
	public YsoQuery getYSO() {
		return this.yso;
	}
/**
 * Palauttaa Verbi-mallin kyselyrajapintaan käärittynä
 * @return VerbQuery-olio
 */
	public VerbQuery getVerbs() {
		return this.verbs;
	}
	
	public void addNames() {
		this.doc = ne.addNames();
	}
	
/**
 * Tyhjentää dokumentin instanssitiedot
 * Huom! tyhjennetään automaattisesti kun uusi dokumentti asetetaan
 */	
	public void clearPokaInstance() {
		this.ysoInstances.clear();
		this.placeInstances.clear();
		this.verbInstances.clear();
		this.regexInstances.clear();
		this.ne.index.clear();
	}
	
	public static void main (String [] args) {
		
		//org.apache.log4j.BasicConfigurator.configure();
		org.apache.log4j.PropertyConfigurator.configure("/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/plainlog4jconfig.txt");
		org.apache.log4j.Logger.getRootLogger().setLevel(Level.INFO); 
				
		
		Logger jenaFileManagerLogger = Logger.getLogger("com.hp.hpl.jena");
		jenaFileManagerLogger.setLevel(Level.OFF);
		
		
		
		Logger pokaLogger = Logger.getLogger("fi.seco.semweb.service.poka");
		pokaLogger.setLevel(Level.INFO);
		GeneralExtractor skytsen = new GeneralExtractor();
//		skytsen.setDocument("http://haku.kansallisbiografia.fi/FMPro?-db=kbnet.fp5&-format=kb%2fkbartikkeli.htm&-lay=www&-sortfield=lajittelukentt%e4&zz=zet5rr342uu5556v&-max=2147483647&-recid=32968&-find=");
		skytsen.setInputString("Petri Possulla oli tänään nimipäivä. Lahjaksi hän sai mopon. Petri Possu Harley Davidson on raju pyörä.");
		System.out.println(skytsen.getDocument());
		
//		skytsen.addRegex("[1-2][0-9][0-9][0-9]", "http://yso.fi/YSO#vuosi");
//		skytsen.addRegex("\\d(\\d)?\\.\\d(\\d)?\\.\\d\\d\\d\\d","http://yso.fi/YSO#päivämäärät");
		
		skytsen.addNames();
//		skytsen.addPlaces(GeneralExtractor.UPNODES, false, true);		
		skytsen.addYsoConcepts(GeneralExtractor.CONCEPTNODES, true, true);
//		skytsen.addRegexConcepts(GeneralExtractor.NUMERALNODES, false);
//		skytsen.addVerbs(false, true);
		System.out.println(skytsen.getDocument());

	}

}