/**
 * SAX-Dokumenttiparseri, parsii fdg:n ulostulon Pokan muotoon.
 * Nykyinen Poka-muoto (12-07-06) sisältää kaikki fdg:n kehittelemät
 * tagit. Outputissa seuraavia eroja Fdg:n outputtiin:
 * -Fdg:n geneerinen token-tagi korvataan sanan "syntaktista" tyyppiä
 *  kuvaavalla tagilla: 
 *  UP -- UPPER
 *  SY -- SYMBOL
 *  VB -- VERB
 *  NM -- NUMERAL
 *  AD -- ADJECTIVE
 *  PN -- PRONOUN
 *  BS -- BASE (substantiivit+muuta, jotka eivät ala isolla alkukirjaimella)
 *  OT -- OTHER (jossain määrin merkityksettömät sanaluokat) 
 *  (Pokan dokumenttirakenteeseen merkataan löydetyt käsitteet
 *   muuttamalla solmun tyyppi ontologiaa / työkalua edustavalla
 *   solmutyypillä:
 *   C -- Concept (YSO)
 *   PL -- PLACE (Kulsa paikat)
 *   V  -- Verbiontologia
 *   R  -- täsmännyt REGEX
 *   NA -- NAME (nimentunnistimen tunnistamat solmut)
 *   --> HUOM! Jos tarvetta, solmun tyyppejä ei välttämätön 
 *       merkata, muutos tulossa joskus. Jos solmuja 
 *       ei merkata, käsite ei muutu 'löydetyksi', vaan
 *       voi täsmätä useampaan kuin yhteen ontologiaan / työkaluun.
 *  ) 
 *  -fdg:ssä indeksointi on juokseva numero:
 *   1: lauseen alkuun
 *   2: lauseen 1. sana
 *   3: lauseen 2. sana ...
 *   4: 2. lauseen alku
 *   5: 2. lauseen 1. sana
 *   
 *  -pokassa 2 juoksevaa numeroa, lauseille ja tokeneille omat:
 *   1: lauseen alku
 *   1: 1. lauseen 1. token (= 1.1)
 *   2: 1. lauseen 2. token (= 1.2)
 *   2: 2. lauseen alku
 *   3: 2. lauseen 1. token (=2.1) ... 
 * -
 */

package fi.seco.semweb.service.poka;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.helpers.*;
import org.xml.sax.*;

//import fi.seco.semweb.service.poka.frame.FrameFinder;
import fi.seco.semweb.service.poka.util.XFDGConnect;


import java.io.IOException;

public class PokaDoctor extends DefaultHandler {
	private static final Log log = LogFactory.getLog(PokaDoctor.class);
		
	//REGEXIT
private final String PUNCT     = "((\\p{Punct}|»|¿|Ã|\\u00A0|¶).*)";
private final String UCASE     = "\\p{javaUpperCase}(\\p{javaLowerCase}|å|ä|ö|\\.).*";
private final String ISVERB    = "\\p{javaWhitespace}*V\\p{javaWhitespace}?.*";
private final String ISPRONOUN = "\\p{javaWhitespace}*PRON\\p{javaWhitespace}?.*";
private final String ISNUM     = "\\p{javaWhitespace}*NUM\\p{javaWhitespace}?.*";
//ISADJ: A, ei ADV, kysymysmerkki poistettu jälkimmäisen whitespacen perästä
private final String ISADJ     = "\\p{javaWhitespace}*A\\p{javaWhitespace}.*"; 
private final String ISBASE     = "(&lt;\\?&gt;)?\\p{javaWhitespace}*N\\p{javaWhitespace}?.*";
	//VAKIOT
	private final int BASE = 1,
					  VERB = 2,
					  PRONOUN = 3,
			          NUMERAL = 4,
			          ADJECTIVE = 5,
			          OTHER = 6,
			          UPPER = 7,
			          PUNCTUATION = 8;
	
	
	
	private int currentSentence;
	
	private int tokenType,
				tagType,
				tokenCounter;
	private int sentenceCount;
	private StringBuffer result;
	
	private final int TEXTTAG = 1,
				LEMMATAG = 2,
				MORPHOTAG = 3,
				SYNTAXTAG = 4,
				DEPENDTAG = 5,
				BYPASS = -1;
	
	private final int TOKENSTARTNUMBA = 1;
	
	private boolean checkMorpho,
					dependFound,
	                	getNextToken;
	
	private String tokenText,
				   tokenLemma,
				   tokenDepend,
				   tokenMorpho,
				   tokenSyntax;
	
	private int dependID;
	private String dependTags;
	
	private boolean morphoSet,
					syntaxSet;
	
    private XFDGConnect fdg;
	
    public PokaDoctor(String filename) {
    	this(filename, true);
    }
    
	public PokaDoctor(String filename, boolean isUri) {
		super();

		this.tokenType = -1;
		this.tagType = -1;
		this.tokenCounter = TOKENSTARTNUMBA;
		
		
		this.result = new StringBuffer();
		this.checkMorpho = false;
		this.dependFound = false;
		
		this.getNextToken = false;	
		this.sentenceCount = 0;

		

		try {
			this.fdg = new XFDGConnect(filename, isUri);
			InputSource is = fdg.getFDG();
			System.out.print("PokaDoc / luodaan rakenne...");
			XMLReader xr = XMLReaderFactory.createXMLReader();
			xr.setContentHandler(this);
			xr.setErrorHandler(this);
			xr.parse(is);
			System.out.print("...ok!\n");
		} catch(SAXParseException e) {
			System.err.println("POKADOC2 / SAXPARSE: "+e);
			//System.exit(0);
		} catch(SAXException e) { //TODO: IOEXP, SAXEXP
			System.err.println("POKADOC2 / SAXEXP: "+e);
			//System.exit(0);
		} catch(IOException e) {
			System.err.println("POKADOC2 / IOEXP: "+ e);
			//System.exit(0);
		} 
			
	}	
	
	
	public PokaDoctor() {
		super();		
		this.tokenType = -1;
		this.tagType = -1;
		
		this.result = new StringBuffer();
		this.checkMorpho = false;

		this.getNextToken = false;	
		this.sentenceCount = 0;

	}	
	
	
    public void startDocument () {
    		result.append("\n<A>\n");
    }

    public void endDocument () {
    		result.append("</A>");   		
    }
	
	
    public void startElement (String uri, String name,
		      String qName, Attributes atts) {
    		
    		log.debug("START: <" +qName+">");
    		if ("".equals (uri)) {
    			
			if(qName.equals("sentence")) {
				String temp = atts.getValue(0);
				temp =  temp.substring(temp.indexOf(':')+1);
				this.currentSentence = Integer.parseInt(temp);
								
				this.sentenceCount++;		
				result.append("<S no=\""+sentenceCount+"\">");
			} else if(qName.equals("paragraph")) {			
				this.sentenceCount++;		
				result.append("<S no=\""+sentenceCount+"\">\n\t<P /></S>\n\n");
			} else if(qName.equals("token")) {
				result.append("\n\t");
				this.checkMorpho = false;
				this.getNextToken = false;
				
				this.tokenText = "";
				this.tokenLemma = "";
				this.tokenDepend = "";
				this.tokenMorpho = "";
				this.tokenSyntax = "";
				this.morphoSet = false;
				this.syntaxSet = false;
				this.dependFound = false;
 				
    			} else if(this.getNextToken) {
    				return;
    			} else if(qName.equals("text")) {		
    				this.tagType = TEXTTAG;
    			} else if(qName.equals("lemma")) {
    				this.tagType = LEMMATAG;
    			} else if(qName.equals("depend")) {
    				this.tagType = DEPENDTAG;
    				String temp = atts.getValue(0);
    				temp =  temp.substring(temp.indexOf(':')+1);
    				this.dependID = (Integer.parseInt(temp))-this.currentSentence; //oli : -1
    				if(dependID == 0) {
    					dependID = -1; // main: -viittaus
    				}
    				
    			} else if(qName.equals("morpho")) {
    				if(this.morphoSet) {
    					return;
    				}
    				this.tagType = MORPHOTAG;
    				if(tokenMorpho.length() > 0) { //jos useita tageja, otetaan vaan 1.
    					morphoSet = true;
    				}
    			} else if(qName.equals("syntax")) {
    	    		//4.3.06: syntax mukaan kaikkiin  	
//    				if(this.checkMorpho||
    				if(this.syntaxSet) {
    					return;
    				}
    				this.tagType = SYNTAXTAG;
    				if(tokenSyntax.length() > 0) { //jos useita tageja, otetaan vaan 1.
    					syntaxSet = true;
    				}
    				
    			}  else {
    				this.tagType = BYPASS; 
    			}
    		} else
    			System.out.println("Start element: {" + uri + "}" + name);
    	}

    public void endElement (String uri, String name, String qName) {
//  		System.out.println("END  : </"+qName+">");
    	
    		if ("".equals (uri)) {
        		       		    			        			
        		if(qName.equals("sentence")) {
				result.append("</S>\n\n");
//				MUUTOS 15-06-06: 0-->1 (TOKENIEN NUMEROINTI!)
				this.tokenCounter = TOKENSTARTNUMBA; 
				
        		} else if(qName.equals("token")) {		
        			        			
				if(this.tokenText.length()==0) { //tuleeko raskas?
					this.tokenType = PUNCTUATION; //TODO keksi jotain parempaa.
					this.tokenText = "_";		 //oletustekstiksi "_" 
				}
				
				if(this.dependFound) {
					this.dependTags = "<DEP h=\"" + this.dependID +
	                  "\">" + this.tokenDepend +"</DEP>";
				}
				
				switch(tokenType) {			
				
				case BASE:
					result.append("<BS no=\""+tokenCounter+"\"><T>"+tokenText+"</T>"+
							      "<L>"+tokenLemma+"</L>"+
							      "<MO>" +tokenMorpho+ "</MO>"+
						          "<SYN>"+tokenSyntax+"</SYN>" +
							      this.dependTags+ "</BS>");
					break;
					
				case VERB:
					result.append("<VB no=\""+tokenCounter+"\"><T>"+tokenText+ "</T>" +
							          "<L>"+tokenLemma+"</L>" +
							          "<MO>" +tokenMorpho+ "</MO>"+
							          "<SYN>"+tokenSyntax+"</SYN>" +
							          this.dependTags+ "</VB>");
					break;
					
				case PRONOUN:
					result.append("<PN no=\""+tokenCounter+"\"><T>"+tokenText+"</T>" +
											"<L>"+tokenLemma+"</L>" +
											this.dependTags+ "</PN>");
					break;
					
				case NUMERAL:
					result.append("<NM no=\""+tokenCounter+"\"><T>"+tokenText+"</T>" +
												"<L>"+tokenLemma+"</L>" +
												this.dependTags+ "</NM>");
					break;
					
				case ADJECTIVE:
					result.append("<AD no=\""+tokenCounter+"\"><T>"+tokenText+ "</T>"+
							          "<L>"+tokenLemma+"</L>" +
							          "<MO>" +tokenMorpho+ "</MO>"+
							          "<SYN>"+tokenSyntax+"</SYN>" +
							          this.dependTags+ "</AD>");
					break;
					
				case OTHER:
					result.append("<OT no=\""+tokenCounter+"\"><T>"+tokenText+"</T>" +
											"<L>"+tokenLemma+"</L>" +
											this.dependTags+ "</OT>");
					break;				
					
				case UPPER:
					result.append("<UP no=\""+tokenCounter+"\"><T>"  +tokenText+   "</T>"+
							          "<L>"  +tokenLemma+  "</L>"+
							          "<MO>" +tokenMorpho+ "</MO>"+
							          "<SYN>"+tokenSyntax+"</SYN>" +
							          this.dependTags+ "</UP>");
					break;			

				case PUNCTUATION:
//TODO: laiskuudesta johtuvaa toisteisuutta (poista jos tarvii)
					result.append("<SY no=\""+tokenCounter+"\"><T>"+tokenText+"</T>" +
											"<L>"+tokenText+"</L>" +
											this.dependTags+ "</SY>");
					break;
				
				}
				this.tokenCounter++;
				this.dependTags = "";
			}      
		} else
			System.out.println("Start element: {" + uri + "}" + name);
    }
    
    
    public void characters (char ch[], int start, int length) {
    	
    		log.debug("CHARACTERS");
    	
    		if(this.getNextToken) {//odotetaan seuraavaa tokenia
    			return; 
    		}
    	
    		String temp = "";
    		for (int i = start; i < start + length; i++) {

    			switch (ch[i]) {
    			
    			case '&':
    				temp += "&amp;";
    				break;
    			case '>':
    				temp += "&gt;";
    				break;
    			case '<':
    				temp += "&lt;";
    				break;
    			case '\'':
    				temp += "'"; 
    				break;
    			case '\\':	
    				temp += "\\\\";
 				break;
    			case '"':
    				temp += "\"";
		        break;    
    			case '\n':
    				temp += "\\n";
    				break;
    				
    			case '\r':
    				temp += "\\r";
    				break;

    			case '\t':
    				temp += "\\t";
    				break;

    			default:
    				temp += ch[i];
    				break;
    			}
    		}

    		switch(tagType) {
    		
    		case(TEXTTAG):
    			
    			log.debug("TEXTTAG");
    			
    			if(temp.matches(PUNCT)) {
    				    				
    				this.getNextToken = true;
    				this.tokenType = PUNCTUATION;
    				
    			} else if(temp.matches(UCASE)) {

    				log.debug("--> UPPERCASE");
    				this.tokenType = UPPER;    				
    				
    	 		} else {
    	 			
    				this.checkMorpho = true;

    	 		}
    			this.tokenText += temp;
    			break;
    		
    		case(LEMMATAG):

    			log.debug("LEMMATAG");
    			
    			this.tokenLemma += temp;
    			break;
    		
    		case(DEPENDTAG):
    			
    			log.debug("DEPENDTAG");
    			
    			this.tokenDepend += temp;
    			this.dependFound = true;
    			break;	
    			
    		case(SYNTAXTAG):

    			log.debug("SYNTAXTAG");
    			
    			if(this.syntaxSet) {
    				return;
    			}   			
    			this.tokenSyntax += temp;
    			break;
    			
    		case(MORPHOTAG):
    			
    			log.debug("MORPHOTAG");
    		
    			if(this.morphoSet) {
    				//TODO: toinen morpho mukaan päättelyyn!!! (28.2.06)
    				//Luultavasti verbi jää liian usein välistä.
    				//toisaalta, tokenin tyyppiä ei voi täysin päätellä.
    				return;
    			}  			
//TODO: checkMorpho pois, textTagin UP & Punct morphon alle tarkistukseen (31-07-2006)
// tietyt Uppercase-sanat on syytä palauttaa toiseen muotoon, esim. 'Hän' pronominiksi! 
    			
    			if(this.checkMorpho) {
    				this.tokenMorpho += temp;
    				tokenType = this.getTokenType(tokenMorpho, tokenText);
    			} else {
    				this.tokenMorpho += temp;
    			}   	
    			break;
    		} 		
   		
    }

    private int getTokenType(String morpho, String tokenText) {
    	
    	//TODO: pähkäilyn arvoinen asia, pitäisikö postpositio (PSP)
    	//lyödä mukaan BS-solmuun? Ilmaisussa 'vuoden ajan' aika
    	//on postpositio
    	
   		//7.2. poistettu NUMista numeroalkuiset ilmaukset
   	    // esim 4h-kerho ei numeraali, vaan BS
       	//if(tokenText.matches("\\d.*") ||
    		if(morpho.matches(ISNUM)) {
    			return this.NUMERAL;    			
    		} else if(morpho.matches(this.ISVERB)) {
    			return this.VERB;
    		} else if(morpho.matches(this.ISPRONOUN)) {
    			return this.PRONOUN;
    		} else if(morpho.matches(this.ISBASE)) {
      			return this.BASE;
    		} else if(morpho.matches(this.ISADJ)) {
    			return this.ADJECTIVE;
    		} else {
    			return this.OTHER;
    		}
    }
    
    public String getResult() {
    		return this.result.toString();
    }
    
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		PokaDoctor handleri = new PokaDoctor("/home/alm/aaparser/text/Gallenkallela.html");
		System.out.println(handleri.getResult());
		
/*		poka.XFDGConnect mau = new poka.XFDGConnect("/home/alm/termo/penkki/sax.txt");
		InputSource is = mau.getFDG();

		BufferedReader in
		   = new BufferedReader(is.getCharacterStream());
		String some;
		StringBuffer buf = new StringBuffer();
		while((some = in.readLine()) != null) {
			buf.append(some+"\n");
		}
		System.out.println("FDG:\n"+buf.toString());
		in.close();
*/	
		
/*		
		XFDGConnect mau = new poka.XFDGConnect("/home/alm/aaparser/text/AckteAinoHTML.txt");
		InputSource is = mau.getFDG();
		
		XMLReader xr = XMLReaderFactory.createXMLReader();
		PokaDoc2 handler = new PokaDoc2();
		xr.setContentHandler(handler);
		xr.setErrorHandler(handler);
		xr.parse(is);
		
		System.out.println(handler.getResult());
*/		
	}

}
