/**
 * Termistön rakentaja. Luo ontologiasta termitiedoston.
 * Seuraavat metodit olennaisia:
 * 	-createFrameVerbTerms: luo frameverbitermistön
 *  -createOpasTerms: luo Oppaan instansseista termistön (ei käytössä?)
 *  -createPlaceTerms: kulsan paikoista termitiedosto
 *  -createYsoTerms: luo Ysosta termitiedoston, poistaa ruotsinkieliset labelit
 *  -createYSOSubGraphTerms: luo Yson tietyn luokan aliluokista termistön
 *  
 *  Myös jotakin kummallisia apumetodeja löytyy TODO: täytyy selvitellä joskus.
 *  Osa luultavasti Harrin D-palikan käytössä.
 * 
 */

package fi.seco.semweb.service.poka;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.xml.sax.InputSource;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdql.Query;
import com.hp.hpl.jena.rdql.QueryEngine;
import com.hp.hpl.jena.rdql.QueryExecution;
import com.hp.hpl.jena.rdql.QueryResults;
import com.hp.hpl.jena.rdql.ResultBinding;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import fi.seco.semweb.service.poka.util.LemmaLister;
import fi.seco.semweb.service.poka.util.LemmaParser;
import fi.seco.semweb.service.poka.util.TrieIndex;
import fi.seco.semweb.service.poka.util.XFDGConnect;

public class Termo {
	private static final Log log = LogFactory.getLog(Termo.class);

/**
 * HUOM! TERMIT VOISI TOTEUTTAA MIELUUMMIN HASHMAPPINA!
 * N*ALTLABEL ---> PREFLABEL 
 *
 * PREFLABEL --> YSO:CONCEPT (URI)
 * 
 * REGEX EI KOSKAAN PREFLABEL, VAAN AINA ALTLABEL
 *
 * Teoriassa olisi mahdollista generoida
 * tietyn käsitteen termille altlabel pätkimällä
 * käsitettä pienempiin osiin (sillä oletuksella,
 * että lyhempi muoto ei ole olemassa...
 * 
 * Esim: tekstissä käsite komennus, YSOsta 
 * löytyy vain työkomennukset, mutta ei käsitettä komennus:
 * mäpätään komennus työkomennusten yläkäsitteeseen (järjestetty tapahtuma)
 * TAI samaan solmuun.
 * 
 * Jos ronkitaan suoraan varsinaisia käsitteitä, olisi
 * mahdollista luoda uusi käsite 'komennus', joka
 * automaattisesti populoidaan ontologiaan joko
 * 1) työkomennuksen sisarkäsitteeksi TAI
 * 2) työkomennuksen paikalle siten, että
 * työkomennus on komennuksen lapsi
 * (ja komennus nyt järjestetyn tapahtuman lapsi)
 *
 * Vastaavia mäppäyksiä:
 * 
 * valheenpaljastus --> paljastus --> luominen
 *
 * tarkoituksenmukaisuusharkinta --> harkinta --> menetelmät
 * 
 * lupamenettely --> menettely --> menetelmät
 * 
 * liikesalaisuus --> salaisuus --> tieto
 *
 * laskentatoimi --> toimi --> erikoisala
 *
 * Sanoja, joista ei tehtäisi uusia mäppäyksiä:
 * *tiede, *mekaniikka
 */


/**
 * propertyt: prefLabel, altLabel, plural?
 * attribuutti: lang
 * 
 * Mahdollinen monikko: riippuuko kielestä? tarvitaanko?
 *
 * <oma:omaTerm rdf:ID="toiminta">
 * <oma:prefLabel rdf:datatype="http://www.w3.org/2001/XMLSchema#string" xml:lang="fi">
 * toiminta </oma:prefLabel>
 *
 * <oma:altLabel rdf:datatype="http://www.w3.org/2001/XMLSchema#string" xml:lang="fi">
 * jotakin </oma:prefLabel>
 *
 * <oma:prefLabel rdf:datatype="http://www.w3.org/2001/XMLSchema#string" xml:lang="en">
 * action </oma:prefLabel>
 *
 * </oma:omaTerm>
 *
 *
 * 7.12. päivityksiä:
 * kokeillaan pref/altlabel -erottelun tilalle
 * termlabelia ilman kielimäärittelyä. kaikki labelit tasa-arvoisia,
 * käyttäjälle näytettävä label löytyy varsinaisesta YSOsta.
 * 
 *
 *
 * Rakennesuunnittelu:
 * luokka, jolla luodaan uusi terminologia
 * -oletuskieli parametrina
 * -preflabelien automaattinen generointi
 * -tyhjän terminologian generointi
 * -generointi useista lähteistä?
 * -monikkomuodon pois parsinta toimii tällä hetkellä vain suomenkielisille ontologioille (connexor-fi)
 * 
 * -metodi altlabelin lisäämiselle
 * -altlabelin poisto
 * -kaikkien muuttamiselle
 * -mallin tulostus
 *
 * LISÄYS 7.12: 
 * Termitiedosto kieliriippuvaiseksi, so. ei lang-määrettä generoitaviin termeihin.
 * --> PokaTermsToHash: jokaisesta kielitiedostosta oma hash
 *
 */

	private final String ABBR = "oma";

	public static final String DEFAULTNAMESPACE = "http://www.seco.hut.fi/ns/2005/10/alm#";
	private String termNameSpace; 

	
	public static final String DEFAULTINPUTFILE = "/home/alm/ontologies/ontologies/poka/YSO-All.owl";
	public static final String DEFAULTOUTPUTFILE = "/home/alm/termo/Terms-all.owl";
	public static final String ANIMALS =           "/home/alm/termo/penkki/Terms-animal.rdf";
	public static final String BUGS    =           "/home/alm/termo/penkki/Terms-bug.rdf";
	public static final String RELATIVEPLACE =     "/home/alm/termo/penkki/Terms-place-relative.rdf";
	
	public static final String TERMLABEL = "termlabel";

	private final String YSONAMESPACE = "http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#";

	public static final String YSOBASE = "http://yso.fi/YSO#";

    private final String YSOLABEL = "label";

//YSOn uudet labelit (15.12.)
	private final String PLABEL = "prefLabel";
	private final String ALABEL = "altLabel";
	private final String ACTUALITYLABEL = "actuality";

	//YSO: get subjects with this property
	private final String YSOSUBJIDSPACE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    //YSO: LOCAL
    private final String YSOSUBJIDLOCAL = "type";

    
    //YSO --käsitteet
    
//	private Vector <Model> inputOntologies;

	private Model terms;

	//termiavaruuden ominaisuudet 
	private Property pLabel;
	private Property aLabel;
	//private Property plural = terms.createProperty("http://www.seco.hut.fi/ns/2005/10/alm#","plural");

	//Connexor FDG -yhteys sanojen lemmatisointiin
	private XFDGConnect fdg;

	private LemmaParser lp;

	public Termo(String termNameSpace) {
		
		this.termNameSpace = termNameSpace;
		
//		inputOntologies = new Vector <Model>();
		terms = ModelFactory.createDefaultModel();

		//nimiavaruuden lyhenne
		terms.setNsPrefix(ABBR,termNameSpace);

		this.lp = new LemmaParser();
//7.12 --> PREFERREDLABEL --> TERMLABEL
		pLabel = terms.createProperty(termNameSpace, TERMLABEL);
//7.12 --> aLabel pois
//		aLabel = terms.createProperty(TERMNAMESPACE,ALTERNATIVELABEL);
	}

/**
 * Luo paikkatermistön kulsan Paikka.owlista
 * @param filename tiedoston nimi
 * @param encoding enkoodaus
 * @return palauttaa true jos homma onnistui
 */	
	
	public boolean createPlaceTerms(String filename, String encoding) {

		terms.setNsPrefix("yso","http://www.yso.fi/kulttuuriSampo/paikat#");
		
		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

		Model m = ModelFactory.createDefaultModel();
		fdg = new XFDGConnect();
		
		try {
			// read the RDF/XML file
			//m.read(in, "");
			m.read(new InputStreamReader(in, encoding), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
            return false;
        }
		
		Property subjProp = m.getProperty("http://www.yso.fi/kulttuuriSampo/paikat#", "nimi");

		// uusi (prefLabel & altLabel YSOssa
		Property placeNames = m.getProperty(YSOBASE,PLABEL);

		String placeName;
		StringBuffer allLemmas = new StringBuffer(10000);
		
		Resource temp, current;
//		resource iterator		
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);
		int kierros =0, lis = 1;
		while(concepts.hasNext()) {
						
			temp = concepts.nextResource();
			
			
			placeName = temp.getProperty(subjProp).getLiteral().getString();			
						
			log.debug("uri on      : "+temp.getURI());
			log.debug("placename on: "+placeName);
			allLemmas.append(placeName.replaceAll("\\?|_"," ")+"_ ");
			
		}
		fdg.setInput(allLemmas.toString());
		log.info("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		log.info("...ok!");

		LemmaLister ll = new LemmaLister();
		log.info("Parsitaan...");
		ll.parse(is);
		log.info("...ok!");

		ArrayList <String> tempL = ll.getResult();
		log.info("--> listan koko: " + tempL.size()+" <--");
//		System.out.println(tempL.toString());
	
		
		concepts = m.listSubjectsWithProperty(subjProp);
		java.util.Iterator <String> ii = tempL.iterator();		
		while(concepts.hasNext()) {
			String lemmaName = ii.next();
			temp = concepts.nextResource();
			log.info("ii: " +lemmaName+" <---> "+temp.getProperty(subjProp).getLiteral().getString());
			
			current = terms.createResource(temp.getURI());
			current.addProperty(pLabel, terms.createLiteral(lemmaName));
						
		}
		return true;
	}
	
	
	public boolean createOpasTerms(String filename, String encoding) {

//		terms.setNsPrefix("j.0","http://yso.fi/YSO#");
//		terms.setNsPrefix("j.1","http://yso.fi/opas#");
		
		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

		Model m = ModelFactory.createDefaultModel();
		fdg = new XFDGConnect();
		
		try {
			m.read(new InputStreamReader(in, encoding), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
            return false;
        }
		
		Property subjProp = m.getProperty("http://www.w3.org/2000/01/rdf-schema#", "label");
		Property typeProp = m.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#", "type");
		
		
		
//		Property opasTermName = m.getProperty(YSOBASE,PLABEL);

		String label, type;
		ArrayList <String> types = new ArrayList<String>();
		StringBuffer allLemmas = new StringBuffer(10000);
		
		Resource temp, current;
		
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);
		while(concepts.hasNext()) {
						
			temp = concepts.nextResource();
			label = temp.getProperty(subjProp).getLiteral().getString();			
			
			
			System.out.println("---------------------");
			System.out.println("uri on     : "+temp.getURI());
			System.out.println("label on   : "+label);
			RDFNode instType = temp.getProperty(typeProp).getObject();
			if(instType.isLiteral()) {
				type = temp.getProperty(typeProp).getLiteral().getString();
			} else {//resurssi
				type = temp.getProperty(typeProp).getResource().getURI();
			}
			System.out.println("tyyppi on  : " +type);
			allLemmas.append(label.replaceAll("\\?|_"," ")+"_ ");
			types.add(type);
		}
		
		fdg.setInput(allLemmas.toString());
		System.out.print("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		System.out.println("...ok!");
/*
		try {
			System.out.println(fdg.sourceToString(is));
		} catch(Exception e) {
			System.exit(0);
		}
*/ 
		LemmaLister ll = new LemmaLister();
		System.out.print("Parsitaan...");
		ll.parse(is);
		System.out.println("...ok!");

		ArrayList <String> tempL = ll.getResult();
		System.out.println("--> listan koko: " + tempL.size()+" <--");
//		System.out.println(tempL.toString());
	
		
		concepts = m.listSubjectsWithProperty(subjProp);
		
		//luodaan oma:type -literaaliominaisuus, joka sisältää urin, rumaa?
		typeProp = terms.createProperty(Termo.DEFAULTNAMESPACE, "type");
		
		java.util.Iterator <String> typeIt = types.iterator();		
		java.util.Iterator <String> ii = tempL.iterator();
		
//HUOM! vain yksi label per instanssi oletuksena!
		
		while(concepts.hasNext()) { 
//			System.out.println("ii: " +ii.next()+" <---> "+concepts.nextResource().getProperty(subjProp).getLiteral().getString());
			temp = concepts.nextResource();
			current = terms.createResource(temp.getURI());
			current.addProperty(pLabel, terms.createLiteral(ii.next()));
			current.addProperty(typeProp, typeIt.next());
						
		}
		return true;
	}
	
	public boolean createFrameVerbTerms(String filename, String encoding) {
		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			log.info("File '" + filename + "' does not exist!");
			return false;
		}
	    
		Model m = ModelFactory.createDefaultModel();
		fdg = new XFDGConnect();
		
		try {
			m.read(new InputStreamReader(in, encoding), "");
		} catch(Exception e) {
			log.info("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
            return false;
        }
		
		Property subjProp = m.getProperty("http://www.seco.hut.fi/frame/ns/2006/05/alm#", "verbLabel");
		
		int kasitelaskuri = 0;
		int labelLaskuri = 0;
		
		String label;
		ArrayList <String> types = new ArrayList<String>();
		StringBuffer allLemmas = new StringBuffer(10000);
		
		Resource temp, current;
		StmtIterator stmtI;
		
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);
		Statement currentStmt = null;
		log.debug("Läpikäydään labelit");
		
		
		while(concepts.hasNext()) {
					
			temp = concepts.nextResource();
			log.debug("uri on     : "+temp.getURI());
			stmtI = temp.listProperties(subjProp);
			if(stmtI.hasNext()) {
				kasitelaskuri++;
			} 
			while(stmtI.hasNext()) {
				labelLaskuri++;		
				try {
					currentStmt = stmtI.nextStatement();
				} catch (NoSuchElementException e) {
					log.debug("No such element: " + e);
				}
				label = this.getLiteral(currentStmt);
				log.debug("Label on       : '"+label+"'");
//				log.debug("Käsitelty label: '"+label.replaceAll("\\?|_"," ")+"'");
				allLemmas.append(label.replaceAll("\\?|_"," ")+"_ ");
			}									
		}

		log.info("Käsitteitä: "+kasitelaskuri);
		log.info("Labeleita : "+labelLaskuri);
		
		fdg.setInput(allLemmas.toString());
		log.info("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		log.info("...ok!");

		LemmaLister ll = new LemmaLister();
		log.info("Parsitaan...");
		ll.parse(is);
		log.info("...ok!");

		ArrayList <String> tempL = ll.getResult();
		log.info("--> listan koko: " + tempL.size()+" <--");
		
		java.util.Iterator <String> ii = tempL.iterator();
		
		concepts = m.listSubjectsWithProperty(subjProp);	
		
		while(concepts.hasNext()) {
			
			temp = concepts.nextResource();
			log.debug("Lisätään uri: "+temp.getURI());
			current = terms.createResource(temp.getURI());
			stmtI = temp.listProperties(subjProp);
			
			while(stmtI.hasNext()) {
				String lemmatizedLabel = ii.next();
				log.debug("label lemmauksen jälkeen: "+lemmatizedLabel);
				currentStmt = stmtI.nextStatement();
				current.addProperty(pLabel, terms.createLiteral(lemmatizedLabel));
			}
					
		}
		
//		this.frameVerbFinisher(terms);
		
		return true;
		
	}

	
	/**
	 * Lisää pluskvamperfektin & perfektin termistöön, eli
	 * poimii uriin liittyvän verbLabelin, lisää eteen 'olla ' ja
	 * lisää toisen vastaavan: 
	 *   uri: pirstoutuminen
	 *   verbLabel: pirstoutua
	 *   lisättävä verbLabel: 'olla pirstoutua', joka
	 *   täsmää monisanaisena käsitteenä esimerkiksi muotoihin
	 *   'oli pirstoutunut' ja 'on pirstoutunut'
	 */
	private void frameVerbFinisher(Model terms) {

		ResIterator concepts = terms.listSubjectsWithProperty(pLabel);

		Statement currentStmt = null;
		String label;
		log.debug("lisätään perfektit / pluskvamperfektit");
		Resource temp;
		StmtIterator stmtI;
		
		ArrayList <String> labels = new ArrayList<String>();
		
		final String OLEMINEN = "olla ";
		
		while(concepts.hasNext()) {
				
			temp = concepts.nextResource();
			log.debug("uri on     : "+temp.getURI());
			stmtI = temp.listProperties(pLabel);

			labels.clear();
			while(stmtI.hasNext()) {
				
				try {
					currentStmt = stmtI.nextStatement();
				} catch (NoSuchElementException e) {
					log.debug("No such element: " + e);
				}
				System.out.println(currentStmt.getString());
				label = this.getLiteral(currentStmt);
				//log.debug("Label on       : '"+label+"'");
				labels.add(label);
				
			}
			for(String s:labels) { //jos whilen sisällä, iteraatio ottaa mukaan lisättävät
				log.debug("Lisätään       : '"+s+"'");
				temp.addProperty(pLabel, terms.createLiteral(OLEMINEN+s));
			}
			
		}
		
	}
	
	private OntModel addModel(OntModel m, String url, String enc) {
		InputStream in = null;
		in = FileManager.get().open( url );

		if (in == null) {
			System.out.println("File '" + url + "' does not exist!");
			return null;
		}
		
		OntModel neu = ModelFactory.createOntologyModel();

		System.out.print("Luetaan ysa-groups..");
		try {
			neu.read(new InputStreamReader(in, enc), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
		}
		System.out.println(".ok!");
//		neu.addLoadedImport("http://yso.fi/ysa-groups");
		m.add(neu);
		return m;
	}
	
/**
 * LUO YSOn tietyn luokan aliluokista termistön
 */
	public boolean createYSOSubGraphTerms(String filename, String encoding, String classURI) {

		terms.setNsPrefix("yso","http://yso.fi/YSO#");
		InputStream in = null;
		in = FileManager.get().open( filename );

		if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

//	    Model m = ModelFactory.createDefaultModel();
	    OntModel m = ModelFactory.createOntologyModel();
	    
	    
//	    m = this.addModel(m, "/home/alm/ontocvs/ontologies/YSO/YSA-Groups.owl", "UTF-8");
		System.out.print("Luetaan malli..");
		try {
			m.read(new InputStreamReader(in, encoding), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
        }
		System.out.println(".ok!");
		Property subjProp = m.getProperty(YSOSUBJIDSPACE, YSOSUBJIDLOCAL);

		// uusi (prefLabel & altLabel YSOssa
		Property pl = m.getProperty(YSOBASE,PLABEL);
		Property al = m.getProperty(YSOBASE, ALABEL);


		OntClass rr = m.getOntClass(classURI);
		StmtIterator stmtI;
		System.out.println("rr on: "+rr);

		int kasitelaskuri = 0;
		int labelLaskuri = 0;
		int suomiLabel = 0;
		int ruotsiLabel = 0;
		int muut = 0;
		
		fdg = new XFDGConnect();
		String before;
		StringBuffer allLemmas = new StringBuffer(5000);
		OntClass temp;
		Resource current;
		Statement currentStmt = null;
		
		ExtendedIterator ei = rr.listSubClasses();
		while(ei.hasNext()) {
			
			temp = (OntClass)ei.next();
			stmtI = temp.listProperties(pl); //prefLabel
			
			if(stmtI.hasNext()) {
				kasitelaskuri++;
			} 
			while(stmtI.hasNext()) {
				labelLaskuri++;				

				try {
					currentStmt = stmtI.nextStatement();
				} catch (NoSuchElementException e) {
					System.out.println("No such element: " + e);
				}					
				if(this.getLang(currentStmt).equals("se")) {
					ruotsiLabel++;
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
					suomiLabel++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				} else {
					muut++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				}
			
			}
			
			stmtI = temp.listProperties(al); //altLabel

			while(stmtI.hasNext()) {
				labelLaskuri++;		
				
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {
					ruotsiLabel++;
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
					suomiLabel++;				
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				} else {
					muut++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				}
									
			}					
		
		} //end subclass iteration
		
		System.out.println("------------------------------");
		System.out.println("Käsitteitä yhteensä: "+ kasitelaskuri);
		System.out.println("Labeleita yhteensä : "+ labelLaskuri);
		System.out.println("Suomenkielisiä     : "+ suomiLabel);
		System.out.println("Ruotsinkielisiä    : "+ ruotsiLabel);
		System.out.println("Määreettömiä       : "+ muut);
		System.out.println("Suomilabel+muut    : "+ (suomiLabel+muut));
		System.out.println("------------------------------");
		
		
		fdg.setInput(allLemmas.toString());
		System.out.print("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		System.out.println("...ok!");
		LemmaLister ll = new LemmaLister();
		System.out.print("Parsitaan...");
		ll.parse(is);
		System.out.println("...ok!");

		ArrayList <String> tempL = ll.getResult();
	
		System.out.println("--> listan koko: " + tempL.size()+" <--");
	
		java.util.Iterator <String> ii = tempL.iterator();
		
		ei = rr.listSubClasses();

		while(ei.hasNext()) {
			
			temp = (OntClass)ei.next();
			current = terms.createResource(temp.getURI());
			stmtI = temp.listProperties(pl);
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();				
				
				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi				
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				} else {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				}
			}
			
			stmtI = temp.listProperties(al);
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {								
				} else if(this.getLang(currentStmt).equals("fi")) {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				} else {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				}
									
			}
					
		}
		
		return true;		
		
	}
	
/**
 * Jos termien generointi aiotaan yleistää, pitäisi
 * tähän kyseiseen metodiin pistää parametrit, joilla
 * on mahdollista määrittää yleisessä tapauksessa,
 * mitkä resurssit annetusta ontologiasta ovat käsitteitä
 * ja mitkä kyseisten käsitteiden label-kenttiä.
 * (=Mikä tahansa ontologia ikinä ladataankin, niin...)
 *
 */
	public boolean createYsoTerms(String filename, String encoding) {

		//määritä nimiavaruus yso (TODO: yleisessä tapauksessa!)
		terms.setNsPrefix("yso","http://yso.fi/YSO#");

		//jos kieltä ei määritelty, kaiva kielitieto ontologiasta.
		//--> tällöin oletetaan että xml:lang on merkattu ontologiaan

		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

		Model m = ModelFactory.createDefaultModel();
	
		try {
			// read the RDF/XML file
			//m.read(in, "");
			m.read(new InputStreamReader(in, encoding), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
        }

		Property subjProp = m.getProperty(YSOSUBJIDSPACE, YSOSUBJIDLOCAL);

		// uusi (prefLabel & altLabel YSOssa
		Property pl = m.getProperty(YSOBASE,PLABEL);
		Property al = m.getProperty(YSOBASE, ALABEL);

		//resource iterator		
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);

		Resource temp, current;
		Statement currentStmt = null;
		StmtIterator stmtI;
		//ota käsittelyyn resurssit, joilla ominaisuutena rdf:type 


		fdg = new XFDGConnect();

		String before;

		int kasitelaskuri = 0;
		int labelLaskuri = 0;
		int suomiLabel = 0;
		int ruotsiLabel = 0;
		int muut = 0;
		
		StringBuffer allLemmas = new StringBuffer(390000);
/*
 * 3.2.: muutettu Termoa, siten, että 
 * kaikki labelit otetaan isoksi Stringiksi,
 * lemmataan, ja liitetään uudessa YSOn iteroinnissa
 * Termistön termeiksi. 
 * Syy tähän toimintaan on javanatiivin mahdollinen
 * bugi, lisätietoja täältä: http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6223635
 * 
 * (Toisaalta, nykyinen ratkaisu on myös huomattavasti nopeampi)
 */
		
		while(concepts.hasNext()) {

			temp = concepts.nextResource();

			//jos resurssilla ominaisuus yso:label (prefLabel) , kyseessä concept

			stmtI = temp.listProperties(pl); //prefLabel
			
			if(stmtI.hasNext()) {
				kasitelaskuri++;
			} 
			while(stmtI.hasNext()) {
				labelLaskuri++;				

				try {
					currentStmt = stmtI.nextStatement();
				} catch (NoSuchElementException e) {
					System.out.println("No such element: " + e);
				}					
				if(this.getLang(currentStmt).equals("se")) {
					ruotsiLabel++;
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
					suomiLabel++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				} else {
					muut++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				}
			
			}	
								
//--> 7.12. poistettu määre initLang!
/*
				current.addProperty(pLabel, terms.createLiteral(
							lemmatized, initLang));
*/

			stmtI = temp.listProperties(al); //altLabel

			while(stmtI.hasNext()) {
				labelLaskuri++;		
				
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {
					ruotsiLabel++;
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
					suomiLabel++;				
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				} else {
					muut++;
					before = this.getLiteral(currentStmt);
					allLemmas.append(before.replaceAll("_"," ")+"_ ");
				}
									
			}					
	
		}
		
		System.out.println("------------------------------");
		System.out.println("Käsitteitä yhteensä: "+ kasitelaskuri);
		System.out.println("Labeleita yhteensä : "+ labelLaskuri);
		System.out.println("Suomenkielisiä     : "+ suomiLabel);
		System.out.println("Ruotsinkielisiä    : "+ ruotsiLabel);
		System.out.println("Määreettömiä       : "+ muut);
		System.out.println("Suomilabel+muut    : "+ (suomiLabel+muut));
		System.out.println("------------------------------");
		
		
		fdg.setInput(allLemmas.toString());
		System.out.print("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		System.out.println("...ok!");
		LemmaLister ll = new LemmaLister();
		System.out.print("Parsitaan...");
		ll.parse(is);
		System.out.println("...ok!");

		ArrayList <String> tempL = ll.getResult();
	
		System.out.println("--> listan koko: " + tempL.size()+" <--");
	
		java.util.Iterator <String> ii = tempL.iterator();
		
		concepts = m.listSubjectsWithProperty(subjProp);

/*
		Alla kokeilupätkä koodia, jolla
		voi testata saman lemmatun muodon
		omaavien käsitteiden määrän
*/
		
/*
		HashSet <String> sameLabels = new HashSet<String>();
		ArrayList <String> sameLabelsArray = new ArrayList<String>();
		int samecounter = 0;
		String sii;
		while(ii.hasNext()) {
			sii = ii.next();
			if(sameLabels.contains(sii)) {
				samecounter++;
				sameLabelsArray.add(sii);
			}
			sameLabels.add(sii);	
		}
		System.out.println("samoja labeleita: "+samecounter);
		for(String s: sameLabelsArray) {
			System.out.println(s);
		}
*/		
		while(concepts.hasNext()) {
			
			temp = concepts.nextResource();
			current = terms.createResource(temp.getURI());
			stmtI = temp.listProperties(pl);
						
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();				
				
				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi				
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				} else {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				}
			}
			
			stmtI = temp.listProperties(al);
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {								
				} else if(this.getLang(currentStmt).equals("fi")) {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				} else {
					current.addProperty(pLabel, terms.createLiteral(ii.next()));
				}
									
			}
					
		}
		
		return true;
	}
	
/**
 * Eristetään Muusasta Kulsa-henkilöt.
 * 
 * @param filename
 * @param encoding
 * @return
 */	
	
	public boolean extractKulsaPersons(String filename, String encoding) {
		return true;
	}
	
	
	/**
	 * 
	 * YSA-termien labeleiden eristys uusimmasta YSO-YSA-fuusiosta...
	 * (29.11.2006)
	 *
	 */
		public boolean createYSATerms(String filename, String encoding) {

			
			//YSO: get subjects with this property
			final String YSASKOSSUBJIDSPACE = "http://www.domain2.com#";

			
			//määritä nimiavaruus yso (TODO: yleisessä tapauksessa!)
			terms.setNsPrefix("yso","http://yso.fi/YSO#");

			//jos kieltä ei määritelty, kaiva kielitieto ontologiasta.
			//--> tällöin oletetaan että xml:lang on merkattu ontologiaan

			InputStream in = null;
			in = FileManager.get().open( filename );
		    if (in == null) {
				System.out.println("File '" + filename + "' does not exist!");
				return false;
			}

			Model m = ModelFactory.createDefaultModel();
		
			try {
				// read the RDF/XML file
				//m.read(in, "");
				m.read(new InputStreamReader(in, encoding), "");
			} catch(Exception e) {
				System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
	            e.printStackTrace();
	        }

			Property subjProp = m.getProperty(YSOSUBJIDSPACE, YSOSUBJIDLOCAL);

			// uusi (prefLabel & altLabel YSOssa
			Property pl = m.getProperty(YSASKOSSUBJIDSPACE,PLABEL);
			Property al = m.getProperty(YSASKOSSUBJIDSPACE, ALABEL);

			//resource iterator		
			ResIterator concepts = m.listSubjectsWithProperty(subjProp);

			Resource temp, current;
			Statement currentStmt = null;
			StmtIterator stmtI;
			//ota käsittelyyn resurssit, joilla ominaisuutena rdf:type 


			fdg = new XFDGConnect();

			String before;

			int kasitelaskuri = 0;
			int labelLaskuri = 0;
			int suomiLabel = 0;
			int ruotsiLabel = 0;
			int muut = 0;
			
			StringBuffer allLemmas = new StringBuffer(390000);
	/*
	 * 3.2.: muutettu Termoa, siten, että 
	 * kaikki labelit otetaan isoksi Stringiksi,
	 * lemmataan, ja liitetään uudessa YSOn iteroinnissa
	 * Termistön termeiksi. 
	 * Syy tähän toimintaan on javanatiivin mahdollinen
	 * bugi, lisätietoja täältä: http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6223635
	 * 
	 * (Toisaalta, nykyinen ratkaisu on myös huomattavasti nopeampi)
	 */
			
			while(concepts.hasNext()) {

				temp = concepts.nextResource();

				//jos resurssilla ominaisuus yso:label (prefLabel) , kyseessä concept

				stmtI = temp.listProperties(pl); //prefLabel
				
				if(stmtI.hasNext()) {
					kasitelaskuri++;
				} 
				while(stmtI.hasNext()) {
					labelLaskuri++;				

					try {
						currentStmt = stmtI.nextStatement();
					} catch (NoSuchElementException e) {
						System.out.println("No such element: " + e);
					}					
					if(this.getLang(currentStmt).equals("sv")) {
						ruotsiLabel++;
					} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
						suomiLabel++;
						before = this.getLiteral(currentStmt);
						allLemmas.append(before.replaceAll("_"," ")+"_ ");
					} else {
						muut++;
						before = this.getLiteral(currentStmt);
						allLemmas.append(before.replaceAll("_"," ")+"_ ");
					}
				
				}	
									
//	--> 7.12. poistettu määre initLang!
	/*
					current.addProperty(pLabel, terms.createLiteral(
								lemmatized, initLang));
	*/

				stmtI = temp.listProperties(al); //altLabel

				while(stmtI.hasNext()) {
					labelLaskuri++;		
					
					currentStmt = stmtI.nextStatement();

					if(this.getLang(currentStmt).equals("sv")) {
						ruotsiLabel++;
					} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
						suomiLabel++;				
						before = this.getLiteral(currentStmt);
						allLemmas.append(before.replaceAll("_"," ")+"_ ");
					} else {
						muut++;
						before = this.getLiteral(currentStmt);
						allLemmas.append(before.replaceAll("_"," ")+"_ ");
					}
										
				}					
		
			}
			
			System.out.println("------------------------------");
			System.out.println("Käsitteitä yhteensä: "+ kasitelaskuri);
			System.out.println("Labeleita yhteensä : "+ labelLaskuri);
			System.out.println("Suomenkielisiä     : "+ suomiLabel);
			System.out.println("Ruotsinkielisiä    : "+ ruotsiLabel);
			System.out.println("Määreettömiä       : "+ muut);
			System.out.println("Suomilabel+muut    : "+ (suomiLabel+muut));
			System.out.println("------------------------------");
			
			
			fdg.setInput(allLemmas.toString());
			System.out.print("Lemmataan (koko: " +allLemmas.toString().length()+")...");
			InputSource is = fdg.getFDG();
			System.out.println("...ok!");
			LemmaLister ll = new LemmaLister();
			System.out.print("Parsitaan...");
			ll.parse(is);
			System.out.println("...ok!");

			ArrayList <String> tempL = ll.getResult();
		
			System.out.println("--> listan koko: " + tempL.size()+" <--");
		
			java.util.Iterator <String> ii = tempL.iterator();
			
			concepts = m.listSubjectsWithProperty(subjProp);

	/*
			Alla kokeilupätkä koodia, jolla
			voi testata saman lemmatun muodon
			omaavien käsitteiden määrän
	*/
			
			
			HashMap<String, HashSet<String>> labelsWithUris= new HashMap<String, HashSet<String>>(); 
			
			while(concepts.hasNext()) {
				
				temp = concepts.nextResource();
				stmtI = temp.listProperties(pl);
							
				while(stmtI.hasNext()) {
					currentStmt = stmtI.nextStatement();				
					
					if(this.getLang(currentStmt).equals("sv")) {
					} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi				

						String sip = ii.next();

						HashSet<String> hs = labelsWithUris.get(sip);														
						if(hs==null) {
							hs = new HashSet<String>();
						}
						hs.add(temp.getURI());
												
						labelsWithUris.put(sip, hs);
												
					} else {

						String sip = ii.next();

						HashSet<String> hs = labelsWithUris.get(sip);														
						if(hs==null) {
							hs = new HashSet<String>();
						}
						hs.add(temp.getURI());
												
						labelsWithUris.put(sip, hs);
						
					}
				}
				
				stmtI = temp.listProperties(al);
				
				while(stmtI.hasNext()) {
					currentStmt = stmtI.nextStatement();

					if(this.getLang(currentStmt).equals("sv")) {								
					} else if(this.getLang(currentStmt).equals("fi")) {
						
						String sip = ii.next();

						HashSet<String> hs = labelsWithUris.get(sip);														
						if(hs==null) {
							hs = new HashSet<String>();
						}
						hs.add(temp.getURI());
												
						labelsWithUris.put(sip, hs);
						
					} else {
						
						String sip = ii.next();

						HashSet<String> hs = labelsWithUris.get(sip);														
						if(hs==null) {
							hs = new HashSet<String>();
						}
						hs.add(temp.getURI());
												
						labelsWithUris.put(sip, hs);
						
					}
										
				}
						
			}
			String result = "";			
			int labelCount = 0;
			Set<String> labels = labelsWithUris.keySet();
			for(String s: labels) {
				if(labelsWithUris.get(s).size() > 1) {
					labelCount ++;
					HashSet<String> uriSet = labelsWithUris.get(s);
					String urisOfLabel = "";
					for(String uris: uriSet) {
						urisOfLabel += uris+" "; 
					}
					result += s +": "+urisOfLabel+"\n"; 
				}
				
			}
			System.out.println("Saman labelin jakavia käsitteitä: "+labelCount+":");
			System.out.println(result);
			return true;
		}
		
	
	public String listActuality(String filename) {

		terms.setNsPrefix("yso","http://yso.fi/YSO#");
		Resource temp;
		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");		
			return null;
	    }
		
		Model m = ModelFactory.createDefaultModel();
		try {
			m.read(new InputStreamReader(in, "UTF-8"), "");
		} catch(Exception e) {
			System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
            e.printStackTrace();
        }
		Property subjProp = m.getProperty(YSOSUBJIDSPACE, YSOSUBJIDLOCAL);
		Property act = m.getProperty(YSOBASE,ACTUALITYLABEL);
		
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);
		StmtIterator stmtI;
		Statement currentStmt;
		
		StringBuffer result = new StringBuffer();
				
		while(concepts.hasNext()) {
			temp = concepts.nextResource();
			stmtI = temp.listProperties(act);
			
			if(stmtI.hasNext()) {
				result.append("\n"+temp.getURI()+"\n");
			}	
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();
				result.append("\t"+currentStmt.getObject().toString()+"\n");
			}
		}
		
		return result.toString();

	}
	
	
	
	private String getLang(Statement stmt) {
		return stmt.getLanguage();
	}
	
	private String getLiteral(Statement stmt) {
		return stmt.getLiteral().getString();
	}


	@SuppressWarnings("unused")
	private String getLemma(String terms) {
		terms = terms.replaceAll("_"," ");
		fdg.setInput(terms);
		InputSource fdgResult = fdg.getFDG();
		this.lp.parse(fdgResult);
		return this.lp.getResult();
	}

/**
 * Adds new preferred label to the model in the 
 * given language.
 * If preflabel already exist in this language,
 * old preflabel is changed to altlabel. 
 * If preflabel to be added exist 
 * as altlabel, altlabel is removed and
 * preflabel added.
 *
 */
	public boolean addPrefLabel(String uri, String prefLabel, String lang) {

		Resource r = terms.getResource(uri);

		//resurssi on olemassa
		if(terms.containsResource(r)) {
			//jos prefLabel on jo olemassa ko. kielellä ko. resurssille
			if(this.containsPref(uri,lang)) {
				Statement s = this.getPrefStatement(uri,lang);
				//tee olemassaolevasta altLabel...
                //...KUNHAN altName ei ole sama kuin uusi nimi!
				String altName = s.getString();
				if(altName.equals(prefLabel)) {
					//HUOM, nimi olisi tietysti sama lisätä, se ei muuttaisi tilannetta
					System.out.println("addPrefLabel: nimi on jo olemassa, lisäystä ei suoriteta!");
					return false;
				}
				System.out.println("addAltLabel");
				//ja poista ko. prefLabel ennen uuden lisäystä
				s.remove();
				this.addAltLabel(uri, altName, lang);

			//jos kyseinen resurssi on olemassa altLabelina
			} else if (this.containsAlt(uri,lang)) {
				Statement s = this.getAltStatement(uri,lang, prefLabel);
				if(s != null) {
					s.remove();
					System.out.println("Alt olemassa, kasvatetaan preferred labeliksi...");
				}
			}
			r.addProperty(pLabel, terms.createLiteral(prefLabel, lang));
			return true;
		}
		System.out.println("addPrefLabel: resource doesn't exist!");
		return false;
	}

/**
 * Adds new altlabel to model in the given language.
 * if altlabel to be added exist as a preflabel, 
 * altlabel is not added.
 *
 */
	public boolean addAltLabel(String uri, String altLabel, String lang) {

		Resource r = terms.getResource(uri);
		if(terms.containsResource(r)) {			
			if(this.containsPref(uri,altLabel,lang)) {
 				System.out.println("addAltLabel: Preflabel ("+lang+"): '"+altLabel + "' on olemassa.");
				return false;
			}
			r.addProperty(aLabel, terms.createLiteral(altLabel, lang));
			return true;
		}
		System.out.println("addAltLabel: resource doesn't exist!");
		return false;
	}
/**
 * Returns true if model contains preflabel of uri of given
 * label and lang.
 *
 */

	private boolean containsPref(String uri, String label, String lang) {
		Resource r = terms.getResource(uri);
		boolean b = false;
		Statement s;
		StmtIterator st = r.listProperties(pLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			
			if(s.getLiteral().getLanguage().equals(lang) &&
               s.getLiteral().getString().equals(label)) {
	
				b = true;
				break;
			} 
		}
		return b;	
	}
	//TO DO: korjaa bulkkiratkaisu jos tarpeen...
/**
 * Returns true if uri contains altlabel of given lang.
 *
 */
	public boolean containsAlt(String uri, String lang) {
		Resource r = terms.getResource(uri);
		boolean b = false;
		Statement s;
		StmtIterator st = r.listProperties(aLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			if(s.getLiteral().getLanguage().equals(lang)) {
				b = true;
                break;
			} 
		}
		return b;
	}

/**
 * Returns statement if there is altlabel of uri of given lang
 * and label.
 */
	public Statement getAltStatement(String uri, String lang, String value) {
		Resource r = terms.getResource(uri);
		Statement s;
		StmtIterator st = r.listProperties(aLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			if(s.getLiteral().getLanguage().equals(lang) &&
               s.getLiteral().equals(value)) {
				return s;
			} 
		}
		return null;
	}




/**
 * Returns true if there is preflabel of uri of given lang
 *
 */
	public boolean containsPref(String uri, String lang) {
		Resource r = terms.getResource(uri);
		boolean b = false;
		Statement s;
		StmtIterator st = r.listProperties(pLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			if(s.getLiteral().getLanguage().equals(lang)) {
				b = true;
				break;
			} 
		}
		return b;
	}
/**
 * Returns all preflabels of given uri as a String
 *
 */
	public String getPref(String uri) {
		StmtIterator st = terms.getResource(uri).
			listProperties(pLabel);

		String result = "";
		Statement s;
		while(st.hasNext()) {
			s = st.nextStatement();
			result += s.getString() + "\n";
		}
		return result;
	}

/**
 * Returns prefLabel of uri of given lang as a String
 *
 */	
	public String getPref(String uri, String lang) {

		Resource r = terms.getResource(uri);
		Statement s;
		StmtIterator st = r.listProperties(pLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			if(s.getLiteral().getLanguage().equals(lang)) {
				return s.getLiteral().getString();
			} 
		}
		return null;
	}
/**
 * Returns prefLabel of uri of given lang as a Statement
 *
 */
	public Statement getPrefStatement(String uri, String lang) {

		Resource r = terms.getResource(uri);
		Statement s;
		StmtIterator st = r.listProperties(pLabel);
		while(st.hasNext()) {
			s = st.nextStatement();
			if(s.getLiteral().getLanguage().equals(lang)) {
				return s;
			} 
		}
		return null;
	}
/**
 * Returns altLabels of uri as a String
 *
 */
	public String getAlt(String uri) {
		StmtIterator st = terms.getResource(uri).
				listProperties(aLabel);

		String result = "";
		Statement s;
		while(st.hasNext()) {
			s = st.nextStatement();
			result += s.getString() + ", lang:"+
                      s.getLiteral().getLanguage() +
                      "\n";
		}
		return result;
	}
/**
 * Returns Stringrep of uri (that is, preflabels and altlabels)
 *
 */

	public String toString(String uri) {
		String result = "PREF:\n"+this.getPref(uri)+"\n";
		result += "ALT:\n" +this.getAlt(uri);

		return result;
	}
/**
 * Returns the complete model as a String
 *
 */
	public String toString() {
		StringWriter sw = new StringWriter();	
		 
		String result;
//		terms.write(sw);
		terms.write(sw, "N-TRIPLE");
		result = sw.toString();
		return result;
	}
	
	public boolean writeToFile(String filename) throws Exception {
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		terms.write(pw);
		return true;
	}
	
	public boolean writeActualityToFile(String filename, String actuality) throws Exception {
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		pw.write(actuality);
		return true;
	}
	
/**
 * Returns Model
 *
 */
	public Model getModel() {
		return this.terms;
	}

	public static void main(String [] args) {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka.Termo");
		hep.setLevel(Level.DEBUG);
		
		Termo my = new Termo(Termo.DEFAULTNAMESPACE);
/*
		String inputFileName;
		InputStream in = null;
		try {
			inputFileName = args[0];
			in = FileManager.get().open( inputFileName );
	        if (in == null) {
	    		throw new IllegalArgumentException(
                   "File: '" + inputFileName + "' not found");
			}

		} catch(ArrayIndexOutOfBoundsException e) {
			System.out.println("Usage: Termo <filename>");
			System.exit(0);
		}
*/
		
		
		System.out.println("Lisätään ontologia (ilman oletuskieltä): " +
		my.createYSATerms("/home/alm/ontologies/kura/AAA_EIIMPORTTIA_uusiYSO_16_11_2006.owl", "UTF-8"));
//		my.createYSOSubGraphTerms(TrieIndex.getNeuOntology(),"UTF-8", "http://yso.fi/YSO#juhlapäivät"));

/*
		try {
		 my.writeToFile("/home/alm/termo/penkki/aika2.rdf");
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
*/		

/*
		System.out.println("Lisätään paikat");
		my.createPlaceTerms("/home/alm/ontologies/ontologies/poka/Paikat.owl", "UTF-8");

		try {
			my.writeToFile("/home/alm/termo/penkki/PaikkaTerms.rdf");
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
*/
		
/*
		System.out.println("Lisätään opastermit");
		my.createOpasTerms("/home/alm/ontocvs/ontologies/YSO/instanssidata.rdf", "ISO-8859-1");

		try {
			my.writeToFile("/home/alm/termo/penkki/opastesti.rdf");
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
*/

		
/*
		System.out.println("Lisätään kehysverbit");
//		my.createFrameVerbTerms(TrieIndex.getFrameVerbs(), "UTF-8");
		my.createFrameVerbTerms(TrieIndex.getNeuFrameVerbs(), "UTF-8");
		try {
			System.out.print("Kirjoitetaan tiedostoon..");
			my.writeToFile("/home/alm/termo/penkki/kehysverbotesti.rdf");
			System.out.println(".ok!");
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
*/
		
		
		
/*
		System.out.print("Luodaan actuality...");
		try {
		 my.writeActualityToFile("/home/alm/termo/penkki/actuality.txt", my.listActuality(args[0]));
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa: "+e);
			System.exit(0);
		}
		System.out.println("...ok!");
*/
		
		
		
		//	    my.addOntology(args[0], "fi");
//		System.out.println(my);
/*
		System.out.println(my.toString("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit"));
		
		System.out.println("lisätään huulikukille altlabel 'huulityyppinen kasvi' (fi): " +
		my.addAltLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "huulityyppinen kasvi", "fi"));

		System.out.println("----------------------------------");

		System.out.println("lisätään huulikukille toinen altlabel 'typpinen kasvi' (fi): " +
		my.addAltLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", 
			"typpinen kasvi", "fi"));

		System.out.println("----------------------------------");

		System.out.println("lisätään huulikukille 1. englkielinen altlabel 'lip-a-like plant' (en):");
        System.out.println(my.addAltLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "lip-a-like plant", "en"));

		System.out.println("----------------------------------");

		System.out.println("lisätään huulikukille preflabel 'huulikukkaiskasvinen' (fi):");
		System.out.println(my.addPrefLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "huulikukkaiskasvinen", "fi"));

		System.out.println("----------------------------------");
		
		System.out.println("lisätään huulikukille preflabel 'huulikkainen kasvinen' (fi): " +
		my.addPrefLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "huulikkainen kasvinen", "fi"));

		System.out.println("lisätään huulikukille preflabel 'liposomatas' (es): " +
		my.addPrefLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "liposomatas", "es"));

		System.out.println("----------------------------------");

		System.out.println("lisätään huulikukille altlabel 'huulikukkaiskasvinen' (fi): " +
		my.addAltLabel("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "huulikukkaiskasvinen", "fi"));

		System.out.println("----------------------------------");

		System.out.println("Huulikukkien pref (fi) on: " + my.getPref("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit", "fi"));

		System.out.println("----------------------------------");
	
		System.out.println("Huulikukkien altlabelit ovat:\n" +
		my.getAlt("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit"));

		System.out.println("----------------------------------");

		System.out.println("tulostetaan resurssin\n" + "'http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit'\ntiedot:\n" +
		my.toString("http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#huulikukkaiskasvit"));
*/
	}

}