/**
 * Harrin palikkaan liittyvi apukilkkeit
 */

package fi.seco.semweb.service.poka.disambiguation;
import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.PokaDoctor;
import fi.seco.semweb.service.poka.Termo;

public class D_eristin {

	private GeneralExtractor doc;
	private Document xmldoc;
	private boolean ok;
	private int noOfSentences;
	private ArrayList <ConceptIndex> index;
	private	ArrayList <Extracted> content;
	
	private Extracted current;
	
	private final String PREFIX = "bio.";
	
	
	private HashMap <String, Integer> wordToCount;
	
	private HashMap <String, String> uriToWord;
	
	private HashMap <String, Integer> wordToIndex;
	private HashMap <Integer, String> indexToWord;
	
	
		private class ConceptIndex {
			
			private int sentence;
			private int token;
		
			private ConceptIndex(int sentence, int token) {
				this.sentence = sentence;
				this.token = token;
			}

			private int getSentence() {
				return this.sentence;
			}
		
			private int getToken() {
				return this.token;
			}
		}
		
		private class Extracted {
			
			private String instanceID;
			private String text;
			private String lemma;
			private ArrayList <String> uris;
			private int count;
			private int word;
						
			private Extracted() {
				this.instanceID = null;
				this.text = null;
				this.lemma = null;
				
				uris = new ArrayList <String>();
			}

			private void addCount(int counter) {
				this.count = counter;
			}
			private void addWord(int word) {
				this.word = word;
			}
			
			private void addId(String id) {
				this.instanceID = id;
			}
			private void addText(String text) {
				this.text = text;
			}
			private void addLemma(String lemma) {
				this.lemma = lemma;
			}
			private void addConcept(String uri) {
				uris.add(uri);
			}

			private String getId(){
				return this.instanceID;
			}
			
			private String getText() {
				return this.text;			
			}
			private String getLemma() {
				return this.lemma;
			}
			private ArrayList<String> getUris() {
				return this.uris;
			}
			
			private int getUriSize() {
				return this.uris.size();
			}
			
			private int getCount() {
				return this.count;
			}
			private int getWord() {
				return this.word;
			}
			
		}
	
	
	public D_eristin() {
		doc = new GeneralExtractor();
		index = new ArrayList <ConceptIndex>();
		content = new ArrayList <Extracted>();
		ok = false;

		this.wordToCount = new HashMap<String, Integer>();
		this.uriToWord = new HashMap<String, String>();
		this.wordToIndex = new HashMap<String, Integer>();
		this.indexToWord = new HashMap<Integer, String>();
		
		this.createWordForm();
		
	}
	
	//TODO: toivottavasti toimii oikein! muutetttu 16.5.!
	private void createWordForm() {
		HashSet <String> temp = this.doc.getYsoUris();
		Iterator <String> ii = temp.iterator();

		String uriValue;
		String uriLabel;
		
		while(ii.hasNext()) {
			uriValue = ii.next();
			
			uriLabel = this.doc.getYsoLabel(uriValue);
			
			//tekniikka --> 1 (lkm)
			this.wordToCount.put(uriLabel, 0);
			
			//pitisi sislt aina risuaidan TODO: ksittely kauniisti
			uriValue = uriValue.substring(uriValue.indexOf('#')+1,uriValue.length());
			
//			 tekniikka_1 --> tekniikka
			this.uriToWord.put(uriValue,uriLabel); 	
		
		}
		
	   ii = this.wordToCount.keySet().iterator();
	   int indexer = 1;
	   String nextword;
	   while(ii.hasNext()) {
		   nextword = ii.next();
		   this.wordToIndex.put(nextword,indexer);
		   this.indexToWord.put(indexer,nextword);
		   
		   indexer++;
	   }
		
	}
	
	public void setDoc(String file) {
		
		String pokadoc = new PokaDoctor(file).getResult();
		this.doc.setDocument(pokadoc);
		this.index = new ArrayList<ConceptIndex>();
		
	}
	public void addConcepts() {
		this.doc.addYsoConcepts(GeneralExtractor.CONCEPTNODES,true,false);
		this.xmldoc = this.getXML();
		String expression = "/A/S";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		 
		try {
			nodes = (NodeList) xpath.evaluate(expression, xmldoc, XPathConstants.NODESET);
			this.noOfSentences = nodes.getLength();
		} catch (XPathExpressionException e) {
			System.out.println("XPATHEXCEPTION: " +e);
			System.exit(0);
		}
	}
	
	public String getDoc() {
		return this.doc.getDocument();
	}
	public Document getXML() {
		return this.doc.getXMLDoc();
	}
	
	public void getConceptLocations() {
		 String expression = "/A/S/C";
		 NodeList nodes;

		 XPath xpath = XPathFactory.newInstance().newXPath();
		 
		 int sentence;
		 int token;
		 
		 try {

			//ota docin ksitesolmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, xmldoc, XPathConstants.NODESET);

	        	for(int i = 0; i < nodes.getLength(); i++){
	           		Node item = nodes.item(i);		
	     	        
	           		token = Integer.parseInt(item.getAttributes().
	           							getNamedItem("no").getTextContent());

	           		sentence = Integer.parseInt(item.getParentNode().
       					getAttributes().getNamedItem("no").getTextContent());
	           		
	           		index.add(new ConceptIndex(sentence,token));      		
			}
	        	

		} catch (XPathExpressionException e) {
	    		System.out.println("XPATHEXCEPTION: " +e);
	    		System.exit(0);
	    	}
		this.ok = true;
	}
	public boolean fill() {


		
		if(ok) {
		
			for(ConceptIndex c:index) {
				current = new Extracted();
				current.addId(this.getInstName(this.getConcept(c)));
				current.addText(this.getSentences(1, "b", c, true));
				current.addLemma(this.getSentences(1, "head", c, false));
				
				this.content.add(current);
/*				System.out.println("--------Luotiin ksite:--------\n"+
						     	   "ID: " + current.getId()+"\n"+
						     	   "WO :" + current.getWord()+"\n"+
						     	   "INS:" + current.getCount()+"\n"+
						     	   "TEX:" + current.getText()+"\n"+
						     	   "LEM:" + current.getLemma()+"\n"+
						     	   "URI:" + current.getUris().toString()+"\n"+
						     	   "---------------------------------");
*/				
			}
			
		}
		return ok;
	}

	/**
	 * Luo uuden instanssin nimen tavat_1 --> bio.tavat.3
	 * 
	 * @param concept ksite, esim 'tavat_1' (ilman #-merkki)
	 * @return
	 */
	
	private String getInstName(String concept) {
		String name = this.uriToWord.get(concept);
		
		this.current.addWord(this.wordToIndex.get(name));
		
		int count = this.wordToCount.get(name);
		count++;
		this.wordToCount.put(name, count);
		
		this.current.addCount(count);
		name = this.PREFIX+name+"."+count;
		return name;	
	}
	
	private String getConcept(ConceptIndex c) {
			
		String result = null;
		int sent = c.getSentence();
		int token = c.getToken();
		String expression = "/A/S["+sent+"]/*["+(token+1)+"]/I";
   	    Element node;
		 
	    XPath xpath = XPathFactory.newInstance().newXPath();
	 	
	    try {
			    		
	    		node = (Element) xpath.evaluate(expression, xmldoc, XPathConstants.NODE);
	      		
	    		result = node.getTextContent();	
	    		StringTokenizer st = new StringTokenizer(result);
	    		String temp;
	    		
	    		while(st.hasMoreTokens()) {
	    			temp = st.nextToken();
	    			
	    			this.current.addConcept("#"+temp+": "+
	    					this.doc.getYSO().getUpper(Termo.YSOBASE+temp)); //rivin ksitevaihtoehdot
	    			result = temp;	
	    		}
	    		
				        	
		 } catch (XPathExpressionException e) {
		    		System.out.println("XPATHEXCEPTION: " +e);
		    		System.exit(0);
		 }
		
		return result;
	}
	
/**
 * Palauttaa tiedoilla s=0, hep, CI= 4,2
 * lauseen 4 lemman siten, ett 2. token on ymprity
 * tageilla hep, lemmattuna tekstin.
 *  
 * @param sentences ympriv lause-etisyys (0= tm lause, 1= ed+seur...)
 * @param tag mill tagilla ksite <tag>merkataan</tag>
 * @param c sislt lauseen nron ja tokenin nron
 */
	
	private String getSentences(int sentences, String tag, ConceptIndex c, boolean isText) {
		
		 int sent = c.getSentence();
		 int surroundings = sent - sentences;

		 StringBuffer result = new StringBuffer();

		 if(surroundings > 0) {
			while(surroundings < sent) {
				result.append(this.getSentence(surroundings, isText));
				surroundings++;
			}		
		 }
		 	 
		 
		 String expression;
		 if(isText) {
			 expression = "/A/S["+sent+"]/*/T";
		 } else {
			 expression = "/A/S["+sent+"]/*/L";
		 }
		 NodeList nodes;
		 
		 XPath xpath = XPathFactory.newInstance().newXPath();
	 
		 int token = c.getToken();
		 
		 try {
		
			nodes = (NodeList) xpath.evaluate(expression, xmldoc, XPathConstants.NODESET);
			int i;
			for(i = 0; i < token; i++){
				result.append(nodes.item(i).getTextContent()+" ");
			}
			result.append("<"+tag+">"+nodes.item(token).getTextContent()+"</"+tag+"> ");
					
	        	for(i = token+1; i < nodes.getLength(); i++){
	        		result.append(nodes.item(i).getTextContent()+" ");		
	        	}
	        	
		 } catch (XPathExpressionException e) {
		    		System.out.println("XPATHEXCEPTION: " +e);
		    		System.exit(0);
		 }
		 
		 surroundings = sent + sentences;
		 int counter = sent+1;
		 if(surroundings <= this.noOfSentences) {
			 while(counter <= surroundings) {
				 result.append(this.getSentence(counter, isText));
				 counter++;
			 }
		 }
		 
		 
		 
		 return result.toString().replaceAll("\"","&quot;"); 
		 
	}
	
	private String getSentence(int sentence, boolean isText) {
		String expression;
		if(isText) {
			expression = "/A/S["+sentence+"]/*/T";
		} else {
			expression = "/A/S["+sentence+"]/*/L";
		}
		NodeList nodes;
		StringBuffer sb = new StringBuffer();
		
		XPath xpath = XPathFactory.newInstance().newXPath();
		 try {
			 nodes = (NodeList) xpath.evaluate(expression, xmldoc, XPathConstants.NODESET);
			 	for(int i =  0; i < nodes.getLength(); i++){
			 		sb.append(nodes.item(i).getTextContent()+" ");
			 	}
		 } catch (XPathExpressionException e) {
	    		System.out.println("XPATHEXCEPTION: " +e);
	    		System.exit(0);
	 
		 }
		 return sb.toString();
	}
	
	public boolean printHTML(String filename) throws Exception {
		System.out.print("Kirjoitetaan tiedostoon '"+filename+"'...");

		StringBuffer text = new StringBuffer();
		
//------------------------------------------------------
		text.append(
		"<html><head><title>S-Tagger</title></head>\n"+
		"<body>\n<p></p>\n<form action=\"formhandler.php\" enctype=\"multipart/form-data\" method=\"post\">"+
		"\n<table border=\"2\"><tbody>\n<tr><th>Instance ID</th><th>Instance text</th><th>Select key sense</th>"+
		"<th></th></tr>\n<input name=sysname type=hidden value=\"O\">\n");

		Iterator <Integer> i = this.indexToWord.keySet().iterator();
		int index;
		
		//listn nimet: <input name=word_1 type=hidden value="mallit">
		while(i.hasNext()) {
			index = i.next();
			text.append("<input name=word_");
			text.append(index);
			text.append(" type=hidden value=\"");
			text.append(this.indexToWord.get(index));
			text.append("\">\n");	
		}
		//listn laskurikentt
		text.append("<input name=word_count type=hidden value=\""+
				       this.wordToIndex.size()+"\">\n");
		text.append("<input name =inst_count type=hidden value=\""+
					this.content.size()+"\">\n");
		
		Iterator <String> iii;
		String uriOption;
		
		//listn instanssit
		Iterator <Extracted> ii = this.content.iterator();
		Extracted toProcess;
		while(ii.hasNext()) {
			toProcess = ii.next();

			text.append("\n<tr><td>'"); //lis ID: <tr><td>'bio.mallit.1'</td> 
			text.append(toProcess.getId());
			text.append("'</td>\n");

			text.append("\t<td>"+toProcess.getText()+"</td>\n"); //lis text
			
			//lis <input name=instance_id_1_2 type=hidden value="bio.mallit.2">
			text.append("\t<input name=instance_id_");
			text.append(toProcess.getWord()+"_"+toProcess.getCount()+" ");
			text.append("type=hidden value=\""+toProcess.getId()+"\">\n");
			
			//lis hidden text
			text.append("\t<input name=insttext_");
			text.append(toProcess.getWord()+"_"+toProcess.getCount()+" ");
			text.append("type=hidden value=\""+toProcess.getText()+"\">\n");
			
			//lis hidden lemma
			text.append("\t<input name=instlemma_");
			text.append(toProcess.getWord()+"_"+toProcess.getCount()+" ");
			text.append("type=hidden value=\""+toProcess.getLemma()+"\">\n");
			
			//lis optiot
			text.append("\t<td><select size=4 multiple name=\"key_"+
					    toProcess.getWord()+"_"+toProcess.getCount()+"\">\n");
			text.append("\t\t<optgroup label=\"'"+
					this.indexToWord.get(toProcess.getWord())+"'\">\n");
			iii = toProcess.getUris().iterator();
			
			while(iii.hasNext()) {
				uriOption = iii.next();
				text.append("\t\t<option value=\"http://yso.fi/YSO" +
						uriOption.substring(0, uriOption.indexOf(':'))+"\">"+
						uriOption+" </option>\n");				
			}
			text.append("\t\t</optgroup>\n");
			text.append("\t\t<option value=\"none\">none of the above</option>\n");
			text.append("\t\t<option value=\"eos\">do not know (ask me later) </option>\n");
			text.append("\t</select>\n");
			text.append("</td></tr>\n");
			
			
		}
		
		text.append("</tbody></table><br></br>\n");
		text.append("<input type=\"submit\" value=\"Feed to DB  | Open D-Factory GUI\"></input>\n");
		text.append("</form>\n</body></input");
		
		
//-----------------------------------------------------		
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		pw.write(text.toString());
		pw.flush();
		System.out.println("...tiedosto valmis!");

		return true;
	}
	
	public static void main(String[] args) {
	
		String filePath = "/home/alm/aaparser/text/all/";
		int running = 1;
		String suffix = ".html";
		String temp;
		
		D_eristin my = new D_eristin();
		
		
		while(running <= 100) {
			temp = filePath+running+suffix;
			System.out.print("Ksitelln dokumentti "+running+"...");

			my.setDoc(temp);

			my.addConcepts();
			
			my.getConceptLocations();
			my.fill();

			running++;
			System.out.println("...ok!");
		}
		try {
			my.printHTML("/home/alm/termo/penkki/D-result.html");
		} catch(Exception e) {
			System.out.println("Virhe! "+e);
			System.exit(0);
		}
		
	}

}
