package fi.seco.semweb.service.poka.frame;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fi.seco.semweb.service.poka.Configuration;

/**
 * Kuvaa yksittisen kehyksen
 * @author alm
 *
 * Dokumentin nimi kokoelmaan, ei tnne
 * 
 */

public class Frame {
	private static final Log log = LogFactory.getLog(Frame.class);
	
	 private boolean discarded; //jos true, hyltn kehys
	
	private String verbUri;
	private int sentence;
	private boolean negation;
	
	//tokenIndeksit
	private int sentenceStart;
	private int sentenceEnd;
	
	//mppys rooli-id:st tokenin numeroon
	private HashMap<Integer, ArrayList<Integer>> roleToToken;

	//mppys roolista ehdolliseen tokeniin, conditional role
	private HashMap<Integer, ArrayList<Integer>> conditionalRoleToToken;
	
	public Frame(int sentence) {
		this();
		this.sentence = sentence;
	}
	
	public Frame() {
		this.roleToToken = new HashMap<Integer, ArrayList<Integer>>();
		this.conditionalRoleToToken = new HashMap<Integer, ArrayList<Integer>>();
		this.negation = false;
		this.discarded  = false;
	}
	
	public Frame(String verbUri) {
		this();
		this.verbUri = verbUri;
	}
/**
 * Prova-pttelij asettaa, jos
 * koko kehys hyltn:
 * tm on tapaus esimerkiksi
 * olla-verbin yhteydess jos
 * olla sana on perfektin / pluskvamperfektin
 * osa. Perfektin ja pluskan jlkimminen
 * verbi tunnistetaan erikseen (olla ptelln
 * pois)
 * 
 */
	public void setDiscard() {
		this.discarded = true;
	}
	public boolean discarded() {
		return this.discarded;
	}
	
	public String getLocalName() {
		return this.verbUri.substring(verbUri.indexOf('#'));
	}
	 
	public void setSentence(int sentence) {
		this.sentence = sentence;
	}
	
	public int getSentenceNumber() {
		return this.sentence;
	}
/**
 * Kehyksen sisltvn lauseen sijainnin
 * alkukoordinaatin asetus
 * @param start
 */
	public void setStart(int start) {
		this.sentenceStart = start;
	}
/**
 * Lauseen alkukoordinaatin palautus
 * @return
 */
	public int getStart() {
		return this.sentenceStart;
	}
/**
 * Loppukoordinaatin asettaminen
 * @param end
 */
 	public void setEnd(int end) {
 		this.sentenceEnd = end;
 	}
/**
 * Loppukoordinaatin palautus
 * @return
 */
 	public int getEnd() {
 		return this.sentenceEnd;
 	}
	
	public void setNegation() {
		log.debug("Negaation lisys");
		this.negation = true;
	}
	public boolean getNegation()  {
		return this.negation;
	}
	
	public void setAgent(int token) {
		log.debug("Agentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.AGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.AGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}

	public void setPatient(int token) {
		log.debug("Patientin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.PATIENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.PATIENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}

	public void setPlace(int token) {
		log.debug("Paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.PLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.PLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setTime(int token) {
		log.debug("Ajan lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setMaterial(int token) {
		log.debug("Materiaalin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.MATERIAL))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.MATERIAL, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setInstrument(int token) {
		log.debug("Instrumentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.INSTRUMENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.INSTRUMENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setRelativePlace(int token) {
		log.debug("Suhteellisen paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.RELATIVEPLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.RELATIVEPLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setAbsolutePlace(int token) {
		log.debug("Absoluutin paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.ABSOLUTEPLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.ABSOLUTEPLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setAbsoluteTime(int token) {
		log.debug("Absolutti aika lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.ABSOLUTE_TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.ABSOLUTE_TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setHumanAgent(int token) {
		log.debug("Ihmisagentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.HUMANAGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.HUMANAGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setAnimalAgent(int token) {
		log.debug("Elinagentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.ANIMALAGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.ANIMALAGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setEffector(int token) {
		log.debug("Effectorin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.EFFECTOR))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.EFFECTOR, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	public void setEssence(int token) {
		log.debug("Essencen lisys");

		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.ESSENCE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.ESSENCE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}		
		
	}
	
	public void setResource(int token) {
		log.debug("Resurssin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.RESOURCE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.RESOURCE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setRelativeTime(int token) {
		log.debug("Suhteellisen ajan lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.RELATIVE_TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.RELATIVE_TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setGoal(int token) {
		log.debug("Goalin lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.GOAL))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.GOAL, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	public void setDestination(int token) {
		log.debug("Destinaation lisys");
		
		ArrayList <Integer> al;
		if((al = this.roleToToken.get(Configuration.DESTINATION))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.roleToToken.put(Configuration.DESTINATION, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
//--------------------------------------------
//--------------------------------------------
//--------------------------------------------
//--------------------------------------------
//lisys 18-10-06: ehdolliset vastineet, joita suositellaan:
	public void setCondAgent(int token) {
		log.debug("Ehdollisen agentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.AGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.AGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}

	public void setCondPatient(int token) {
		log.debug("Ehdollisen patientin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.PATIENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.PATIENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}

	public void setCondPlace(int token) {
		log.debug("Ehdollisen paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.PLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.PLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondTime(int token) {
		log.debug("Ehdollisen ajan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondMaterial(int token) {
		log.debug("Ehdollisen materiaalin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.MATERIAL))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.MATERIAL, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondInstrument(int token) {
		log.debug("Ehdollisen instrumentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.INSTRUMENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.INSTRUMENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondRelativePlace(int token) {
		log.debug("Ehdollisen suhteellisen paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.RELATIVEPLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.RELATIVEPLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondAbsolutePlace(int token) {
		log.debug("Ehdollisen absoluutin paikan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.ABSOLUTEPLACE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.ABSOLUTEPLACE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondAbsoluteTime(int token) {
		log.debug("Ehdollisen absoluttisen ajan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.ABSOLUTE_TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.ABSOLUTE_TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondHumanAgent(int token) {
		log.debug("Ehdollisen ihmisagentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.HUMANAGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.HUMANAGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondAnimalAgent(int token) {
		log.debug("Ehdollisen elinagentin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.ANIMALAGENT))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.ANIMALAGENT, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondEffector(int token) {
		log.debug("Ehdollisen effectorin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.EFFECTOR))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.EFFECTOR, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondEssence(int token) {
		log.debug("Ehdollisen essencen lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.ESSENCE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.ESSENCE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondResource(int token) {
		log.debug("Ehdollisen resurssin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.RESOURCE))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.RESOURCE, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondRelativeTime(int token) {
		log.debug("Ehdollisen suhteellisen ajan lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.RELATIVE_TIME))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.RELATIVE_TIME, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
	public void setCondGoal(int token) {
		log.debug("Ehdollisen goalin lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.GOAL))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.GOAL, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	public void setCondDestination(int token) {
		log.debug("Ehdollisen destinaation lisys");
		
		ArrayList <Integer> al;
		if((al = this.conditionalRoleToToken.get(Configuration.DESTINATION))==null) {
			log.debug("lista null, luodaan ja listn");
			al = new ArrayList<Integer>();
			al.add(token);
			this.conditionalRoleToToken.put(Configuration.DESTINATION, al);
		} else {
			log.info("LISTN TOINEN!");
			al.add(token);
		}
		
	}
	
//--------------------------------------------
	

	
	public String getUri() {
		return this.verbUri;
	}
	
	//KYT Verb-luokan vakioita!
	public boolean hasValue(int frameEntity) {
		return this.roleToToken.containsKey(frameEntity);
	}
	
	public boolean hasConditionalValue(int frameEntity) {
		return this.conditionalRoleToToken.containsKey(frameEntity);
	}

/**
 * Palauttaa roolia vastaavien arvojen joukosta
 * ensimmisen. Esim. parametri (frameEntity) 0
 * vastaa agenttia, palautetaan agenttiluettelon
 * ensimmisen olevan tokenin numero. 
 * 
 * @param frameEntity
 * @return ensimmisen ominaisuuteen tsmvn tokenin numero
 */
	public int getValue(int frameEntity) {
		return this.roleToToken.get(frameEntity).get(0);
	}
/**
 * Palautetaan ehdollinen arvo, esim. agentilla 'hn' 
 * (=pttelykone mritt mik on ehdollinen arvo
 * @param frameEntity
 * @return
 */
	public int getConditionalValue(int frameEntity) {
		return this.conditionalRoleToToken.get(frameEntity).get(0);
	}

/**
 * Palauta kaikki tsmvt arvot, so. koko arraylist
 * vrt. getValue
 * @param frameEntity
 * @return
 */
	public ArrayList<Integer> getValues(int frameEntity) {
		return this.roleToToken.get(frameEntity);
	}
	/**
	 * Palauta kaikki tsmvt arvot, so. koko arraylist
	 * vrt. getConditionalValue
	 * @param frameEntity
	 * @return
	 */
	public ArrayList<Integer> getConditionalValues(int frameEntity) {
			return this.conditionalRoleToToken.get(frameEntity);
	}	
	
	

	
	public String getLineRepresentation(Sentence sentence) {
		


		String addition = "";
		
		Set <Integer> s = this.roleToToken.keySet();
		for(Integer i: s) {
			switch(i) {
			
			case Configuration.AGENT:
				if(this.roleToToken.containsKey(i)) {
					addition += "Agent : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}					
				}
				break;
			case Configuration.PATIENT:
				if(this.roleToToken.containsKey(i)) {
					addition += "Patient : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.PLACE:
				if(this.roleToToken.containsKey(i)) {
					addition += "Place : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.TIME:
				if(this.roleToToken.containsKey(i)) {
					addition += "Time : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.MATERIAL:
				if(this.roleToToken.containsKey(i)) {
					addition += "Material : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.INSTRUMENT:
				if(this.roleToToken.containsKey(i)) {
					addition += "Instrument : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.RELATIVEPLACE:
				if(this.roleToToken.containsKey(i)) {
					addition += "RelPlace : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.ABSOLUTEPLACE:
				if(this.roleToToken.containsKey(i)) {
					addition += "AbsPlace : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.ABSOLUTE_TIME:
				if(this.roleToToken.containsKey(i)) {
					addition += "AbsTime : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.HUMANAGENT:
				if(this.roleToToken.containsKey(i)) {
					addition += "HAgent : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.ANIMALAGENT:
				if(this.roleToToken.containsKey(i)) {
					addition += "Animal : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.EFFECTOR:
				if(this.roleToToken.containsKey(i)) {
					addition += "Effector : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.RESOURCE:
				if(this.roleToToken.containsKey(i)) {
					addition += "Resource : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.RELATIVE_TIME:
				if(this.roleToToken.containsKey(i)) {
					addition += "RelTime : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.GOAL:
				if(this.roleToToken.containsKey(i)) {
					addition += "Goal : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			case Configuration.DESTINATION:
				if(this.roleToToken.containsKey(i)) {
					addition += "Destination : '";
					for(int k: this.roleToToken.get(i)) {
						addition += sentence.getToken(k).getText()+"'| ";
					}
				}
				break;
			
			}
		}
		if(addition.equals("")) {
			//oli aiemmin return null; 
			return this.sentence+". "+this.getLocalName();
		} else {
			String result = this.sentence+". "+this.getLocalName()+": "+addition;
			return result;
		}
	}
	
	
	public String toString() {
		String result = "-------------->>\n";

		Set <Integer> s = this.roleToToken.keySet();
		for(Integer i: s) {
			switch(i) {
			
			case Configuration.AGENT:
				result += "Agent         : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.PATIENT:
				result += "Patient       : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.PLACE:
				result += "Place         : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.TIME:
				result += "Time          : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.MATERIAL:
				result += "Material      : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.INSTRUMENT:
				result += "Instrument    : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.RELATIVEPLACE:
				result += "Relative place: "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.ABSOLUTEPLACE:
				result += "Absolute place: "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.ABSOLUTE_TIME:
				result += "Absolute time : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.HUMANAGENT:
				result += "Human agent   : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.ANIMALAGENT:
				result += "Animal agent  : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.EFFECTOR:
				result += "Effector      : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.RESOURCE:
				result += "Resource      : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.RELATIVE_TIME:
				result += "Relative time : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.GOAL:
				result += "Goal          : "+this.roleToToken.get(i).toString()+"\n";
				break;
			case Configuration.DESTINATION:
				result += "Destination   : "+this.roleToToken.get(i).toString()+"\n";
				break;
			
			}
		}
		result += "--------------<<\n";
		return result;
	}
/**
 * Tulostaa sanat mukaan kun annetaan oikea lause mukaan
 * @param sent
 * @return
 */
	public String toString(Sentence sent) {
		String result = "-------------->>\n";
			  
		result +=  this.getUri()+"\n----------------\n";
			
		Set <Integer> s = this.roleToToken.keySet();
		for(Integer i: s) {
			switch(i) {
										
			case Configuration.AGENT:
				result += "Agent         : ";
				break;
			case Configuration.PATIENT:
				result += "Patient       : ";
				break;
			case Configuration.PLACE:
				result += "Place         : ";
				break;
			case Configuration.TIME:
				result += "Time          : ";
				break;
			case Configuration.MATERIAL:
				result += "Material      : ";
				break;
			case Configuration.INSTRUMENT:
				result += "Instrument    : ";
				break;
			case Configuration.RELATIVEPLACE:
				result += "Relative place: ";
				break;
			case Configuration.ABSOLUTEPLACE:
				result += "Absolute place: ";
				break;
			case Configuration.ABSOLUTE_TIME:
				result += "Absolute time : ";
				break;
			case Configuration.HUMANAGENT:
				result += "Human agent   : ";
				break;
			case Configuration.ANIMALAGENT:
				result += "Animal agent  : ";
				break;
			case Configuration.EFFECTOR:
				result += "Effector      : ";
				break;
			case Configuration.RESOURCE:
				result += "Resource      : ";
				break;
			case Configuration.RELATIVE_TIME:
				result += "Relative time : ";
				break;
			case Configuration.GOAL:
				result += "Goal          : ";
				break;
			case Configuration.DESTINATION:
				result += "Destination   : ";
				break;
			
			}
			for(Integer j: this.roleToToken.get(i)) {
				result += "\n\t"+j+": "+sent.getToken(j).getText()+"\n";
			}
			
		}
		result += "\n----------------\n";
		return result;
	}
	
	
}
