/**
 * 
 * Uutta verbiontologiaa varten luotu kopio FrameExtractorista
 * 
 * Yleinen kehyseristin, hieman vastaava kuin OpasExtractor,
 * tarkennettu yleisestä eristimestä.
 */


package fi.seco.semweb.service.poka.frame;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.hp.hpl.jena.ontology.AllValuesFromRestriction;
import com.hp.hpl.jena.ontology.HasValueRestriction;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.ontology.Restriction;
import com.hp.hpl.jena.ontology.SomeValuesFromRestriction;
import com.hp.hpl.jena.ontology.UnionClass;
import com.hp.hpl.jena.rdf.model.Container;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Seq;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import fi.seco.semweb.service.poka.Configuration;
import fi.seco.semweb.service.poka.DocumentExtractor;
import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.name.NameExtractor;
import fi.seco.semweb.service.poka.name.NameExtractorFrame;
import fi.seco.semweb.service.poka.name.NameIndex.Name;
import fi.seco.semweb.service.poka.util.DocToHTML;
import fi.seco.semweb.service.poka.util.PlaceQuery;
import fi.seco.semweb.service.poka.util.FoundResources;
import fi.seco.semweb.service.poka.util.PokaResource;
import fi.seco.semweb.service.poka.util.TermoQuery;
import fi.seco.semweb.service.poka.util.TrieIndex;
import fi.seco.semweb.service.poka.util.VerbQuery;
import fi.seco.semweb.service.poka.util.YsoQuery;

public class FrameExtractorCook extends DocumentExtractor {
	private static final Log log = LogFactory.getLog(FrameExtractorCook.class);
	
	private OntModel m; //malli: YSO, verbit, paikat
	
	public static final String FRAMEPATH = Configuration.getOldFullModel(); 
	public static final String COOKFRAMEPATH = Configuration.getNewFullModel();
	
	public static final String COOKVTERMS = Configuration.getCookVerbTerms();	
//	public static final String VERBPATH = "/home/alm/ontologies/ontologies/poka/frame/Term-verbs.rdf"; 
	public static final String VERBPATH = Configuration.getVerbTermsFull();
	public static final String VERBPATHBE = Configuration.getBeVerbTermsFull();
//objektityypisten verbiominaisuuksien (=frame roolit) nimet:
	public static final String BASE  = FrameOntologyBuilder.DEFAULTNAMESPACE;
	
//-------------------------------------------
	
	public static final String CONCEPTNODE = "C"; //CONCEPT -- käsitesolmu
	public static final String REGEXNODE = "R"; //REGEX -- regexillä lÃ¶ydetty käsite
	public static final String NAMENODE = "NA"; //NAME -- tulee myÃ¶hemmin nimien tunnistukseen
	public static final String VERBNODE = "V";
	public static final String PLACENODE = "PL";
	public static final String ANIMALNODE = "AN";
	public static final String RELATIVEPLACENODE = "RP";
	public static final String RELATIVETIMENODE = "RT";
	public static final String PRONOUNNODE = "PN";

	//regexin kohde, HUOM: päivämäärä menee <OT>-solmuksi eikä sisällä dependenssejä!
	public static final String [] NUMERALNODES = {"NM"}; //=regex
//---------------------------------------------

	//verbiHash: uri --> Verb
	private HashMap<String, Verb> uriToVerb;	
	
	//regex --> Uri
	private HashMap<String, String> regexToC;
	
	//kyselyrajapinnat
	private YsoQuery yso;
	private FrameVerbQuery verbs;
	private PlaceQuery places;

	private NameExtractorFrame ne;
	private TrieIndex termExist;
	private TrieIndex verbExist; //TODO: korvataan uudella framemallilla
	private TrieIndex placeExist;
	
	private TrieIndex faunaExist; //elukat
	private TrieIndex relativePlaceExist; //suhteelliset paikat 
	
	//HUOM! seuraava kyseenalainen: osa termeistä ilman käsitevastaavuutta, kokeilu!
	private TrieIndex relativeTimeExist; //suhteelliset ajat
	
//	private FoundResources all; //FIXME poista kun ei tarvita enää
	
	private FoundResources placeRes;
	private FoundResources ysoRes;
	private FoundResources verbRes;
	private FoundResources regexRes;
	
	public FrameExtractorCook() {
		super();
		
		this.regexToC = new HashMap<String, String>();
		
//		m = ModelFactory.createDefaultModel();
        //mikä paras / nopein OntModel? getVerbInfo hidasteli oletuksella.
		// --> Ollaan fullin puolella kovasti, päättelijät pois
		m = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
//		m = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_MICRO_RULE_INF);

		ne = new NameExtractorFrame();
		this.ne.setNameConcept("http://yso.fi/YSO#henkilÃ¶t");
		
		log.info("lisätään Framemalli"); 
		this.addOntology(FrameExtractorCook.COOKFRAMEPATH, "UTF-8");
		log.info("Frame ok");				
		this.places = new PlaceQuery(m);
		this.yso = new YsoQuery(m);
		this.verbs = new FrameVerbQuery(m);
		
		log.info("Lisätään termitiedostot");
		this.termExist = new TrieIndex(Configuration.getNeulemmas(),
				 					   "ISO-8859-1",
				 					   Configuration.DEFAULTFRAMENAMESPACE);

		
//yso ilman elukoita ja paikkojaAll-animals-bugs-paikka.rdf
/*		
		this.termExist = new TrieIndex("/home/alm/ontologies/ontologies/poka/frame/All-animals-bugs-paikka.rdf",
									   "UTF-8", TrieIndex.DEFAULTNAMESPACE);
*/		
		log.debug("Yso ok");

		//HUOM! asuminen hoidettu termitiedostolla!
		//uudet verbit 01-08-06! (olla on mukana)
		this.verbExist = new TrieIndex(COOKVTERMS,
				 					   "ISO-8859-1",
				 					   Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("VerbistÃ¶ ok");
		this.placeExist = new TrieIndex(Configuration.getPlaceLemmas(),
										"ISO-8859-1",
										Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Paikat ok");
		
		log.info("termistÃ¶t ok.");
		
		//näihin dokumentista lÃ¶ydetyt käsitteet
		this.ysoRes = new FoundResources();
		this.placeRes = new FoundResources();
		this.verbRes = new FoundResources();
		this.regexRes = new FoundResources(); 
		//verbitietojen indeksointi: verbien termindeksiltä otetaan
		//kaikki urit, urien tiedot haetaan framemallista metodia
		//getVerbInfo käyttäen
		log.info("Luodaan verbimäppäys");
		this.uriToVerb = this.createVerbIndex();
		log.info("verbimäppäys ok");
		
	}
	
/**
 * Mallien lisäys tiedostoista, apinoitu TermoQuerystä
 * 
 *
 */
	public boolean addOntology(String filename, String encoding) {
		Model mAttach;
		try {
			InputStream in = this.readFile(filename);
			mAttach = ModelFactory.createDefaultModel();
			mAttach.read(new InputStreamReader(in, encoding), "");
			this.m.add(mAttach);				

			return true;
		} catch(IOException e) {
			System.out.println("Error reading file '"+
	  			filename+"'.");
			//System.exit(0);
			return false;
		}
	}

	public void setInputString(String documentUrl) {
		log.debug("Asetetaan teksti");
		super.setInputString(documentUrl);
		log.debug("Asetetaan nimentunnistin dokumentille");
		this.ne.setDocument(this.getXMLDoc());
		this.clearPokaInstance();
	}
	
	public void setDocument(String documentUrl) {
		log.debug("Asetetaan dokumentti '"+documentUrl+"'");
		super.setDocument(documentUrl);
		log.debug("Asetetaan nimentunnistin dokumentille");
		this.ne.setDocument(doc);
		this.clearPokaInstance();
	}
	/**
	 * Tyhjentää dokumentin instanssitiedot
	 * Huom! tyhjennetään automaattisesti kun uusi dokumentti asetetaan
	 */	
		public void clearPokaInstance() {
			this.placeRes.clear();
			this.verbRes.clear();
			this.regexRes.clear();
			this.ysoRes.clear();
			this.ne.index.clear();
		}

		
    /**
     * Palauttaa mallin päättelijälle FrameFinderCookin kautta
     */
		public OntModel getModel() {
			return this.m;
		}
		
    /**
     * Palauttaa tiedon ovatko annetut urit samat TAI 
     * ylä-alaluokkasuhteessa --
     * käytetään kehyspäättelyssä 
     * 
     * @param child alaluokan uri
     * @param father yläluokan uri
     */
		public boolean inRangeOf(String child, String father) {
			if(child.equals(father)) {
				return true;
			}

			OntClass childClass  = this.m.getOntClass(child);
			if(childClass != null) {
				OntClass fatherClass = this.m.getOntClass(father);
				if(fatherClass != null) {
					return childClass.hasSuperClass(fatherClass);
				}
			}
			return false;
		}
		
	/**
	 * Luo ontologian perusteella verbi-indeksin: 
	 * mäppäys urista Verbi-olioon. Yksittäinen 
	 * verbiolio sisältää tiedot rooleista, joita kyseiselle
	 * verbillä on.
	 * @return
	 */
		public HashMap<String, Verb> createVerbIndex() {
			//luodaan verbejä varten String-->Integer hashmap, joten ei
			//tarvi tehdä merkkijonovertailuja (=vertailu 'automatisoitu'):
			HashMap<String, Integer> propNameToInt = new HashMap<String,Integer>(Configuration.VERBSIZE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.AGENT_STR, Configuration.AGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.PATIENT_STR, Configuration.PATIENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.PLACE_STR, Configuration.PLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.TIME_STR, Configuration.TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.MATERIAL_STR, Configuration.MATERIAL);
			propNameToInt.put(FrameExtractor.BASE+Configuration.INSTRUMENT_STR, Configuration.INSTRUMENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RELATIVE_PLACE_STR, Configuration.RELATIVEPLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ABSOLUTE_PLACE_STR, Configuration.ABSOLUTEPLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ABSOLUTE_TIME_STR, Configuration.ABSOLUTE_TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.HUMAN_AGENT_STR, Configuration.HUMANAGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ANIMAL_AGENT_STR, Configuration.ANIMALAGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.EFFECTOR_STR, Configuration.EFFECTOR);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RESOURCE_STR, Configuration.RESOURCE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RELATIVE_TIME_STR, Configuration.RELATIVE_TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.GOAL_STR, Configuration.GOAL);
			propNameToInt.put(FrameExtractor.BASE+Configuration.DESTINATION_STR, Configuration.DESTINATION);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ESSENCE_STR, Configuration.ESSENCE);
						
			HashSet <String> verbUris = verbExist.getTermURIs();
			HashMap<String, Verb> uriToVerbs = new HashMap<String, Verb>();
			
			uriToVerbs = this.getVerbInfo(verbUris, propNameToInt, true);
			
			return uriToVerbs;
		}
		
		
		
/**
 * Tiedoston lukija (TermoQuerystä)
 */

	private InputStream readFile(String filename)
							throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
        if (in == null) {
    		throw new IllegalArgumentException(
            	"File '" + filename + "' not found");
		}
		return in;
	}
	public TrieIndex getVerbIndex() {
		return this.verbExist;
	}
	
//------------------------------

	public String getYsoLabel(String uri) {
		return this.yso.getLabel(uri);
	}
	public String getYsoUpperStr(String uri) {
		String temp = "";
		ArrayList <String> al = this.yso.getContextInformation(uri);
		for(String s:al) {
			temp += s + " | ";
		}
		if(temp.length()>3) {
			temp = temp.substring(0, temp.length()-3);
		}
		return temp;
	}
	/**
	 * Palautetaan YsoQueryltä käsitteen prefLabel +
	 * yläluokan nimi
	 */
	
	public String getYsoLabelUp(String uri) {
		String temp = this.yso.getLabel(uri);
		ArrayList <String> al = this.yso.getContextInformation(uri);
		for(String s:al) {
			temp += " / " +s;
		}
		return temp;
	}
	
	public String getPlaceLabel(String uri) {
		return this.places.getLabel(uri);
	}
	
	public String getPlaceType(String uri) {
		return this.places.getType(uri);
	}
	
	public String getPlaceUpperStr(String uri) {
		String temp = "";
		ArrayList <String> al = this.places.getContextInformation(uri);
		for(String s:al) {
			temp += s + " | ";
		}
		if(temp.length()>3) {
			temp = temp.substring(0, temp.length()-3);
		}
		return temp;
	}
	
		
	public String getPaikkaLabelUp(String uri) {
		String temp = this.places.getLabel(uri);
		ArrayList <String> al = this.places.getContextInformation(uri);
		for(String s:al) {
			temp += " / " +s;
		}
		return temp;
	}

	
	public void addPlaces(boolean upper) {
		log.info("Lisätään paikat");
 	    this.addConcepts(FrameExtractor.PLACENODE, GeneralExtractor.CONCEPTNODES, false, upper, this.placeExist, this.places, this.placeRes,true, false); 	    
	}
	
	//palauttaa löydettyjen resurssien nimet laskevassa järjestyksessä
	public ArrayList<String> getPlaces() {
		return this.placeRes.getOrderedResults(true);
	}
	public ArrayList<String> getYso() {
		return this.ysoRes.getOrderedResults(true);
	}
	public ArrayList<String> getRegex() {
		return this.regexRes.getOrderedResults(true);
	}
	
/**
 * Palauttaa lÃ¶ydetyt paikkaresurssit hashMappina
 * muodossa hash: uri --> PokaResource
 * @return resurssit hashmappina
 */
	public HashMap<String, PokaResource> getPlaceResources() {
		return this.placeRes.getResources();
	}
/**
 * Vastaava ysolle.
 * @return
 */
	public HashMap<String, PokaResource> getYsoResources() {
		return this.ysoRes.getResources();
	}
/**
 * Vastaava regex-käsitteille
 * @return
 */	
	public HashMap<String, PokaResource> getRegexResources() {
		return this.regexRes.getResources();
	}
	public void addVerbs() { //TODO: multiword nyt false! (kävi koulunsa)
		log.info("Lisätään verbit");
		this.addConcepts(FrameExtractor.VERBNODE,GeneralExtractor.VERBNODES, false, false, verbExist,
				         verbs, verbRes, true, true);
	}
	
	public void addYso() {
		log.info("Lisätään YSO");
		this.addConcepts(FrameExtractor.CONCEPTNODE,GeneralExtractor.CONCEPTNODES, true, false, termExist,
						yso, ysoRes, true, true);
	}
	
	public void addNames() {
		log.info("Lisätään nimet");
		this.doc = ne.addNames();
	}
	
	public String getName(int id) {
		return this.ne.index.getName(id);		
	}
	
	public ArrayList<Name> getNames() {
		return this.ne.index.getNameObjects();
	}
	
	public String getHTML() { 
		String result;
		log.debug("Luodaan HTML...");
		result =  DocToHTML.convertHTML(this.getDocument(),"/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/stylesheets/pokadoc.xsl");
		log.debug("...HTML ok!");
		return result;
	}
	
	/**
	 * Entityfinderin käytössä, palauttaa tietyn lauseen käsitteet
	 * 
	 */
			
	public String getConceptsOfSentence(int sentence) {
		String expression = "/A/S[@no='"+sentence+"']/C|"+ //concept
							"/A/S[@no='"+sentence+"']/NA|"+ //name
							"/A/S[@no='"+sentence+"']/V|"+ //verb
							"/A/S[@no='"+sentence+"']/R|"+ //regex
						    "/A/S[@no='"+sentence+"']/PL";

		String result = "";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		try {
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			for(int i = 0; i < nodes.getLength(); i++){
				Node item = nodes.item(i);
				result += item.getNodeName()+" ";
				result += item.getChildNodes().item(0).getTextContent()+" ";
				result += item.getChildNodes().item(1).getTextContent()+"\n";
			}
		 } catch (XPathExpressionException e) {
		   			System.out.println("PokaDom / Verbs: " + e);
		 }
		 return result;
	}
/**
 * Palauttaa ominaisuudet (domainit), jotka on määritetty
 * frameontologiassa kyseiselle verbille.
 * @param uri verbi-käsitteen uri
 * @return kyseltävä verbi-mÃ¶ykky
 */
	public Verb getVerb(String uri) {
		return this.uriToVerb.get(uri);
	}
	
//-----------------------------------------------------
/**
 * Kun verbi on lÃ¶ytynyt, tällä metodilla palautetaan
 * FrameFinderille tieto siitä mitä rooleja kyseinen verbi sisältää
 * 
 * --> TÃTÃ NYKYÃ KÃYTETÃÃN KONSTRUKTORISTA LUOMAAN
 *     MÃPPÃYS URI --> VERB
 *     @param verbUris HashSet joka sisältää verbien urit
 *     @param prop mäppäys ominaisuuden urista vastaavaan lukuarvoon, jota käytetään 'apuindeksinä'
 *     @param subclasses jos true, ulotetaan rajoite koskemaan myÃ¶s aliluokkia
 */	

	private HashMap<String, Verb> getVerbInfo(HashSet<String> verbUris, 
			                                  HashMap<String, Integer> prop, 
			                                  boolean subclasses) {

		//uri --> Verbiolio mäppäys
		HashMap <String, Verb> result = new HashMap<String, Verb>();

		//TODO: purkka, joka lisää hashiin kaikki verbit (=ei tarvi iteroida
		//vaikeasti läpi verbejä, joilla ei ominaisuuksia (poistetaan myÃ¶hemmin)
		//--> käytÃ¶ssä tällä hetkellä jotta nähdään dokumentista mitä verbejä verbistÃ¶ sisältää
		for(String s: verbUris) {
			result.put(s, new Verb(s));
		}
		
//		kehysominaisuuksien (agent, instrument...) urit

//		Set <String> propertyUris = prop.keySet(); 
		
		ExtendedIterator restrictions = this.m.listRestrictions();
		while(restrictions.hasNext()) {
			Restriction rr = (Restriction)restrictions.next();
			if(rr.isAllValuesFromRestriction()) {
				AllValuesFromRestriction av = rr.asAllValuesFromRestriction();

			    log.debug( "AllValuesFrom class " +
			    			av.getAllValuesFrom().getURI() +
	                        "\nOn property " + av.getOnProperty().getURI() );	   
			    
			    Integer propValue = prop.get(av.getOnProperty().getURI());
			    if(propValue == null) {
			    	log.error("Urille EI VASTINETTA FRAMEOMINAISUUKSISSA!");
			    	
			    	log.info("AllValuesFrom class " +
	    			av.getAllValuesFrom().getURI() +
                    "\nOn property " + av.getOnProperty().getURI());
			    	
			    	continue;		    
			    	
			    }
			    			    							    			    			    
				this.traverseSubClasses(rr, result, verbUris, av.getAllValuesFrom().getURI(),
										propValue );
					
				
			}
				
		}
				
/*			
			result = this.verbToHash(resourceURI, property, result, prop);
					
			if(subclasses) {
				this.checkSubClasses(or, property, result, prop);							
			}
*/						
		return result;
	}
/**
 * iteroi annetun luokan kaikki aliluokat: jos annetun luokan aliluokilla
 * on aliluokkia, iteroidaan ne rekursiivisesti
 * @param oc luokka, jonka aliluokat iteroidaan
 * @param result vastausjoukko, hashmap uri --> verbin tiedot
 * @param verbUris luettelo verbien ureista
 * @param range luokka, jonka rangesta on kysymys
 * @param ominaisuus, jota rajoite koskee
 */
	
	private void traverseSubClasses(OntClass oc,
									HashMap <String, Verb> result,
									HashSet <String> verbUris,
									String range,
									int property) {
		
		log.debug("TraverseSubClasses");
		
		ExtendedIterator subclassesList = (ExtendedIterator)oc.listSubClasses();
	    while(subclassesList.hasNext()) {
	    	OntClass sub = (OntClass)subclassesList.next();
	    	if(!sub.isAnon()) { //anonyymisolmut ohitetaan
	    		
	    		String verbUri = sub.getURI();
		    		
	    		if(verbUris.contains(verbUri)) {
	
		    		log.debug("\nRajoitettava luokka: "+verbUri+"\n");
		    		
		    		Verb v = result.get(verbUri);
		    		if(v==null) {
		    			v = new Verb(verbUri);
		    		}
		    		v.setRestriction(property, range);
		    		
	    		}
		    	if(sub.hasSubClass()) {
		    		this.traverseSubClasses(sub, result, verbUris, range, property);
		    	}

			}
	    
	    }
	}
	
/**
 * Verbin lisäys parametrina annettuun hashMappiin
 * 
 * Jos näyttää siltä, poistetaan hashmap parametreista
 * 
 * @param verbURI käsitteen uri
 * @param property lisättävä rooli
 * @param result uri --> Verbi -mäppäys
 * @param prop uri --> kokonaisluku -mäppäys (
 * 
 * TODO: Verbin lisäykseen pitää tallettaa tieto rangesta
 */
	private HashMap<String, Verb> verbToHash(String verbURI, String property, 
			               HashMap <String, Verb> result, HashMap<String, Integer> prop) {
		Verb temp;

		log.debug("verbi: "+verbURI);
		log.debug("ominaisuudella "+property+" saadaan hashista: "+prop.get(property));
		
		if((temp =result.get(verbURI))==null) {
			log.debug("Verbiä '"+verbURI+"' ei resultsetissä, lisätään.");
			temp = new Verb(verbURI);
			temp.setValue(prop.get(property));
			result.put(verbURI, temp);
			log.debug("Verbi mapissa: "+result.containsKey(verbURI));
		} else {
			log.debug("Verbi lÃ¶ytyy, lisätään vain ominaisuus.");
			temp.setValue(prop.get(property));
		}
		
		return result;	
	}
/**
 * Tarkistaa onko luokalla aliluokkia. Esim jos domainiksi määritelty frame JA
 * framelta lÃ¶ytyy ominaisuus x, liitetään ominaisuus x kaikkiin aliluokkiin
 * eksplisiittisesti
 * @param verbResource
 * @param property
 * @param result
 * @param prop
 * @return
 */	
	private HashMap<String, Verb> checkSubClasses(OntClass verbResource, String property,
					HashMap <String, Verb> result, HashMap<String, Integer> prop) {
		ExtendedIterator sub = verbResource.listSubClasses();
		String resourceURI;
		while(sub.hasNext()) {
			log.debug("Luokalla '"+verbResource.getURI()+"' aliluokkia.");
			OntClass subclass = (OntClass)sub.next();

			//rekursio: Depth-First-Search, tarkistetaan aliluokan aliluokat!									
			result = this.checkSubClasses(subclass, property, result, prop);
			
			resourceURI = subclass.getURI();
			log.debug("Aliluokka: '"+ resourceURI+"'");	
			result = this.verbToHash(resourceURI, property, result, prop);
		}
		return result;
	}
	
	
//-----------------------------------------------------
//REGEX toisteisena täällä (=lÃ¶ytyy myÃ¶s GeneralExtractorista, jota ei käytetä tässä
// TODO: jos siltä tuntuu, ota pois YSO-kytkentä parametrisoimalla se
//-----------------------------------------------------

	public void addRegex(String regexp, String uri) {
		this.regexToC.put(regexp, uri);
	}
	public void removeRegex(String regexp) {
		this.regexToC.remove(regexp);
	}
/**
 * 
 * @param nodeNames
 * @param upper
 * @param tokenIndex jos true, lisätään lÃ¶ydettyihin resursseihin esiintymän sijainti
 */	
	public void addRegexConcepts(String [] nodeNames, boolean upper, boolean tokenIndex) {
		Iterator <String> i = regexToC.keySet().iterator();
		String tempRegex;

		String expression = "";
		for(String s: nodeNames) {
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);
        NodeList nodes;
		String temp, currURI;
		String concepts;
		XPath xpath = XPathFactory.newInstance().newXPath();
		while(i.hasNext()) {
		
			tempRegex = i.next();

			try {
				currURI = this.regexToC.get(tempRegex);
							
				if(upper) {
					concepts = currURI.substring(	
						currURI.indexOf("#"), currURI.length())+": "+
                            yso.getUpper(currURI);			
				} else {
					concepts = currURI+" | "; //purukumia	
						
				}
					 
				nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
        		for(int ii = 0; ii < nodes.getLength(); ii++){
          			Node item = nodes.item(ii);
          			
//         			log.debug(fi.seco.semweb.service.poka.util.debug.DomDebugger.printNode(item));

          			temp = item.getFirstChild().getTextContent(); //tekstimuoto
					log.debug("Regex tekstimuoto: "+temp);
					if(temp.matches(tempRegex)) {
						log.debug("REGEX MÃTSÃÃ");
						int sentence = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getTextContent());
						int token = Integer.parseInt(item.getAttributes().getNamedItem("no").getTextContent());
						
						
						if(tokenIndex) {
							int lastChild = item.getChildNodes().getLength()-1;
														
							try {
								//parsitaan alkuindeksi
								int start = Integer.valueOf(item.getChildNodes().item(lastChild).
											getChildNodes().item(0).getTextContent());
								//parsitaan loppuindeksi
								int end = Integer.valueOf(item.getChildNodes().item(lastChild).
											getChildNodes().item(1).getTextContent());
							
								log.debug("START: "+start+"\nEND: "+end);
							
								this.regexRes.addURI(currURI, sentence, token, true, start, end);
							} catch(NumberFormatException e) {
								log.error("Tokenin sisältämä paikkaindeksi virheellinen: luultavasti tokenIndexer poissa synkronista");
							} catch(NullPointerException e) {
								//TODO: fiksaa järkeväksi.
								log.error("Tokenin sisältämä paikkaindeksi virheellinen: luultavasti tokenIndexer poissa synkronista");
							}
							 			
						} else {
						//	System.out.println("ARVOT: "+sentence+", "+token+", "+s);
							this.regexRes.addURI(currURI, sentence, token, true);
						}
						this.appendConcept(FrameExtractor.REGEXNODE, item, concepts, false);
					}
				}
			} catch (XPathExpressionException e) {
	    		System.out.println(e);
		    }
		}

	}


	/**
	 * AddConceptsin viritelty versio FrameExtractor-softan
	 * käyttÃ¶Ã¶n: lisätään käsitteiden indeksointi päälle.
	 * 
	 * Lisää käsitteet dokumenttiin
	 * @param nodeType lisättävän solmun tunniste, esim 'C' (=CONCEPT)
	 * @param nodeNames solmut, joista käsitettä etsitään, esim BS, AD, UP
	 * @param multiword true, jos halutaan etsiä monisanaisia käsitteitä
	 * @param upper true, jos solmun tietoihin halutaan lisätä yläkäsite
	 * @param wordIndex indeksi, josta sanoja halutaan etsiä
	 * @param kb tietämyskanta, josta haetaan tieto yläkäsitteestä
	 * @param instBase resurssikanta, jonne lÃ¶ydetyt resurssit t tallennetaan
	 * @param tokenIndex jos true, lisätään myÃ¶s tokenien indeksit PokaResourceihin
	 * 
	 * @param split jos true, tarkistetaan sanan osat: lippu#tanko --> tanko
	 */

	public void addConcepts(String nodeType, 
							String [] nodeNames, 
							boolean multiword, 
							boolean upper, 
							TrieIndex wordIndex, 
							TermoQuery kb, 
							FoundResources instBase,
							boolean tokenIndex,
							boolean split) {
	  	    
		String expression = "";
		for(String s: nodeNames) {
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);
		
		log.debug("XPATH / etsitään käsitettä solmuista "+expression);
			
	    NodeList nodes;
		String temp = "";
		XPath xpath = XPathFactory.newInstance().newXPath();

		try {

			//ota docin käsitesolmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			
	        for(int i = 0; i < nodes.getLength(); i++){
	        	Node item = nodes.item(i);
				
	        	if(multiword) {
	        		log.debug("i ennen  : "+ i);
	        		i = this.checkLongTerm(nodeType, item, nodeNames, i, wordIndex, kb, upper, instBase);
	        		log.debug("i jälkeen: "+ i);
	        	}
		        	
	        	item = nodes.item(i); //mahd. iteroitu käsite
	        	if(item == null) {
	        		log.debug("iteroitavat solmut lopussa");
	        		break;
	        	}
	        	temp = item.getChildNodes().item(1).getTextContent();
	        	log.debug("Etsitään vastaavuutta yksittäiselle tokenille: '"+temp+"'");
	           	String concepts = "";
	           		
				if(this.containsURI(temp, wordIndex, split)) { //vanha osa PokaDomista
					
					//jos käsitemuoto on lyhyt
					if(this.hasShortForm()) {
						temp = this.getShort();
						log.debug("\t--> lyhyt vastine lÃ¶ytyy");
					} else {
						log.debug("\t--> pitkä vastine lÃ¶ytyy");
					}
					HashSet <String> foundUris = wordIndex.getUris(temp);
					//TODO: virheiden käsittely
					int sentence = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getTextContent());
					int token = Integer.parseInt(item.getAttributes().getNamedItem("no").getTextContent());		
					
					for(String s: foundUris) {
						if(tokenIndex) {
							int lastChild = item.getChildNodes().getLength()-1;
							
							//parsitaan alkuindeksi
							int start = Integer.valueOf(item.getChildNodes().item(lastChild).
											getChildNodes().item(0).getTextContent());
							//parsitaan loppuindeksi
							int end = Integer.valueOf(item.getChildNodes().item(lastChild).
											getChildNodes().item(1).getTextContent());
							
							log.debug("START: "+start+"\nEND: "+end);
							
							instBase.addURI(s, sentence, token, true, start, end);							
							 			
						} else {
						//	System.out.println("ARVOT: "+sentence+", "+token+", "+s);
							instBase.addURI(s, sentence, token, true);
						}
					}
					if(upper) {
						concepts = this.upperClasses(foundUris, kb);
					} else {
						for(String s: foundUris) { //wordIndex.getUris(temp)
							concepts += s +" | ";
						}
//							HUOM:Â ei saa poistaa, käytetään OpasExtractorissa!
//							TODO: ehkä vois trimmata vikan '|' pois ja pistää metodit kauniimmiksi...
//							--> Muutoksia OpasExtractorin metodiin addInstanceUris
//							concepts = concepts.substring(0, concepts.length()-3);//viim. ' | ' pois

							//concepts = wordIndex.getConceptString(temp);
					}
					log.debug("\tKäsite: "+ concepts);
							
					this.appendConcept(nodeType, item, concepts, true);//true, SOLMULLA LEMMA, käsitteen lisäys
				}
	        }
	         		
		} catch (XPathExpressionException e) {
			System.out.println(e);
		}

	}
	
	/**
	 * Iteroidaan solmusta eteenpäin, tarkistetaan
	 * muodostavatko peräkkäiset tokenit käsitteen
	 * Jos käsite lÃ¶ytyy, se merkataan suoraan
	 * appendConceptilla dokumenttiin.
	 * 
	 * 
	 *  @param tokenIndex jos true, lisätään myÃ¶s tokenien indeksit PokaResourceihin
	 * 
	 */
	protected int checkLongTerm(String nodeType, 
							    Node item, 
							    String [] nodeNames, 
							    int counter, 
							    TrieIndex wordIndex, 
							    TermoQuery kb, 
							    boolean upper,
							    FoundResources instBase,
							    boolean tokenIndex) {

		ArrayList <Node> conceptTokens = new ArrayList <Node>();
		int increment = 0;
		String preceding = null;
//		Node precedingNode = null;		
		String nodetypesRegex = "";

		for(String types:nodeNames) {
			nodetypesRegex += types+"|";
		}			
		
		nodetypesRegex = nodetypesRegex.substring(0,nodetypesRegex.length()-1);
		
		String temp = item.getChildNodes().item(1).getTextContent()+" ";
		
		conceptTokens.add(item);
		item = this.nextNode(item);
		
		if(item == null) {
			log.debug("Termi '"+temp+"' lauseen lopussa, siirrytään yksittäiseen");
		}
		while(item != null) {

			temp += item.getChildNodes().item(1).getTextContent();  			
       		log.debug("Etsitään termiä: '"+temp+"'");
			
			if(item.getNodeName().matches(nodetypesRegex)) {
				log.debug("\t--noden tyyppi mätsää, kasvatetaan laskuria--");
				increment++;
			}/* else { //TODO: keksi jotain parempaa: kys. versio hylkää monisanaisen käsitteen, vaikka preceding olisikin käsite.
				log.debug("\t--node väärää tyyppiä, ("+item.getNodeName()+") LOPETETAAN!");
				return counter;
			}*/
       		
       		if(wordIndex.containsPrefix(temp)) {
           		log.debug("Sana '"+temp+"' saattaa jatkua...");
           		preceding = temp;
           		List <String> l = wordIndex.matchByPrefix(temp);
           		for(String s:l) {
           			log.debug("\t--> '"+s+"'");
           		}
           		temp += " ";
           		conceptTokens.add(item);
           		item = this.nextNode(item);          		
           	} else {   		
         		log.debug("\t--> Termiä ei lÃ¶ydy, tarkastetaan edellinen...");
           		if(preceding != null) {
           			if(wordIndex.contains(preceding)) {

    					HashSet <String> foundUris = wordIndex.getUris(preceding);

//    					TODO: virheiden käsittely
    					           				
           				String concepts = "";
           				if(upper) {
           					concepts = this.upperClasses(foundUris, kb); //wordIndex.getUris(preceding)
           				} else {
           					for(String s: wordIndex.getUris(preceding)) {
    							concepts += s +" | ";
    						}
           					//concepts = wordIndex.getConceptString(temp);
           				}
           				log.debug("\tMonisanainen käsite: "+ concepts);
    					
    					int last = conceptTokens.size()-1;
    					
    					Node foundToken = conceptTokens.get(last);
    					
    					int sentence = Integer.parseInt(foundToken.getParentNode().getAttributes().getNamedItem("no").getTextContent());
    					int token = Integer.parseInt(foundToken.getAttributes().getNamedItem("no").getTextContent());
    					log.debug("Monisanaisen viimeinen token: lause "+sentence+", token "+token);
    					
    					this.appendConcept(nodeType, foundToken, concepts, true);	
   						
    					for(String s: foundUris) { //jokaista uria kohti lisätään sijainti
    						if(tokenIndex) {
    							int lastChild = item.getChildNodes().getLength()-1;
    							
    							//parsitaan alkuindeksi
    							int start = Integer.valueOf(item.getChildNodes().item(lastChild).
    											getChildNodes().item(0).getTextContent());
    							//parsitaan loppuindeksi
    							int end = Integer.valueOf(item.getChildNodes().item(lastChild).
    											getChildNodes().item(1).getTextContent());
    							
    							log.debug("START: "+start+"\nEND: "+end);
    							
    							instBase.addURI(s, sentence, token, true, start, end);							
    						} else {
    							instBase.addURI(s, sentence, token, true);
    						}
    					}
    					
    					for(int i=0; i<last;i++) {
    						foundToken = conceptTokens.get(i);
    						
    						this.appendConcept(nodeType, foundToken, ""+i, true);
    						
//    						sentence = Integer.parseInt(foundToken.getParentNode().getAttributes().getNamedItem("no").getTextContent());
        					token = Integer.parseInt(foundToken.getAttributes().getNamedItem("no").getTextContent());
        					log.debug("Monisanaisen aiempi token: lause "+sentence+", token "+token);
        					for(String s: foundUris) { //jokaista täsmäävää uria kohti lisätään tokenin sijaintitieto
        						if(tokenIndex) {
        							int lastChild = item.getChildNodes().getLength()-1;
        							
        							//parsitaan alkuindeksi
        							int start = Integer.valueOf(item.getChildNodes().item(lastChild).
        											getChildNodes().item(0).getTextContent());
        							//parsitaan loppuindeksi
        							int end = Integer.valueOf(item.getChildNodes().item(lastChild).
        											getChildNodes().item(1).getTextContent());
        							
        							log.debug("START: "+start+"\nEND: "+end);
        							
        							instBase.addURI(s, sentence, token, true, start, end);							
        						} else {
        							instBase.addURI(s, sentence, token, true);
        						}

        					}	
    					}   					
//TODO: tarkista viat!
    					return counter+increment+2;
           			}
           		} else {
           			log.debug("\t--> Preceding oli null...");
           		}	
        		return counter;
           	}	      		
   		}
		return counter;
	}
	
//	-----------------------------------------------------


	
	public static void main (String [] args) 
							throws IOException {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka");
		hep.setLevel(Level.INFO);
		
		FrameExtractorCook fe = new FrameExtractorCook();
		fe.setDocument("/home/alm/termo/FRAME/Asumistesti.txt");
		fe.addNames();
		fe.addPlaces(true);
		fe.addVerbs();
//		fe.addYso(); 
		fe.addYso();
		
		
//		System.out.println(fe.getDocument());
		fe.writeToFile("/home/alm/termo/penkki/output.html",fe.getDocument());
		
	}
	
	
}
