package fi.seco.semweb.service.poka.frame;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.GeneralExtractor;
/**
 * tulee korvaamaan EntityFinderin: TODO: sielt kamat tnne.
 * tunnistus: kohdistetaan lauseen main verbin kanssa, lhdetn
 * etenemn fdg:n kaaria (ja miksei tokenien lheisyydenkin mukaan)
 * pitkin muihin sanoihin ja etsitn
 * sopivia vastineita eri rooleihin.
 */
public class FrameFinder {
	private static final Log log = LogFactory.getLog(FrameFinder.class);
	
	private HashMap <Integer, Sentence> sentences;
	private FrameExtractor fe;
	
	private FrameResolver resolver;
	
	public FrameFinder() {
		this.fe = new FrameExtractor();
		this.sentences = new HashMap<Integer, Sentence>();
		
		log.debug("Asetetaan FrameResolver");
		this.resolver = new FrameResolver();
		log.debug("FrameResolver ok!");
		
	}
	
	public void setDocument(String filename) {
		this.fe.setDocument(filename);
	}
	public String getDocument() {
		return this.fe.getDocument();
	}
	public String getDocumentAsText() {
		return this.fe.getDocumentAsText();
	}
/**
 * Parsii oletusarvoilla dokumentista esiin
 * ihmisten nimet, paikat, verbit ja yso-ksitteet
 *
 */
	
	public void preParse() {
		fe.addRegex("[1-2][0-9][0-9][0-9]", "http://yso.fi/YSO#vuosi");
		fe.addRegex("\\d(\\d)?\\.\\d(\\d)?\\.\\d\\d\\d\\d","http://yso.fi/YSO#pivmrt");
		fe.addRegexConcepts(FrameExtractor.NUMERALNODES, false);
		fe.addNames();
		fe.addPlaces(true);
		fe.addVerbs();
//		fe.addYso(); korvataan seuraavilla
		fe.addRelativeTime(); //TODO: poistetaan kytst jos tarpeen!
		fe.addAnimals();
		fe.addRelativePlace();
		fe.addYso();
		this.parseDependencies();
	}
	
	public void provaParseTest() {
		String expression = "/A/S/V";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			nodes = (NodeList)xpath.evaluate(expression, this.fe.getXMLDoc(), XPathConstants.NODESET);
 			log.debug("Iteroidaan verbisolmut");
 			for(int i = 0; i < nodes.getLength(); i++){
 				log.debug("Solmu "+i);
 				Node item = nodes.item(i);
           		int sentNumba = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getNodeValue());
           		int tokenNumba = Integer.parseInt(item.getAttributes().getNamedItem("no").getNodeValue());

         		System.out.println(fe.getSentence(sentNumba));   		         		         		
           		Sentence sent = this.sentences.get(sentNumba);
         		System.out.println("Lause "+sentNumba+": "+sent.getText());
         		
         		String desc = sent.getToken(tokenNumba).getDesc();
         		log.debug("DESC: '"+desc+"'");

         		
         		int uriSeparator = desc.indexOf('|');
         		String uri;
         		while(uriSeparator != -1) {
         			int separator = desc.indexOf('|');
         			uri = desc.substring(0, separator-1);
         			log.debug("uri on '"+uri+"'");
         			
         			Verb v = this.fe.getVerb(uri);
               		if(v==null) {
               			log.error("V NULL");
               			break;
               		}
               		//this.resolver.getFrames(sent, v, tokenNumba);
               		Frame f = this.resolver.getFrame(sent, v, tokenNumba);
               		System.out.println(f.toString(sent));
               		
         			desc = desc.substring(separator+2);
         			log.debug("loput: '"+desc+"'");         			
         			
         			uriSeparator = desc.indexOf('|');
         		}
    		
           		
           		
 			}
 			
    	} catch (XPathExpressionException e) {
 				e.printStackTrace();
 				return;
 	 	}	
	}
	
	public void parseTest() {
		String expression = "/A/S/V";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			nodes = (NodeList)xpath.evaluate(expression, this.fe.getXMLDoc(), XPathConstants.NODESET);

 			for(int i = 0; i < nodes.getLength(); i++){
           		Node item = nodes.item(i);
           		int sentNumba = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getNodeValue());
           		int tokenNumba = Integer.parseInt(item.getAttributes().getNamedItem("no").getNodeValue());

         		System.out.println(fe.getSentence(sentNumba));   		         		         		
           		Sentence sent = this.sentences.get(sentNumba);
         		System.out.println("Lause "+sentNumba+": "+sent.getText());
         		
         		String desc = sent.getToken(tokenNumba).getDesc();
         		//infosolmun lopussa '|'
         		//TODO: useiden tsmvien verbien ksittely
         		desc  = desc.substring(0,desc.length()-3);

         		Verb v = this.fe.getVerb(desc);
           		if(v==null) {
           			log.error("V NULL");
           			break;
           		}
           		if(v.hasAgent()) {
           			HashSet<Token> refs = this.getRefOfType(sent, tokenNumba, "subj:");
              		if(!refs.isEmpty()) {
             			for(Token t:refs) {
             				
             				if(t.getNodeType().matches("NA|PN")) {
//             	         		System.out.println(fe.getSentence(sentNumba));
//             	         		System.out.println("Lause "+sentNumba+": "+sent.getText());
             					System.out.println("Agentti+Toiminta: '" + t.getText()+" "+item.getChildNodes().item(0).getTextContent()+"'");

             	           		HashSet<Integer> allRefs = sent.getRefs(tokenNumba);
             	           		if(!allRefs.isEmpty()) {
             	           			for(Integer ii: allRefs) {
             	           				System.out.println("Ref "+i+":"+sent.getToken(ii).getText());
             	           			}
             	           		}
             	           		
             	           		if(v.hasAbsolutePlace()) {
             	           			HashSet<Token> locRefs = this.getRefOfType(sent, tokenNumba, "loc:");
             	           			if(!refs.isEmpty()) {
             	           				for(Token tt:locRefs) {
             	           					if(tt.getNodeType().matches("PL")) {
             	           						System.out.println("Paikka          : '" + tt.getText()+"'");
             	           					}
             	           				}
             	           			}
             	           		}
             	           		
             				}
               			}
              			
              		}/* else {
              			System.out.println("--> Syntaktista agenttia ei lydy!");
              		}*/
           		}          		
           		
/*           		
           		HashSet<Integer> allRefs = sent.getRefs(tokenNumba);
           		if(!allRefs.isEmpty()) {
           			for(Integer ii: allRefs) {
           				System.out.println("Ref "+i+":"+sent.getToken(ii).getText());
           			}
           		}
 */
  			}
 			
    	} catch (XPathExpressionException e) {
 				e.printStackTrace();
 				return;
 	 	}
	}

/**
 * Parsii esiin kehykset lauseista
 *
 */
	
//	TODO: dynaamisesti vai staattisesti, onko tarvetta
//	tallettaa kaikki lauseet parsinnassa taulukkoon?
	public void parseDependencies() { 
		
		int count = fe.sentenceCount(),
	    i = 1;
		
		while(i < count) {
			log.debug("---lause "+i+" ---------");
			log.debug(this.fe.getSentence(i));
			Sentence s = this.addSentence(i);
			this.sentences.put(i, s);
//			koo.getFrames(s);
//			System.out.println(koo.poka.getConceptsOfSentence(i)+"\n");
			i++;
		}
		
	}
	
	
	private Sentence addSentence(int sentence) {
		String expression = "/A/S["+sentence+"]";
       	XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			Node n = (Node)xpath.evaluate(expression, this.fe.getXMLDoc(), XPathConstants.NODE);
 			return new Sentence(n);
    	} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
 	 	}
	}
	
	
	/**
	 * Palauttaa parametrina annettavaan tokeniin viittaavat tokenit
	 * joiden dependenssityyppi on 'type' (esim 'obj:', 'subj:',  'main:')   
	 * HashSettin
	 * 
	 * @param s Sentence, ko. lause
	 * @param referent Token, johon viitataan
	 * @param type dependenssityyppi, jota etsitn
	 * @return 
	 */
	
	public HashSet<Token> getRefOfType(Sentence s, int referent, String type) {
		HashSet <Token> result = new HashSet <Token> ();
		if(s.hasRefs(referent)) { 
			HashSet <Integer> referrers = s.getRefs(referent);
			Token t;
			for(int i: referrers) {
				t = s.getToken(i);
				if(t.getDType().equals(type)) {
					result.add(t);
				}
			}
		}
		return result;
	}
	
	public static void main(String [] args) throws IOException {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka.name");
		hep.setLevel(Level.OFF);
		Logger de = Logger.getLogger("fi.seco.semweb.service.poka.DocumentExtractor");
		de.setLevel(Level.OFF);
		Logger frame = Logger.getLogger("fi.seco.semweb.service.poka.frame");
		frame.setLevel(Level.INFO);
	
		new FrameOntologyBuilder(); //rakennetaan ontologiat uudestaan

//Tiedoston enkoodaus lytyy XFDGConnectista.
//TODO: voisi tuoda tnne asti.
		FrameFinder ff = new FrameFinder();
//		ff.setDocument("/home/alm/termo/FRAME/Asumistesti.txt");
//		ff.setDocument("/home/alm/aaparser/text/LarinParaske.html");
//		ff.setDocument("/home/alm/aaparser/text/AckteAinoHTML.txt");
//		ff.setDocument("/home/alm/aaparser/text/LonkkaEliasHTML.txt");
//		ff.setDocument("/home/alm/aaparser/text/AleksisKivi.html");
		ff.setDocument("/home/alm/aaparser/text/Gallenkallela.html");
		
		ff.preParse();

//		System.out.println(ff.getDocument());
		ff.provaParseTest();
//		ff.parseTest();
		
	}

	
	
}
