/**
 * HUOM! Ei listty SVN:n, ei kyll tarpeenkaan
 * 
 * Kehysontologian kuvauksen hahmottelu. tnne tulee lydetyt
 * framet talteen. FrameFinder kytt
 */

package fi.seco.semweb.service.poka.frame;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdql.Query;
import com.hp.hpl.jena.rdql.QueryEngine;
import com.hp.hpl.jena.rdql.QueryException;
import com.hp.hpl.jena.rdql.QueryExecution;
import com.hp.hpl.jena.rdql.QueryResults;
import com.hp.hpl.jena.rdql.ResultBinding;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

public class FrameKokeilua {
    
	//nimiavaruus
	private final String BASE = "http://www.seco.hut.fi/ns/2006/02/alm#";
	
	private OntModel frame;
	private Property agent;
	private Property patient;
	
	public FrameKokeilua(String frameModel) {
		try {
			InputStream in = this.readFile(frameModel);
			frame = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_TRANS_INF);
			frame.read(in, BASE);
			this.agent = frame.getProperty(BASE+"agent");
			this.patient = frame.getProperty(BASE+"patient");
		} catch(IOException e) {
			
		}
	}
	public void addInstance(String agentInst, String verbLemma, String objectInst) {
		List <String> verbConcepts = this.getVerbs(verbLemma);
		Individual  inst,
					action;
		
		Property p = frame.getProperty(BASE+"action");
		
		for(String concept:verbConcepts) {
			
			inst = 
			frame.getOntClass(concept).
					createIndividual(concept+System.currentTimeMillis());
						
			action = frame.getOntClass(BASE+"Act").createIndividual();
			action.addProperty(agent, agentInst);
			action.addProperty(patient,objectInst);
			
			inst.addProperty(p, action);
			System.out.println("Luotiin ilmentym:\n"+this.getInstance(inst));
		}
	}
/**
 * Kokeilu anononyymin toimintainstanssin luomisesta
 * ja liittmisest munintainstanssiksi.
 *
 */
	
	public void muninta() {
		
		//anon individual
		Individual action = frame.getOntClass(BASE+"Act").createIndividual();
		
		action.addProperty(frame.getProperty(BASE+"agent"), "Kana");		
		action.addProperty(frame.getProperty(BASE+"environment"), "kanalassa");
		action.addProperty(frame.getProperty(BASE+"patient"), "munan");
			
		OntClass op = frame.getOntClass(BASE+"muninta");
		Individual m1 = op.createIndividual(BASE+"muninta_1");
		Property p = frame.getProperty(BASE+"action");
		m1.addProperty(p, action);		
		
//		m1.addProperty
			
	}
	
	public HashSet<String> getVerbs() {

		String qString = 
			"SELECT ?verbLabel " +
			"WHERE (?concept, <oma:verbLabel>, ?verbLabel) " +
	        "USING oma FOR <http://www.seco.hut.fi/ns/2006/02/alm#>";
		
		Query q = null;
		try {
			q = new Query(qString);
		} catch(QueryException e) {
			System.out.println("Virheellinen kyselymerkkijonon sislt:\n"+qString+"\nVirhe:"+e);
		}
		q.setSource(frame);	
		QueryExecution qe = new QueryEngine(q);
		QueryResults results = qe.exec();

		HashSet <String> set = new HashSet <String> (600); //init 600
		for(Iterator i = results;i.hasNext();) {
			ResultBinding res = (ResultBinding)i.next();
			set.add(((Literal)res.get("verbLabel")).getString());
		}
		return set;
	}
	
	public List<String> getVerbs(String verb) {

		String qString = 
			"SELECT ?concept " +
			"WHERE (?concept, <oma:verbLabel>, \""+verb+"\") " +
	        "USING oma FOR <http://www.seco.hut.fi/ns/2006/02/alm#>";
		
		Query q = null;
		try {
			q = new Query(qString);
		} catch(QueryException e) {
			System.out.println("Virheellinen kyselymerkkijonon sislt:\n"+qString+"\nVirhe:"+e);
		}

		q.setSource(frame);	
		QueryExecution qe = new QueryEngine(q);

		QueryResults results = qe.exec();
		
		List <String> result = new ArrayList<String>();
		for(Iterator i = results;i.hasNext();) {
			ResultBinding res = (ResultBinding)i.next();
			result.add(((Resource)res.get("concept")).getURI());		
		}
		return result;
	}
	
	
	private InputStream readFile(String filename)
		throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
		if (in == null) {
			throw new IllegalArgumentException(
					"File '" + filename + "' not found");
		}
		return in;
	}
	private String getInstance(Individual inst) {
		String result = "";
		
		result += "--------------------\n"+
				  inst.getURI()+" ("+inst.getRDFType().toString()+")\n"+
				  "--------------------\n";
		RDFNode anon = inst.getPropertyValue(frame.getProperty(BASE+"action"));
		Individual anonAction = (Individual)anon.as(Individual.class);
//		Resource r = (Resource)anon;
		
		Statement s = anonAction.getProperty(agent);
		result += "agent:    "+s.getLiteral()+"\n";
		s = anonAction.getProperty(patient);
		result += "patient: "+s.getLiteral()+"\n";

		return result;
	}
	
	public String toString() {
		String result = "";
		ExtendedIterator individuals = this.frame.listIndividuals();

		Individual temp;
		while(individuals.hasNext()) {
			temp = (Individual)individuals.next();

//			System.out.println(temp.getPropertyValue(frame.getProperty(BASE+"action")));
			RDFNode tempNode = temp.getPropertyValue(frame.getProperty(BASE+"action"));

			if(tempNode != null) {
/*
				System.out.println(temp.toString());
				System.out.println(tempNode.toString());
				System.out.println(tempNode.isURIResource());
				System.out.println(tempNode.isResource());
*/				
				if(tempNode.isResource()) {
					result += "------------\n";
					result += "Toiminta: "+temp.toString()+"\n";
					
					Resource i = (Resource)tempNode;
					result += "Agentti: "+i.getProperty(agent).getString()+"\n";
					result += "Patient: "+i.getProperty(patient).getString()+"\n";
				}
				
			}
			
		}
		return result;
	}
	
	
	public static void main(String [] args) {
		FrameKokeilua fr = new FrameKokeilua("/home/alm/termo/FRAME/frameUTF8.owl");
		fr.muninta();
		System.out.println(fr.toString());
		
		HashSet <String> vbs = fr.getVerbs();
		for(String ss: vbs) {
			System.out.println(ss);
//			System.out.println("Ksite: "+fr.getVerbs(ss));
		}
		
	}
	
}
