package fi.seco.semweb.service.poka.frame;

/**
 * Rakentaa kehysma(/)llin, joka sislt
 * kehysrakenteen (mys verbit), yson ja paikat  
 */

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;

import fi.seco.semweb.service.poka.Configuration;

public class FrameOntologyBuilder {
	private static final Log log = LogFactory.getLog(FrameOntologyBuilder.class);
	
	private OntModel frame;
	private String ysoPath = Configuration.getYsoNoYsa();

	private String ysaPath = Configuration.getYsaGroups();
	
	//TODO: pitisik list YSO-import YSA-Groups?
	
	private String placePath = Configuration.getPlaces();
	
	private OntDocumentManager manager;
	private OntModelSpec spec;
	
//	private String outputFile = FrameExtractor.FRAMEPATH;
	private String outputFile = "/home/alm/ontologies/ontologies/poka/frame/CookFull2.owl";
	
	public static final String DEFAULTNAMESPACE = "http://www.seco.hut.fi/frame/ns/2006/05/alm#";
	
	public FrameOntologyBuilder () 
						throws IOException {
		
		manager = new OntDocumentManager();
		manager.setProcessImports(false);
		
		spec = new OntModelSpec(OntModelSpec.OWL_MEM);
		spec.setDocumentManager(manager);
		
		frame = ModelFactory.createOntologyModel(spec);
		frame.createOntology(FrameOntologyBuilder.DEFAULTNAMESPACE);
		
				
//		frame.setNsPrefix("frame", DEFAULTNAMESPACE);
//		frame.setNsPrefix("yso", "http://yso.fi/YSO#");
//		frame.setNsPrefix("paikka", "http://www.yso.fi/kulttuuriSampo/paikat#");

		log.info("listn Frame");
//		this.addOntology(TrieIndex.getFrameVerbs(), "UTF-8");

		/*frame defaultiksi, ei namespacea*/
		this.addOntology(Configuration.getNeuFrameVerbs(), "UTF-8", "");
		
//		this.addOntology(TrieIndex.getNeuFrameVerbs(), "UTF-8", "frame");
		
		log.info("Frame ok");

		log.info("listn Paikat");
		this.addOntology(placePath, "UTF-8", "paikka");
		log.info("Paikat ok");

//		log.info("listn YSA-groups");
//		this.addOntology(ysaPath, "UTF-8", "ysa-groups");
//		log.info("YSA-GROUPS ok");
		
		log.info("listn YSO");
		this.addOntology(ysoPath, "UTF-8", "yso");
		log.info("YSO ok");
		
		log.info("Kirjoitetaan tiedostoon '"+this.outputFile+"'");
		this.writeToFile(outputFile);
		log.info("Tiedosto valmis!");
	}
	
	
	/**
	 * Jos TermoQuerylle luetaan malli useista tiedostoista, 
	 * tll metodilla voidaan list alkuperiseen malliin
	 * tavaraa muista tiedostoista (alkujaan: YSO-fyysinen + YSO-UO)
	 *
	 */
	public boolean addOntology(String filename, String encoding, String nsPrefix) {
		OntModel mAttach;
		try {
			InputStream in = this.readFile(filename);
			
			mAttach = ModelFactory.createOntologyModel(spec);
			mAttach.read(new InputStreamReader(in, encoding), "");
			
			String namespace = mAttach.getNsPrefixURI("");
			manager.loadImport(mAttach, namespace);

			this.frame.setNsPrefix(nsPrefix, namespace);
			this.frame.add(mAttach);
			
			return true;
		} catch(IOException e) {
			System.out.println("Error reading file '"+
  				filename+"'.");
			//System.exit(0);
			return false;
		}
	}
	
	/**
	 * Tiedoston lukija
	 */

	private InputStream readFile(String filename)
							throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
	       if (in == null) {
	   		throw new IllegalArgumentException(
	           	"File '" + filename + "' not found");
		}
		return in;
	}
	
	public void writeToFile(String filename) throws IOException {
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "UTF-8");
		frame.write(pw);
	}
	
	public static void main (String [] args) 
								throws IOException {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka.frame");
		hep.setLevel(Level.DEBUG);
		
		FrameOntologyBuilder fob = new FrameOntologyBuilder();
		
		
		
	}
	
	
}
