package fi.seco.semweb.service.poka.frame;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.name.NameExtractor;
import fi.seco.semweb.service.poka.Configuration;

/**
 * TODO
 * 
 * HUOM! Jos hitaus haittaa, tm on syyt parsia
 * jo alussa. Mys viittaukset tokenien vlill
 * voidaan muodostaa jrkevmmin: kertn 
 * kaksi listaa: viittaajat ja viitatut 
 * ja yhdistetn lopussa, nopeampaa nin.
 * Parsintaa voi olla vaikea tehd PokaDoctorin sax-parserissa,
 * ehk jrkevint on muuttaa sax-parser dom-parseriksi
 * ja sitten tehd tarvittavat hommat. 
 *  
 * Luultavasti hydyllist pttelyss, jossa joudutaan 
 * kyselemn lauseen sisltmien tokenien tietoja
 * useaan kertaan.
 * 
 * Kyseess tavaraluokka,
 * yhteen yksikkn kertn tavara
 * yksittisest lauseesta
 * 
 * Keskeisin elementti lauseessa on knteisten viittaussuhteiden
 * kirjanpito. Fdg:ss esim subj: viittaa main: -jseneen, mutta
 * knteist selvityst ei pystynyt suoraan tekemn. Metodilla
 * getRefs saadaan selville, mitk tokenit viittaavat tokeniin x.
 * 
 * --> pysytn alustavasti pelkiss Stringeiss (vaatetus?)
 * 
 * @author alm
 *
 */

public class Sentence {

	private static final Log log = LogFactory.getLog(Sentence.class);
/**
 * Staattinen hashset joka sislt ne verbit, jotka kuuluvat
 * provan sisll hylttviin tietyiss tapauksissa
 */
	private static final HashSet <String> DISCARDVERBS = 
			new HashSet <String>(Arrays.asList(Configuration.DISCARDVERBS));

	
	private HashMap <Integer, Token> tokens;
	
	//mppys tyyppi --> indeksi
	private HashMap <String, HashSet<Integer>> dependIndex;
	
	private String agent;
	private String patient;
	private String main;
	
	private ArrayList<String> agentAttr;
	private ArrayList<String> patAttr;
	private ArrayList<String> mainAttr;

	//refs sislt tokenkohtaisen tiedon, mitk muut
	//tokenit viittaavat kyseiseen tokeniin
	private HashMap <Integer, HashSet<Integer>> refs;
	
	private String mainComp;
	
	private int tokenCount;
	
	public Sentence(Node n) {
		
		this.tokenCount = n.getChildNodes().getLength();
		this.tokens = new HashMap <Integer, Token>();
		
		this.dependIndex = 
			new HashMap<String, HashSet<Integer>>();
		
		this.refs = 
			new HashMap <Integer, HashSet<Integer>>();
		
		this.parseTokens(n);
	}
	
	//mainindex voidaan tarkistaa katsomalla dep = -1
	//n = Sentence
	
	private void parseTokens(Node n) {
		
		NodeList nl = n.getChildNodes();
		Node temp;

		for(int i=0; i<this.tokenCount; i++) {
			
			temp = nl.item(i);
			if(temp.getNodeType()==Node.TEXT_NODE) {
				continue; //iteroi tyhjt ohi
			}
			
			if(temp.getNodeName().equals("P")) {
				return;
			}
			this.createToken(temp);		
			
		}
	}
	
	private int createToken(Node token) {
		
		Token t;
		NodeList info;
		
		t = new Token();
		t.setNodetype(token.getNodeName());
		
		int tokenID = Integer.parseInt(
				token.getAttributes().
				getNamedItem("no").getTextContent());

		
		t.setID(tokenID);
							
		info = token.getChildNodes();
		
		t.setText(info.item(0).getTextContent()); //VAKIO			
		
		log.debug("Parsitaan token "+tokenID+": "+t.getText());
			
		for(int ii=0;ii<info.getLength();ii++) {
			token = info.item(ii);
			if(token.getNodeType()==Node.TEXT_NODE) {
				continue;
			}
			
			String childNode = token.getNodeName();
			if(childNode.equals("L")) {
				t.setLemma(token.getTextContent());
			} else if(childNode.equals("MO")) {
				t.setMorpho(token.getTextContent());
			} else if(childNode.equals("SYN")) {
				t.setSyntax(token.getTextContent());
			} else if(childNode.equals("DEP")) {
								
				String dep = token.getTextContent();
				t.setDepend(dep);

				int dHead = Integer.parseInt(token.getAttributes().
						getNamedItem("h").getTextContent());
				t.setHead(dHead);
				
				log.debug("DEP lytyi: "+dep+", "+tokenID+" --> "+dHead);
				
				HashSet <Integer> depType;
				if((depType = this.dependIndex.get(dep)) == null) {
					depType = new HashSet <Integer>();
				}
				depType.add(tokenID);
				this.dependIndex.put(dep, depType);
				
				HashSet <Integer> hs;
				if((hs = this.refs.get(dHead)) == null) {
					log.debug("headist ei viittauksia, luodaan uusi HashSet");
					hs = new HashSet <Integer>();				
				}
				hs.add(tokenID);
				log.debug("listtiin refMp "+dHead+" --> "+tokenID);
				refs.put(dHead, hs);
			} else if(childNode.equals("I")) {
				t.setDesc(token.getTextContent());
			}
		}
		this.tokens.put(tokenID, t);
		
		return tokenID;
	}
	
	private String getElements(ArrayList<String> list) {
		String result = "";
		for(String i:list) {
			result += i + " ";
		}
		return result;
	}

	//palauttaa tiedon, mitk tokenit viittaavat
	//kyseiseen tokeniin, tyhj HashSet palautetaan
	//jos tokeneita ei ole.
	public HashSet<Integer> getRefs(int token) {
		HashSet <Integer> temp = this.refs.get(token);
		if(temp == null) {
			return new HashSet<Integer>();
		}
		return this.refs.get(token);
	}
	
	public boolean hasRefs(int token) {
		return this.refs.get(token)!=null;
	}
		
/**
 * Onko kyseisell tokenilla DHeadina token, joka on 'ei'
 * @param token
 * @return
 */
	public boolean hasNegation(int token) {
		return this.getToken(
				this.getToken(token).getDHead())
					.getText().equalsIgnoreCase("ei");
	}
	
	public Token getToken(int index) {
		return this.tokens.get(index);
	}
	public boolean tokenLemmaInDiscList(int index) {
		return DISCARDVERBS.contains(this.getToken(index).getLemma());		
	}
	public HashSet<Integer> getTokenByDepend(String dependType) {
		HashSet<Integer> result = this.dependIndex.get(dependType);
		if(result == null) {
			return new HashSet<Integer>();
		} else {
			return result;
		}
	}
	public String getText() {
		
		StringBuffer result = new StringBuffer(); 
		
		for(int i=1; i<tokens.size()+1;i++) {
			result.append(this.tokens.get(i).getText()+" ");
		}
		
		return result.toString();
		
	}
	
//apuvlineit: 	
/**
 * 	parsii infonodesta <I> urit esiin
 */
	private HashSet<String> getUriHash(String nodeInfo) {
		
		HashSet <String> result = new HashSet<String>();
		int sep = nodeInfo.indexOf('|'); //first occurence of '|'
		
		if(sep==-1) {
			log.error("VIRHE: <I>-solmun pitisi sislt '|' -merkki!");
			return result;
		}
		do {		
			String temp = nodeInfo.substring(0,sep).trim();
			result.add(temp);
			nodeInfo = nodeInfo.substring(sep+1);
			sep = nodeInfo.indexOf('|');
		} while(sep != -1);
		
		return result;
	}
	
//provan toimintaa varten totuusarvometodeja
	
	/**
	 * Palauttaa tiedon siit, onko token tyypiltn
	 * <NA> tai <PN> eli nimi tai pronomini
	 *  @param tokenin jrjestysnumero
	 *  @return true, jos token tyypiltn <NA> tai <PN>
	 */
	public boolean isPronounOrName(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.NAMENODE+
													  "|"+FrameExtractor.PRONOUNNODE);
	}
/**
 * Tarkistaa onko tokenin tyyppi <NA>
 * @param token
 * @return
 */
	public boolean isName(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.NAMENODE);
	}
/**
 * Tarkistaa onko tokenin tyyppi (eli nimi) <PN>
 * @param token
 * @return
 */
	public boolean isPronoun(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.PRONOUNNODE);
	}
	
	/**
	 * Palauttaa <I>-solmun sisltmn kuvauksen (=URIt)
	 * String-iteraattorina
	 */

	public Iterator<String> getDesc(int token) {
		String temp = this.getToken(token).getDesc();
		return this.getUriHash(temp).iterator();
	}
	
	
	public boolean isRelativeTime(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.RELATIVETIMENODE);
	}
	
	public boolean isYsoConcept(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.CONCEPTNODE);
	}
	//teksti tsm merkkijonoon 'joka' (sivulause-plause agentin tunnistuksessa)
	public boolean pronounIsJoka(int token) { //jos joka, verbi viittaa itse plauseen subjektiin.
		Token t = this.getToken(token);
		return t.getText().equals("joka") &&
			   t.getNodeType().equals(FrameExtractor.PRONOUNNODE);
	}
	
	/**
	 * Palauttaa tiedon onko kyseinen metodi tyypiltn paikka
	 * @param token tokenin jrjestysnumero
	 * @return true, jos token on tyypiltn <PL>
	 */
	public boolean isAbsolutePlace(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.PLACENODE);
	}
	/**
	 * Palauttaa true jos kysytty token on tyypiltn <RP>, relative place
	 * @param token
	 * @return
	 */
	public boolean isRelativePlace(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.RELATIVEPLACENODE);
	}
	/**
	 * Palauttaa true jos node tyyppi <R>, Regex
	 */
	public boolean isRegex(int token) {
		return this.getToken(token).getNodeType().matches(FrameExtractor.REGEXNODE);
	}
/**
 * Tarkistaa onko token-1 tyyppi <R>, Regex
 * @param token
 * @return -1 jos ei kumpikaan, muuten tokenin numero
 */
	public boolean absoluteTimeBefore(int token) {
		Token t;
		if((t = this.getToken(token-1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.REGEXNODE)) {
				return true;
			}
		}
		return false;
	}
	public boolean absoluteTimeAfter(int token) {
		Token t;
		if((t = this.getToken(token+1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.REGEXNODE)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean relativeTimeBefore(int token) {
		Token t;
		if((t = this.getToken(token-1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.RELATIVETIMENODE)) {
				return true;
			}
		}
		return false;
	}
	public boolean relativeTimeAfter(int token) {
		Token t;
		if((t = this.getToken(token+1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.RELATIVETIMENODE)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean absolutePlaceBefore(int token) {
		Token t;
		if((t = this.getToken(token-1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.PLACENODE)) {
				return true;
			}
		}
		return false;
	}
	public boolean relativePlaceBefore(int token) {
		Token t;
		if((t = this.getToken(token-1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.RELATIVEPLACENODE)) {
				return true;
			}
		}
		return false;
	}
/**
 * Tarkistetaan tll, onko olla-verbin yhteydess
 * oleva verbi sellainen, ett olla-verbi tunnistetaan
 * perfektiksi tai pluskvamperfektiksi. Jos nin
 * on olla-frame hyltn.
 * Ehdot ovat seuraavat:
 * -verbi-token on tyyppi (VB, tunnistamaton verbi tai V, tunnistettu)
 * -morpho sislt merkkijonon PCP2 (2. partisiippi)
 * -syntax on -MV (ei-finiittinen verbi)
 * -dependenssi on comp: (komplementti, predikatiivi)
 * 
 * @param token
 * @return
 */
	public boolean isDiscardVerb(int token) {
		Token t = this.getToken(token);
		String temp;
		//jos verbi
		if(t.getNodeType().matches("VB|V")) {
			System.out.println("S: ON VERBI");
			temp  = t.getMorpho();
			if(temp == null) {
				return false;
			}
			if(temp.matches(".*PCP2.*")) {
				System.out.println("S: ON PCP2");
				temp = t.getSyntax();
				if(temp == null) {
					return false;
				}
				if(temp.matches(".*-MV")) {
					System.out.println("S: ON -MV");
					temp = t.getDType(); 
					if(temp == null) {
						return false;
					}
					if(temp.matches("comp:")) {
						System.out.println("S: ON comp:");
						return true;
					}
				}
			}
		}
		return false;
	}
	
	public boolean absolutePlaceAfter(int token) {
		Token t;
		if((t = this.getToken(token+1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.PLACENODE)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean relativePlaceAfter(int token) {
		Token t;
		if((t = this.getToken(token+1)) != null) {
			if(t.getNodeType().matches(FrameExtractor.RELATIVEPLACENODE)) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * True, jos tokenin dependenssityyppi on 'subj:'
	 * @param token
	 * @return
	 */
	public boolean isDepSubj(int token) {
		return this.getToken(token).getDType().equals("subj:");
	}
	
	public boolean isDepLoc(int token) {
		return this.getToken(token).getDType().equals("loc:");
	}
	
	public boolean isDepGoa(int token) {
		return this.getToken(token).getDType().equals("goa:");
	}
	public boolean isDepCC(int token) {
		return this.getToken(token).getDType().equals("cc:");
	}
	public boolean isDepMain(int token) {
		return this.getToken(token).getDType().equals("main:");
	}
	public boolean isDepObj(int token) {
		return this.getToken(token).getDType().equals("obj:");
	}
	public boolean isDepObjOrComp(int token) {
		return this.getToken(token).getDType().matches("obj:|comp:");
	}
	public String toString() {
		
		
		StringBuffer result = new StringBuffer(); 

		for(int i=1; i<tokens.size()+1;i++) {
			result.append("-----token "+i+"---\n");
			result.append(this.tokens.get(i).toString());
		}
		
		return result.toString();
				
	}
	
}
