package fi.seco.semweb.service.poka.frame;

import java.util.ArrayList;
import java.util.HashMap;

import fi.seco.semweb.service.poka.Configuration;

/**
 * Yksittisen toiminnan / verbin roolit yhten mykkyn:
 * FrameExtractor toteuttaa metodin, jolta saadaan
 * Verbi-olio kun annetaan verbin URI. Palautetaan
 * Verb, joka sislt tiedon siit, mit ominaisuuksia
 * tietty verbi sislt. Esim: asumisella on agentti, paikka ja
 * aika
 * 
 * --> 21-08-06: listn allValues-rajoitteen mrittm luokkakohtainen 'range':
 *     esim: munimisessa agentti on yso:lintu, lukemisessa yso:henkilt
 * 
 * --> 22-08-06: tietty luokkakohtainen rajoite voi olla esimerkiksi 
 *     5 luokan rangen yhdistelm: tll olisi syyt sallia
 *     monijoukkoisuus rangen asettamisessa (FIXME)
 * 
 * @author alm
 *
 */
public class Verb {

	private String uri;
	//mppys lukuarvolta (kts. yll) totuusarvolle
	private HashMap <Integer, Boolean> roleExist = new HashMap<Integer, Boolean>();
	
	//mppys lukuarvolta rangeen (String, annetaan URIna)
	private HashMap <Integer, String> roleToUri = new HashMap <Integer, String>();
	
		
	public Verb(String uri) {
		this.uri = uri;	

		//TODO: allaolevat kokeilua, poistetaan myhemmin.
		this.setTime();
		this.setPlace();
		this.setDestination();
		this.setEssence();
	}

//olemassaolojen asettamiset (pit silytt koska kytetn vanhassa versiossa)
	public void setAgent() {
		this.roleExist.put(Configuration.AGENT, true);
	}
	public void setHumanAgent() {
		this.roleExist.put(Configuration.HUMANAGENT, true);
	}
	public void setAnimalAgent() {
		this.roleExist.put(Configuration.ANIMALAGENT, true); 
	}
	public void setEffector() {
		this.roleExist.put(Configuration.EFFECTOR, true);
	}
	public void setEssence() {
		this.roleExist.put(Configuration.ESSENCE, true);
	}
	
	public void setPatient() {
		this.roleExist.put(Configuration.PATIENT, true);
	}
	public void setPlace() {
		this.roleExist.put(Configuration.PLACE, true);
	}
	public void setTime() {
		this.roleExist.put(Configuration.TIME, true);
	}
	public void setMaterial() {
		this.roleExist.put(Configuration.MATERIAL, true);
	}
	public void setInstrument() {
		this.roleExist.put(Configuration.INSTRUMENT, true);
	}
	public void setRelativePlace() {
		this.roleExist.put(Configuration.RELATIVEPLACE, true);
	}
	public void setAbsolutePlace() {
		this.roleExist.put(Configuration.ABSOLUTEPLACE, true);
	}
	public void setAbsoluteTime() {
		this.roleExist.put(Configuration.ABSOLUTE_TIME, true);
	}
	public void setRelativeTime() {
		this.roleExist.put(Configuration.RELATIVE_TIME, true);
	}
	public void setResource() {
		this.roleExist.put(Configuration.RESOURCE, true);
	}
	public void setGoal() {
		this.roleExist.put(Configuration.GOAL, true);
	}
	public void setDestination() {
		this.roleExist.put(Configuration.DESTINATION, true);
	}

	//asettaa kytetyn lukuarvon mppykselle true-arvon (esim. ominaisuus agent on olemassa)
	public void setValue(int frameEntity) {
		this.roleExist.put(frameEntity, true);
	}
/**
 * asettaa kytetyn lukuarvon mppykselle 
 * true-arvon (esim. ominaisuus agent on olemassa)
 * SEK asettaa rajoitteen mukaisen luokan ominaisuuden 
 * rangeksi kyseiselle verbille
 *
 */	
	public void setRestriction(int frameEntity, String uri) {
		this.roleExist.put(frameEntity, true);
		this.roleToUri.put(frameEntity, uri);
	}
	
//rangen asettamiset
	public void setAgentUri(String uri) {
		this.setAgent();
		this.roleToUri.put(Configuration.AGENT, uri);
	}
	public void setHumanAgentUri(String uri) {
		this.setHumanAgent();
		this.roleToUri.put(Configuration.HUMANAGENT, uri);
	}
	public void setAnimalAgentUri(String uri) {
		this.setAnimalAgent();
		this.roleToUri.put(Configuration.ANIMALAGENT, uri);
	}
	public void setEffectorUri(String uri) {
		this.setEffector();
		this.roleToUri.put(Configuration.EFFECTOR, uri);
	}
	public void setEssenceUri(String uri) {
		this.setEssence();
		this.roleToUri.put(Configuration.ESSENCE, uri);
	}
	
	public void setPatientUri(String uri) {
		this.setPatient();
		this.roleToUri.put(Configuration.PATIENT, uri);
	}
	public void setPlaceUri(String uri) {
		this.setPlace();
		this.roleToUri.put(Configuration.PLACE, uri);
	}
	public void setTimeUri(String uri) {
		this.setTime();
		this.roleToUri.put(Configuration.TIME, uri);
	}
	public void setMaterialUri(String uri) {
		this.setMaterial();
		this.roleToUri.put(Configuration.MATERIAL, uri);
	}
	public void setInstrumentUri(String uri) {
		this.setInstrument();
		this.roleToUri.put(Configuration.INSTRUMENT, uri);
	}
	public void setRelativePlaceUri(String uri) {
		this.setRelativePlace();
		this.roleToUri.put(Configuration.RELATIVEPLACE, uri);
	}
	public void setAbsolutePlaceUri(String uri) {
		this.setAbsolutePlace();
		this.roleToUri.put(Configuration.ABSOLUTEPLACE, uri);
	}
	public void setAbsoluteTimeUri(String uri) {
		this.setAbsoluteTime();
		this.roleToUri.put(Configuration.ABSOLUTE_TIME, uri);
	}
	public void setRelativeTimeUri(String uri) {
		this.setRelativeTime();
		this.roleToUri.put(Configuration.RELATIVE_TIME, uri);
	}
	public void setResourceUri(String uri) {
		this.setResource();
		this.roleToUri.put(Configuration.RESOURCE, uri);
	}
	public void setGoalUri(String uri) {
		this.setGoal();
		this.roleToUri.put(Configuration.GOAL, uri);
	}
	public void setDestinationUri(String uri) {
		this.setDestination();
		this.roleToUri.put(Configuration.DESTINATION, uri);
	}	
	

/** 
 * ominaisuuden eksistenssin tiedustelut
 * Jos ko. verbille on mritelty kyseinen
 * ominaisuus, palautetaan true.
 * @param index
 * @return true, jos ominaisuus on mritelty
 */
	public boolean hasValue(int index) {
		Boolean b = roleExist.get(index);
		if(b == null) {
			return false;
		}
		return b;
	}
	public boolean hasEffector() {
		return this.hasValue(Configuration.EFFECTOR);
	}
	public boolean hasAgent() {
		return this.hasValue(Configuration.AGENT);
	}
	public boolean hasHumanAgent() {
		return this.hasValue(Configuration.HUMANAGENT);
	}
	public boolean hasAnimalAgent() {
		return this.hasValue(Configuration.ANIMALAGENT);
	}
	public boolean hasResource() {
		return this.hasValue(Configuration.RESOURCE);
	}
	public boolean hasPatient() {
		return this.hasValue(Configuration.PATIENT);
	}
	public boolean hasPlace() {
		return this.hasValue(Configuration.PLACE);
	}
	public boolean hasTime() {
		return this.hasValue(Configuration.TIME);
	}
	public boolean hasMaterial() {
		return this.hasValue(Configuration.MATERIAL);
	}
	public boolean hasInstrument() {
		return this.hasValue(Configuration.INSTRUMENT);
	}
	public boolean hasRelativePlace() {
		return this.hasValue(Configuration.RELATIVEPLACE);
	}
	public boolean hasAbsolutePlace() {
		return this.hasValue(Configuration.ABSOLUTEPLACE);
	}
	public boolean hasAbsoluteTime() {
		return this.hasValue(Configuration.ABSOLUTE_TIME);
	}
	public boolean hasRelativeTime() {
		return this.hasValue(Configuration.RELATIVE_TIME);
	}
	public boolean hasGoal() {
		return this.hasValue(Configuration.GOAL);
	}
	public boolean hasDestination() {
		return this.hasValue(Configuration.DESTINATION);
	}
	
//	ominaisuuden rangen tiedustelut
	public String getValue(int index) {
		String s = roleToUri.get(index);
		if(s == null) {
			return null;
		}
		return s;
	}
	public String getEffector() {
		return this.getValue(Configuration.EFFECTOR);
	}
	public String getAgent() {
		return this.getValue(Configuration.AGENT);
	}
	public String getHumanAgent() {
		return this.getValue(Configuration.HUMANAGENT);
	}
	public String getAnimalAgent() {
		return this.getValue(Configuration.ANIMALAGENT);
	}
	public String getResource() {
		return this.getValue(Configuration.RESOURCE);
	}
	public String getPatient() {
		return this.getValue(Configuration.PATIENT);
	}
	public String getPlace() {
		return this.getValue(Configuration.PLACE);
	}
	public String getTime() {
		return this.getValue(Configuration.TIME);
	}
	public String getMaterial() {
		return this.getValue(Configuration.MATERIAL);
	}
	public String getInstrument() {
		return this.getValue(Configuration.INSTRUMENT);
	}
	public String getRelativePlace() {
		return this.getValue(Configuration.RELATIVEPLACE);
	}
	public String getAbsolutePlace() {
		return this.getValue(Configuration.ABSOLUTEPLACE);
	}
	public String getAbsoluteTime() {
		return this.getValue(Configuration.ABSOLUTE_TIME);
	}
	public String getRelativeTime() {
		return this.getValue(Configuration.RELATIVE_TIME);
	}
	public String getGoal() {
		return this.getValue(Configuration.GOAL);
	}
	public String getDestination() {
		return this.getValue(Configuration.DESTINATION);
	}
	
	
	public String getUri() {
		return this.uri;
	}
	
	public HashMap <Integer, Boolean> getVerbs() {
		return this.roleExist;
	}
	
}

