/**
 * Erist ontologiasta -minen -ptteiset toimintaa kuvaavat sanat ja
 * lemmaa ne.
 *
 */
package fi.seco.semweb.service.poka.frame;

import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import org.xml.sax.InputSource;

import fi.seco.semweb.service.poka.util.LemmaLister;
import fi.seco.semweb.service.poka.util.XFDGConnect;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.util.iterator.ExtendedIterator; 

public class Verborator {
	private Model verbs;
	private XFDGConnect fdg;
	private Property pLabel;
	
	private int verbLabelCounter;
	
	private StringBuffer allLemmas; 
	
	private final String YSOSUBJIDSPACE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    						 YSOSUBJIDLOCAL = "type",
						 YSOBASE = "http://yso.fi/YSO#",
						 PLABEL = "prefLabel",
						 ALABEL = "altLabel";
	
	private final String VERBUS = "metsn";
	private final String VERBPATTERN = ".*minen";
	
	private final String ABBR = "oma";
	public static final String VERBNAMESPACE = "http://www.seco.hut.fi/ns/2006/02/alm#";
	public static final String VERBLABEL = "verblabel";
	
	private final static String DEFAULTOUTPUTFILE = "/home/alm/termo/Verbs-all.owl";
	private final static String DEFAULTINPUTFILE = "/home/alm/ontocvs/ontologies/YSO/YSO-All.owl";
	private final static String GROUPS = "/home/alm/ontocvs/ontologies/YSO/YSA-Groups.owl";
	private final static String TOIMINTA = "/home/alm/termo/Verbs-all3.owl";
	
	private java.util.Iterator <String> ii;
	private Resource current;
	
	public Verborator() {
		verbs = ModelFactory.createDefaultModel();
		verbs.setNsPrefix(ABBR,VERBNAMESPACE);
		pLabel = verbs.createProperty(VERBNAMESPACE, VERBLABEL);
	}
	
	public boolean addOntology(String filename) {

		verbs.setNsPrefix("yso","http://yso.fi/YSO#");

		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

		OntModel m = ModelFactory.createOntologyModel();
	
		try {
			m.read(new InputStreamReader(in, "UTF-8"), "");
		}catch(Exception e) {
                System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
                e.printStackTrace();
        }

		Property subjProp = m.getProperty(YSOSUBJIDSPACE, YSOSUBJIDLOCAL);

		Property pl = m.getProperty(YSOBASE,PLABEL);
		Property al = m.getProperty(YSOBASE, ALABEL);
	
		ResIterator concepts = m.listSubjectsWithProperty(subjProp);

		Resource temp;
		Statement currentStmt = null;
		StmtIterator stmtI;

		fdg = new XFDGConnect();

		String before;

		int verbilaskuri = 0;
		this.verbLabelCounter = 0;
		
		allLemmas = new StringBuffer(100000);
		
/**
 * HUOM 3.2.2006: Hurjaa: Connexorin fdg osaa muuttaa substantiiviset
 * muodot verbeiksi, kunhan eteen laitetaan jokin sana esimerkiksi
 * genetiiviss, esimerkki:
 * lemma 'mets juokseminen' --> mets, juokseminen
 * lemma 'metsn juokseminen' --> mets, juosta
 * ja lis:
 * lemma 'sosiaalinen kyttytyminen' --> sosiaalinen, kyttytyminen
 * lemma 'metsn kyttytyminen' --> mets, kyttyty
 *
 */		
				
		while(concepts.hasNext()) {

			temp = concepts.nextResource();
			stmtI = temp.listProperties(pl); //prefLabel
			while(stmtI.hasNext()) {	
				try {
					currentStmt = stmtI.nextStatement();
				} catch (NoSuchElementException e) {
					System.out.println("No such element: " + e);
				}					
				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi
					before = this.getLiteral(currentStmt);
					
					if(this.addIfVerb(before)) {
						verbilaskuri++;
					}
															
				} else {
					before = this.getLiteral(currentStmt);
					
					if(this.addIfVerb(before)) {
						verbilaskuri++;
					}		
				}		
			}							

			stmtI = temp.listProperties(al); //altLabel

			while(stmtI.hasNext()) {
				
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi

					before = this.getLiteral(currentStmt);
					if(this.addIfVerb(before)) {
						verbilaskuri++;
					}
				} else {
					before = this.getLiteral(currentStmt);

					if(this.addIfVerb(before)) {
						verbilaskuri++;
					}
				}
									
			}					
	
		}
		
		System.out.println("------------------------------");
		System.out.println("Verbej yhteens (-minen -ptteiset sanat): "+ verbilaskuri);
		System.out.println("Verbien labeleita: " + verbilaskuri+this.verbLabelCounter);
		System.out.println("------------------------------");
		
		
		fdg.setInput(allLemmas.toString());
		System.out.print("Lemmataan (koko: " +allLemmas.toString().length()+")...");
		InputSource is = fdg.getFDG();
		System.out.println("...ok!");
		LemmaLister ll = new LemmaLister();
		System.out.print("Parsitaan...");
		ll.parse(is);
		System.out.println("...ok!");

		ArrayList <String> tempL = ll.getResult();
	
		System.out.println("--> listan koko: " + tempL.size()+" <--");
/*
 * (14.2.2006)
 * Huom, tallennetut muodot ok:
 * jos Verboratorin palauttamat lemmat eivt
 * ole verbiperusmuodossa, eivt ne ole
 * sit myskn dokumentin tekstin lemmatisoinnissa:
 * piha#rakentaminen on piha#rakentaminen, ei piha#rakentaa
 */
		ii = tempL.iterator();
		
		concepts = m.listSubjectsWithProperty(subjProp);
		
		String currentVerb;

		while(concepts.hasNext()) {
			
			temp = concepts.nextResource();
			current = verbs.createResource(temp.getURI());
			stmtI = temp.listProperties(pl); //prefLabel
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();				
				
				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi				
					currentVerb = this.getLiteral(currentStmt);
					this.addVerb(hasLabel(currentVerb), currentVerb);									
				} else {
					currentVerb = this.getLiteral(currentStmt);
					this.addVerb(hasLabel(currentVerb), currentVerb);
				}
			}
			
			stmtI = temp.listProperties(al); //altLabel
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {								
				} else if(this.getLang(currentStmt).equals("fi")) {
					currentVerb = this.getLiteral(currentStmt);
					this.addVerb(hasLabel(currentVerb), currentVerb);
								
				} else {
					currentVerb = this.getLiteral(currentStmt);
					this.addVerb(hasLabel(currentVerb), currentVerb);
				}								
			}
					
		}
							
		return true;
	}

/**
 * Imee annettuun tiedostoon luokan #toiminta alaluokat ja ko. luokkien labelit 
 * @param filename
 * @return
 */
	
	
	public boolean extractToiminta(String filename) {

		verbs.setNsPrefix("yso","http://yso.fi/YSO#");
	
		InputStream in = null;
		in = FileManager.get().open( filename );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}

	    OntDocumentManager dm = OntDocumentManager.getInstance();
	    dm.setProcessImports(false);
	    
		OntModel m = ModelFactory.createOntologyModel();	   

		try {
			m.read(new InputStreamReader(in, "UTF-8"), "");
		}catch(Exception e) {
                System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
                e.printStackTrace();
    		}
//----------------------------
		in = null;
		in = FileManager.get().open( GROUPS );
	    if (in == null) {
			System.out.println("File '" + filename + "' does not exist!");
			return false;
		}
		OntModel add = ModelFactory.createOntologyModel();
		try {
			add.read(new InputStreamReader(in, "UTF-8"), "");
		} catch(Exception e) {
                System.out.println("ERROR WHEN READING ONTOLOGY MODEL...");
                e.printStackTrace();
        }
		
		m.add(add);
		
		
		Property pl = m.getProperty(YSOBASE,PLABEL);
		Property al = m.getProperty(YSOBASE, ALABEL);
	
		OntClass toiminta = m.getOntClass("http://yso.fi/YSO#toiminta");
		
		ExtendedIterator concepts = toiminta.listSubClasses();


		Statement currentStmt = null;
		StmtIterator stmtI;		
		String currentVerb;

//		Resource temp;
		OntClass temp;
		while(concepts.hasNext()) {
			
			temp = (OntClass)(concepts.next());			
			current = verbs.createResource(temp.getURI());
			
			stmtI = temp.listProperties(pl); //prefLabel
//			current = verbs.createResource(temp.getURI());
//			stmtI = temp.listProperties(pl); //prefLabel
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();				
				
				if(this.getLang(currentStmt).equals("se")) {
				} else if(this.getLang(currentStmt).equals("fi")) { //ohitetaan ruotsi				
					currentVerb = this.getLiteral(currentStmt);
					current.addProperty(pLabel, verbs.createLiteral(currentVerb));									
				} else {
					currentVerb = this.getLiteral(currentStmt);
					current.addProperty(pLabel, verbs.createLiteral(currentVerb));
				}
			}
			
			stmtI = temp.listProperties(al); //altLabel
			
			while(stmtI.hasNext()) {
				currentStmt = stmtI.nextStatement();

				if(this.getLang(currentStmt).equals("se")) {								
				} else if(this.getLang(currentStmt).equals("fi")) {
					currentVerb = this.getLiteral(currentStmt);
					current.addProperty(pLabel, verbs.createLiteral(currentVerb));
								
				} else {
					currentVerb = this.getLiteral(currentStmt);
					current.addProperty(pLabel, verbs.createLiteral(currentVerb));
				}								
			}
					
		}
							
		return true;
	}

	private String getLang(Statement stmt) {
		return stmt.getLanguage();
	}
	
	private String getLiteral(Statement stmt) {
		return stmt.getLiteral().getString();
	}
	
	private boolean addIfVerb(String verbLabel) {
		String lastVerb = "_";
		StringTokenizer tokenizer;
		if(verbLabel.matches(VERBPATTERN)) {
			tokenizer = new StringTokenizer(verbLabel);
			if(tokenizer.countTokens()==1) { // jos 1 token
				allLemmas.append(VERBUS + " " + verbLabel.replaceAll("_"," ")+"_ ");
			} else { // jos enemmn, ota viimeinen sana (verbi)
				while(tokenizer.hasMoreTokens()) {
					lastVerb = tokenizer.nextToken();
				}
				allLemmas.append(VERBUS + " " + lastVerb.replaceAll("_"," ")+"_ ");
				this.verbLabelCounter++;
			}
			return true;
		}
		return false;
	}
	
	private boolean hasLabel(String verbLabel) {
		if(verbLabel.matches(VERBPATTERN)) {
			return true;
			
		} else {
			return false;
		}
	}
	
	private void addVerb(boolean isVerb, String currentVerb) {
		if(isVerb) {
			String verbformTemp;
			verbformTemp = ii.next();
			verbformTemp = verbformTemp.substring(6,verbformTemp.length());
			System.out.println("verbi / fi: " +verbformTemp +" ("+currentVerb+")");
			current.addProperty(pLabel, verbs.createLiteral(verbformTemp));
		}
	}
	
	
	public boolean writeToFile(String filename) throws Exception {
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		verbs.write(pw);
		return true;
	}

	public static void main(String [] args) {
		
		Verborator my = new Verborator();

		String inputFileName;
		InputStream in = null;
		try {
			inputFileName = DEFAULTINPUTFILE;
			in = FileManager.get().open( inputFileName );
	        if (in == null) {
	    		throw new IllegalArgumentException(
                   "File: '" + inputFileName + "' not found");
			}

		} catch(ArrayIndexOutOfBoundsException e) {
			System.out.println("Usage: Verborator <filename>");
			System.exit(0);
		}

/*
		System.out.println("Listn ontologia (ilman oletuskielt): " +
				my.addOntology(DEFAULTINPUTFILE));
		try {
		 my.writeToFile(DEFAULTOUTPUTFILE);
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
*/

		System.out.println("Listn toiminnan alaluokat (ilman oletuskielt): " +
				my.extractToiminta(DEFAULTINPUTFILE));
		try {
		 my.writeToFile(TOIMINTA);
		} catch(Exception e) {
			System.out.println("Virhe tiedoston kirjoittaessa...");
			System.exit(0);
		}
		
		
	}
		
}
