/**
 * Nimien eristin.
 */

package fi.seco.semweb.service.poka.name;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.util.Lemmatizer;


import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import java.io.FileNotFoundException;
import java.util.HashSet;
import java.util.Vector;


public class NameExtractor {

private static final Log log = LogFactory.getLog(NameExtractor.class);

/**
 * Mritt ensimmist seuraavien sanojen
 * vrt morphologiset koodit regexin
 */
private final String WMOR = ".*INE|.*ELA";
	
/**
 * <pre>
 * MID defines regex for middle, lower-case
 * parts of a people name.
 * The rule is:
 * in the middle of the name, there can be: 
 * von, af, de, van, van der, von der
 *
 * TO DO: laajenna sanastoa.
 * Voiko nimess olla keskell sulkulauseke? Saattaisi olla jrkev.
 * </pre>
 */
//name can have these in the middle
//22.9.05 --> LISTTY 'van' ja 'van der'
private final String MID =     "(von|af|de|van|van der|von der)";

private final String FNAME = "\\p{javaUpperCase}(\\.|\\p{javaLowerCase}+)" +         			
"(-\\p{javaUpperCase}\\p{javaLowerCase}+)?";

//SNAME voi sislt ylpilkun 'n
private final String SNAME = "\\p{javaUpperCase}(\\.|\\p{javaLowerCase}+)" +         			
	"(-?\\p{javaUpperCase}\\p{javaLowerCase}+)?('n)?";

/**
 * <pre>
 * WEND, regex-pattern representing wrong end of a name.
 * If potential name has WEND afterwards, name is
 * discarded.
 *
 * The rule is:
 * -in-casesensitive word, beginning with
 * -"-" OR 
 * -words "oy", "ry" or beginning with "oy", "ry" OR
 * -words for, of, and
 * </pre>
 */
private final String WENDSYMBOL = "-.+";
private final String WENDTEXT =  "(?i)(of|and|ry.*|for|oy.*)";
private final String NAMENODE = "NA";
private boolean nameFound;
private int upIterator;
private int boost;

private HashSet <String> notAName;
private HashSet <String> isAName;

private Vector <String> lemmatizedName;

private Vector <Node> name; //listtvt nimet solmuina vektorissa
private Document document;
private Lemmatizer lemmatizer;

private NameConfig conf;
public NameIndex index; //29.3.06: vaihdettu public-muotoon, ei jaksa kaivaa 

	public NameExtractor() {

//		System.out.print("Luetaan nimilistat...");
		try {
			conf = new NameConfig();
			notAName = conf.getNoNamelist();
			isAName = conf.getNamelist();
		} catch (FileNotFoundException f) {
			notAName = new HashSet<String>();
			isAName = new HashSet<String>();
		}
//		System.out.println("ok!");
		
		index = new NameIndex();
		lemmatizer = new Lemmatizer();
	}

	public void setDocument(Document doc) {
		this.document = doc;
	}

	public Document addNames() {

		this.parseNames();

//		System.out.print("Tallennetaan nimilistat...");
		this.conf.setNoNamelist(notAName);
//		this.conf.setNamelist(isAName);
///		System.out.print("...ok!");
		log.info(index.toString());
		return this.document;
	}


	private void parseNames() {
        String expression = "/A/S/UP"; // ANALYSIS/SENTENCE/UPPER
        NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		Node next;

		NameInfo itemInfo;
		NameInfo nextInfo;
		
		String nodeName;
		
        upIterator = 0;
		boolean 	firstRound,
				firstIsGen,
				endMorpho = false,
				commaName = false;
		
		try {

			//ota docin UP-solmut NodeListiin
			nodes = (NodeList) xpath.evaluate(expression, document, XPathConstants.NODESET);

        	for(upIterator = 0; upIterator < nodes.getLength(); upIterator++) {

        		this.clearName();
        		        		
        		firstRound = true;
        		nameFound = false;
        		firstIsGen = false;
        		endMorpho = false;
        		commaName = false;
        		
        		Node item = nodes.item(upIterator); // ota solmu (aina UP!)
    		
			itemInfo = new NameInfo(item, true); //true = upper      		
			log.debug("item on  : "+ itemInfo.getText()+"(lemma: '"+itemInfo.getLemma()+"', morpho: '" + itemInfo.getMorpho()+"')");
			
      		this.toName(item); // mahdollinen nimi
      		this.lemmatizedName.add(itemInfo.getLemma());
      		
			next = this.nextNode(item); //looppia varten

			if(next == null){
				log.debug("\t--> Seuraava on null!");
			}
			
			while(next != null) {		

				nodeName = next.getNodeName();
				if(nodeName.equals("UP")) {

					nextInfo = new NameInfo(next, true);
					
					log.debug("next / up: " + nextInfo.getText() + "(lemma: '"+nextInfo.getLemma()+"', morpho: '"+nextInfo.getMorpho()+"')");
					
					if(notAName.contains(itemInfo.getLemma())) {
						log.debug("\t"+itemInfo.getText() + " ei nimi, skipataan!");
						break;
					} 
									
					
					
					if(firstRound) { // 1. kierros
						log.debug("\t--> 1. kierros.");
						if(!this.isAName.contains(itemInfo.getLemma())) { 
							if(!this.isPre(itemInfo)) {	//tarkista item (=edeltv nimi)							
								log.debug("\t--> Ei 1. nimi! (isPre): hyltn '"+itemInfo.getText()+"'");
								break;							
							}
							 if(!this.isFname(itemInfo) || nextInfo.getMorpho().matches(".*INE")) {
								this.notAName.add(itemInfo.getLemma());
								log.debug("\t--> item hyltn (ELA/INE).");
								log.debug("\t--> Listtiin notNameen: " + itemInfo.getLemma());
								if(isAName.contains(itemInfo.getLemma())) {
									log.debug("\t-->nimiluettelo sis. jo ko. nimen, poistetaan!");
									isAName.remove(itemInfo.getLemma());
								}	 
								break;
							 }
							if(itemInfo.getMorpho().matches(".*GEN|.*ESS|.*ALL|.*PTV")) {
								log.debug("\t--> Asetetaan firstIsGen.");
								firstIsGen = true;
							}
						}
					}
					
					//nimen lopun testaus
					if(this.index.containsSurname(nextInfo.getLemma())) {
						this.lemmatizedName.add(nextInfo.getLemma());
						this.toName(next);
						nameFound = true;
						break;
					}
					
//					log.debug("Lemmatizedname on: "+lemmatizedName);
					
					if(this.isEndName(nextInfo)) {
						if(firstIsGen) {
							if(nextInfo.getMorpho().matches(".*GEN|.*INE|.*ILL|.*ALL")) {
								this.notAName.add(itemInfo.getLemma());
								log.debug("\t--> firstIsGen: hyltn!");
								log.debug("\t--> Listtiin notNameen: " + itemInfo.getLemma());
								if(isAName.contains(itemInfo.getLemma())) {
									log.debug("\t-->nimiluettelo sis. jo ko. nimen, poistetaan!");
									isAName.remove(itemInfo.getLemma());
								}
								break;
							}
						}
						if(endMorpho) {
							if(nextInfo.getMorpho().matches(".*GEN|.*ELA|.*INE|.*PTV|.*NOM")) {
								log.debug("\t--> ENDMOR: hyltn!");
								break;
							}
						}
						if(nextInfo.getMorpho().matches(".*GEN")) {
							log.debug("\t--> Asetettiin endMorpho.");
							endMorpho = true;
						}
						this.toName(next);
						this.lemmatizedName.add(nextInfo.getLemma());
						
						nameFound = true;
					} else {
						log.debug("\t--> Ei loppunimi!");
						break;
					}
				} else if(nodeName.equals("BS")) {

					nextInfo = new NameInfo(next, false); //true aiheuttaa kaatumisen...
					
					log.debug("next / bs: " + nextInfo.getText());
					if(!this.isFname(itemInfo)) {
						break;
					}
					
					if(notAName.contains(itemInfo.getLemma())) {
						log.debug("\t"+itemInfo.getText() + " ei nimi, skipataan!");
						break;
					}
									
					if(this.traverseMiddle(next)) {
						log.debug("\t--> Keskinimi lytyi!");
						nameFound = true;
					} else {
						if(nextInfo.getText().matches(WENDTEXT)) { //jos vr loppu, hylk sana
							log.debug("\t-->VR LOPPU, hyltn.");
							next = this.nextNode(next);
							while(next != null) {
								if(next.getNodeName().equals("UP")) {
									this.upIterator++;
									log.debug("ITEROIDAAN OHI:" + next.getFirstChild().getTextContent());								
								} else {
									break;
								}
								next = this.nextNode(next);
							}
							nameFound = false;
						}
					}				
					break;
				} else if(nodeName.equals("SY")) { //etsitn ',' tai '('

					nextInfo = new NameInfo(next, false);

					log.debug("next / sy: '" + nextInfo.getText()+"'");

					if(nextInfo.getText().equals("(")) {
						log.debug("\t--> Next on alkusulku");
						if(notAName.contains(itemInfo.getLemma())) {
							log.debug("\t"+itemInfo.getText() + " ei nimi, skipataan!");
							break;
						}

						next = this.traverseBrackets(next); // iteroi solmua eteen jos '('			
					} else if(nextInfo.getText().equals(",")) { //knteinen nimi itemill (sulkuja ei tarkisteta) 
						if(firstRound) {
							
							if(!itemInfo.getText().matches(SNAME)) { //jos item ei 
								break;							     //sukunimen muotoinen...
							}

							this.clearName(); //poistetaan jonosta item, nimi toisinpin...
						
							if(this.traverseReversed(next, itemInfo)) { //syt node Next + itemin teksti (ennen pilkkua oleva) 								
								this.toName(item);
								this.lemmatizedName.add(itemInfo.getLemma());
								commaName = true;
								nameFound = true;		
								break;
							}
							this.toName(item);
							break;
						
						} else { // jos 1. kierros && muuta kuin ','
							break;
						}				
					} else {
						if(nextInfo.getText().matches(WENDSYMBOL)) { //jos vr loppu, hylk sana
							nameFound = false;
						}
						
						break;
					}
				} else {

					log.debug("\t--> Seuraava jotain muuta...");

					break;
				}
				firstRound = false;
				
				next = this.nextNode(next);
			}
			
			if(nameFound) {
/* 2.1.: nimen lisys nimilistaan, otetaan pois kytst.				
				if(commaName) {
					this.isAName.add(this.nextNode(name.get(0).getFirstChild()).getTextContent().trim());
				} else {
					if(!itemInfo.getText().matches("\\p{javaUpperCase}\\.")) { //jos ei 'P.'
						this.isAName.add(itemInfo.getLemma());
					}
				}
*/				
				upIterator += this.appendNames()-1;
			} else { 		
				log.debug("-----> EI NIMI: <-----");
				for(Node n: name) {
					this.appendIndividual(n);
				}
				
			} //ID: 15
        	} //end for 

		} catch (XPathExpressionException e) {
	    		System.out.println(e);
	    	}

	}
	
/**
 * Palauttaa seuraavan sisaren, jos ei lydy, palautetaan null
 */
	private Node nextNode(Node n) {
		try { //tulee jostain syyst, kaiketi kun tavara loppuu...
			n = n.getNextSibling();
		} catch(NullPointerException e) {
			return null;
		}
				
		if(n != null) { //jos solmussa on tavaraa, 
			while(n.getNodeName().equals("#text")) { //poistetaan #text
				n = n.getNextSibling();
				if(n == null) {
					return null;
				}
			}
		}else {
			return null;
		}
		return n;
	}

	private boolean traverseMiddle(Node middle) {
/*
		if(middle == null) { null tarkistus
			return null;
		}
*/ 
		Node sur; //silyt n
		log.debug("Tarkistetaan keski: " + middle.getFirstChild().getTextContent());
		if(middle.getFirstChild().getTextContent().matches(MID)) { //keskiosa lytyy
			log.debug("--> lytyy!");
			sur = this.nextNode(middle);
			if(sur != null) { //jos seuraava solmu lytyy
				if(sur.getNodeName().equals("UP")) { // TODO: SNAME-tarkistus!
					//if(this.isEndName()) { 

						//muutettu 4.3.06: lemma on 2. lapsisolmuista!
						//this.lemmatizedName.add(middle.getLastChild().getTextContent());
						this.lemmatizedName.add(middle.getChildNodes().item(1).getTextContent());
						this.toName(middle);
						
						NameInfo surInfo = new NameInfo(sur, true);
						this.lemmatizedName.add(surInfo.getLemma());
						this.toName(sur);
						
						return true;
					//}
				}
			}
		}
		return false;
	}

/**
 * Etsii muotoa <sukunimi> , <etunimi> -muotoisia nimi
 * Jotta voidaan hydynt Fdg:n etumrett, nimi muutetaan
 * muotoon <etunimi> <sukunimi> ja heitetn FdgXQueryn kautta
 * fdg:lle.
 * 27.12, kikkailua: tarkistetaan todellakin Lemmatizerilla
 * <etunimi> <sukunimi>, listn vliin keskimmisi jos niit on
 * --> etumre pysyy kyll vastaavana.
 * Kytnnss: 
 * 1) Alkuperinen teksti: 'Paasikivi, Juho Kusti' 
 * 2) Lemmatizer saa tekstin 'Juho Paasikivi'
 * 3) Jos etumre tunnistetaan sanasta 'Juho', niin
 * 4) Listn nimi Juho Kusti Paasikivi
 * 
 * 
 * @param n solmu josta lhdetn liikkeelle
 * @return true, jos pilkkunimi lytyi
 */	
	
	private boolean traverseReversed(Node comma, NameInfo surname) {

		log.debug("surname on: " +surname.getText());
		String firstText = null;
		String ordName = "";
		NameInfo pni;
		Node temp = this.nextNode(comma);

		if(temp != null) {

			if(temp.getNodeName().equals("UP")) {
				pni = new NameInfo(temp, true);
				firstText = pni.getText();
				ordName += firstText+ " " + surname.getText(); //lis sukunimi

				if((lemmatizer.preExist(ordName)) && 
						(firstText.matches(FNAME))) {
					
					log.debug("NIMET: " +pni.getText()+" ("+pni.getMorpho()+"), " +surname.getText()+" ("+surname.getMorpho()+")");
/*
					if(pni.getMorpho().matches(".*GEN|.*ESS") &&
					   surname.getMorpho().matches(".*GEN|.*INE|.*ILL")) {
						return false;
					}
*/
					//TODO: onko ehto liian tiukka? tiukennettu 19.5.2006
					if(pni.getMorpho().matches(".*GEN|.*ESS") ||
							   surname.getMorpho().matches(".*GEN|.*INE|.*ILL")) {
								return false;
					}

					
					log.debug("\t--> pilkkunimi on tll!");

					this.lemmatizedName.add(pni.getLemma());
					this.toName(temp);
					
					temp = this.nextNode(temp);

					while(temp != null) {		
						if(temp.getNodeName().equals("UP")) {
							pni = new NameInfo(temp, true);
							this.lemmatizedName.add(pni.getLemma());
							this.toName(temp);
						} else {
							break;
						}						
						temp = this.nextNode(temp);
					}
					return true;
				}
			}
		}
		return false;		 
	}
	
	
/**
 * Kikkailua: nimi voi jatkua sulkujen yli.
 * Iteroidaan turhat solmut yli, palautetaan niit seuraava
 */
	private Node traverseBrackets(Node leftBracket) {
/* --> ei koskaan null
		if (n == null) {
			return n;
		}
*/
		log.debug("\tIteroidaan sulut yli");
		Node inside = leftBracket;
			do {	
				inside = this.nextNode(inside);
				if(inside == null) { //jos solmua ei ole 
					return null;
				}
				if(inside.getNodeName().equals("UP")) {
					upIterator++; //solmussa nime, iteroidaan looppia, ohitetaan!
				}
				log.debug("\tohitetaan teksti: " +inside.getFirstChild().getTextContent());
			} while(!inside.getFirstChild().getTextContent().equals(")"));
			
			return inside;		
	}
	
	private boolean isEndName(NameInfo n) {
		return (n.getText().matches(SNAME) &&
				!n.getMorpho().matches(".*INE|.*ELA"));
	}

	private void toName(Node n) {
		boost++;
		name.add(n);
	}
	
	private void clearName() {
		boost = 0;
		name = new Vector<Node>();
		this.lemmatizedName = new Vector <String>();
	}
	
	private boolean suspFirst(NameInfo n) { //onko epilyttv 1. sijamuoto
		return n.getMorpho().matches(".*GEN|.*ILL");
	}
	
	private boolean wrongSecond(NameInfo n) { // 
		return n.getMorpho().matches(".*GEN|.*ILL");
	}

	private boolean isPre(NameInfo n) {
		log.debug("\tPRE / A    :" +n.getSyntax().equals("&A>")+ " ("+ n.getSyntax()+")");
		log.debug("\tPRE / MATCH:" +n.getText().matches(FNAME)+ " ("+ n.getText()+")");
		return n.getSyntax().equals("&A>");
		
	}

	private boolean isFname(NameInfo n) {
		return (n.getText().matches(FNAME) && 
		   !(n.getMorpho().matches(".*ELA|.*INE")));
	}
	
	private boolean hasWrongMorpho(NameInfo n) {
		return n.getMorpho().matches(".*INE|.*PTV|.*ELA");		
	}

/**
 * Listn nimi puuhun, 
 * parametreina Node item, ksiteltv solmu
 *         sek listtv tekstisislt
 *
 * TODO: lis morphot ja syntaksit
 *
 */
	private Node appendName(Node item, String desc) {

		Element neuname;
		Element info;
		neuname = (Element) document.createElement(NAMENODE); //luo solmu NA

		neuname.setAttribute("no", item.getAttributes().getNamedItem("no").getTextContent());
		
//		log.debug("\t"+item.getFirstChild().getTextContent());

		neuname.appendChild(item.getFirstChild());	//liit alkuperinen T C:n lapseksi

		info = (Element) document.createElement("I");
 		info.setTextContent(desc); //lis kuvaus
		neuname.appendChild(info);

 		while(item.hasChildNodes()) {
// 			log.info("firstChild on: "+item.getFirstChild());
			neuname.appendChild(item.getFirstChild());
		}


		//poista item, korvaa concept-solmulla
		item.getParentNode().replaceChild(neuname, item);
		return neuname;
	}
/**
 * Yksittisen nimiesiintymn lisminen
 * @param n
 */
	
	private boolean appendIndividual(Node n) {
		
		NameInfo p = new NameInfo(n, true);
		int nameID = this.index.addIndividual(p.getLemma());	

		if(nameID == -1) {
			return false;
		} else {
			log.debug("\t--> Appendindividual, nameID: "+ nameID);
			this.appendName(n, "(ID: "+nameID+")");
			return true;
		}
	}

	private int appendNames(){
//		log.debug("Listn nimi:"+this.lemmatizedName);

		int temp = this.boost;
		this.boost = 0;
				
/* 
 * Kun tlt kutsutaan NameIndexi, tarkistetaan
 * samalla lytyyk keskimminen nimi, jos nin,
 * se heitetn yhten yksikkn NameIndexille:
 * --> mid+sur = sur
 */
		String middle = this.lemmatizedName.get(lemmatizedName.size()-2);
//		log.debug("middle: "+middle);
		
		String surname = "";
		if(middle.matches(MID) && lemmatizedName.size() > 2) { //van Beethoven
			surname = middle + " ";
//			log.debug("sur: "+surname);
//			log.debug("lem1: " +lemmatizedName);
			lemmatizedName.remove(middle);
//			log.debug("lem2: " +lemmatizedName);
		}
		surname += lemmatizedName.remove(lemmatizedName.size()-1);
//		log.debug("sur2: "+surname);
		int nameID = this.index.addName(lemmatizedName, surname); 
				
		for(Node n: name) {	
//			log.debug("node: "+n.toString());
			this.appendName(n, "(ID: "+nameID+")");
		}
	
		return temp;
	}
	
}