package fi.seco.semweb.service.poka.opas;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import fi.seco.semweb.service.poka.name.NameIndex.Name;

public class MatchWrapper implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7835253253081289903L;
	public enum MatchType { CONCEPT, NAME, PLACE, INSTANCE, VERB, REGEX }
	
	private Collection<ConceptMatch> concepts = new ArrayList<ConceptMatch>();
	private Collection<ConceptMatch> names = new ArrayList<ConceptMatch>();
	private Collection<ConceptMatch> places = new ArrayList<ConceptMatch>();
	private Collection<ConceptMatch> instances = new ArrayList<ConceptMatch>();
	private Collection<ConceptMatch> verbs = new ArrayList<ConceptMatch>();
	private Collection<ConceptMatch> regex = new ArrayList<ConceptMatch>();
	
	public Collection<ConceptMatch> getConcepts() {
		return concepts;
	}
	public void setConcepts(Collection<ConceptMatch> concepts) {
		if (concepts == null) {
			this.concepts.clear();
		} else {
			this.concepts = concepts;
		}		
	}
	public Collection<ConceptMatch> getInstances() {
		return instances;
	}
	public void setInstances(Collection<ConceptMatch> instances) {
		if (instances == null) {
			this.instances.clear();
		} else {
			this.instances = instances;
		}				
	}
	public Collection<ConceptMatch> getNames() {
		return names;
	}
	public void setNames(Collection<Name> names) {
		this.names.clear();
		if (names != null) {
			for (Name n : names) {
				ConceptMatch nm = new NameMatch();
				nm.addUri(""+n.getID(), n.toString(), null, n.getFrequency());//TODO: purkat joskus pois
				this.names.add(nm);
			}			
		}
	}
	public Collection<ConceptMatch> getPlaces() {
		return places;
	}
	public void setPlaces(Collection<ConceptMatch> places) {
		if (places == null) {
			this.places.clear();
		} else {
			this.places = places;
		}
	}
	public Collection<ConceptMatch> getRegex() {
		return regex;
	}
	public void setRegex(Collection<ConceptMatch> regex) {
		if (regex == null) {
			this.regex.clear();			
		} else {
			this.regex = regex;
		}
	}
	public Collection<ConceptMatch> getVerbs() {
		return verbs;
	}
	public void setVerbs(Collection<ConceptMatch> verbs) {
		if (verbs == null) {
			this.verbs.clear();
		} else {
			this.verbs = verbs;
		}
	}
	 
	public boolean hasConcepts() {
		return (this.concepts.size() > 0 || this.instances.size() > 0);
	}
	/**
	 * Returns true if this set of matches contains verb- name, regex, instance
	 * -etch.matches.
	 * @return
	 */
	public boolean hasSpecialMatches() {
		if (names.size() > 0 || places.size() > 0 || regex.size() > 0) { 				 
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Method to check if given match is included
	 * @param cm
	 * @return
	 */
	public boolean contains(ConceptMatch cm) {
		return (concepts.contains(cm) || places.contains(cm) ||
				instances.contains(cm) || verbs.contains(cm) ||
				regex.contains(cm));
	}
	public boolean containsName(String name) {		
		return names.contains(name);
	}
	public MatchType getMatchType(ConceptMatch match) {
		if (instances.contains(match)) {
			return MatchType.INSTANCE;
		} else if (names.contains(match)) {
			return MatchType.NAME;
		} else if (places.contains(match)) {
			return MatchType.PLACE;
		} else if (concepts.contains(match)) {
			return MatchType.CONCEPT;
		} else {
			return null;
		}
	}	
}
