package fi.seco.semweb.service.poka.util;
/**
 * Paikkatiedon kontekstipalauttaja
 */

import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdql.Query;
import com.hp.hpl.jena.rdql.QueryEngine;
import com.hp.hpl.jena.rdql.QueryExecution;
import com.hp.hpl.jena.rdql.QueryResults;
import com.hp.hpl.jena.rdql.ResultBinding;

public class PlaceQuery extends TermoQuery 
						 implements ContextInterface {

	private static final Log log = LogFactory.getLog(PlaceQuery.class);
	
	private final String PLACENS = "http://www.yso.fi/kulttuuriSampo/paikat#";
	private final String PLACELABEL = "nimi";
	
	private final String RDFNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	private final String RDFTYPE = "type";
	
	
	private Property label;
	private Property type;
	
	public PlaceQuery(String filename, String encoding) {
		super(filename, encoding);		
		log.debug("PlaceQuery alkaa");
		
		this.label = this.m.getProperty(PLACENS,PLACELABEL);
		this.type = this.m.getProperty(RDFNS,RDFTYPE);
	}
	
	public PlaceQuery(Model model) {
		super(model);
		
		this.label = this.m.getProperty(PLACENS,PLACELABEL);
	}

	/**
	 * palauttaa uria vastaavan yläluokan labelin
	 */
	public String getUpper(String uri) {

		String qString =
        "SELECT ?y " +
        "WHERE (<"+uri+">,<yso:kokonaisuus>,?x) " +
        "      (?x,<yso:nimi>,?y) " +
        "USING rdfs FOR <http://www.w3.org/2000/01/rdf-schema#> " +
        "       yso FOR <http://www.yso.fi/kulttuuriSampo/paikat#>";
		
		com.hp.hpl.jena.query.Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
		} catch(com.hp.hpl.jena.query.QueryException e) {
			log.error("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}

		com.hp.hpl.jena.query.QueryExecution qe = QueryExecutionFactory.create(q, m);

		String result = "";
		try {
			ResultSet results = qe.execSelect();
		
			for(; results.hasNext() ;) {
				QuerySolution res = results.nextSolution();
				result += ((Literal)res.get("y")).getString() +" | ";
			}
		} finally { qe.close(); }
			
		if(result.length() >=3) {
			result = result.substring(0,result.length()-3);
		}
		return result;
	}
	
	/**
	 * palauta YSOsta yläluokat listana
	 * 
	 */
	public ArrayList<String> getContextInformation(String uri) {
		String qString =
        "SELECT ?y " +
        "WHERE (<"+uri+">,<yso:kokonaisuus>,?x) " +
        "      (?x,<yso:nimi>,?y) " +
        "USING rdfs FOR <http://www.w3.org/2000/01/rdf-schema#> " +
        "       yso FOR <http://www.yso.fi/kulttuuriSampo/paikat#>";

		com.hp.hpl.jena.query.Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
		} catch(com.hp.hpl.jena.query.QueryException e) {
			log.error("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}
		
		com.hp.hpl.jena.query.QueryExecution qe = QueryExecutionFactory.create(q, m);

		ArrayList <String> result = new ArrayList<String>();
		
		try {
		
			ResultSet results = qe.execSelect();
				
			for( ; results.hasNext() ;) {
				QuerySolution res = results.nextSolution();
				if(!((Literal)res.get("y")).getLanguage().equals("se")) {
					result.add(((Literal)res.get("y")).getString());
				}
			}
		} finally { qe.close(); }
		
		return result;
	}
	
	public String getType(String uri) {
		log.debug("uri: "+uri);
//		luo uuden resurssin samalla, ei koskaan null
		Resource r = this.m.getResource(uri); 
		Statement rnode;
		if((rnode = r.getProperty(type)) != null) { //muutettu 12.10.06
		
			return rnode.getObject().asNode().getLocalName();		
		}
		log.warn("Urille (tai ominaisuudelle) ei löydy vastinetta ontologiasta");
		return "";
	}

	//TODO, väärän tavaran kyselyt, käsittely
	public String getLabel(String uri) {

		log.debug("uri on:   "+uri);
		log.debug("property: "+label);
		
		Resource r = this.m.getResource(uri);
		Statement rnode;
		if((rnode = r.getProperty(label)) != null) {

			return rnode.getLiteral().getLexicalForm();		
		}
		log.warn("Urille (tai ominaisuudelle) ei löydy vastinetta ontologiasta");
		return "";
	}
	
	
	}
