/**
 * FDG-yhteys
 */

package fi.seco.semweb.service.poka.util;
//package name;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.File;
import java.io.StringReader;

import java.net.*;
import org.xml.sax.InputSource;

/**
 * <pre>
 *	FDG-client for Qizx:
 *	1) takes textfile as an input (url/file)
 *	2) parses fdg
 *	3) returns sax InputSource 
 *  </pre>
 *  @author Olli Alm
 *  @version 1.0
 * 
 */
public class XFDGConnect {

/**
 * constant value for host name
 */
  private final String HOSTNAME = "nipo.seco.hut.fi";
/**
 * Port of Connexor's FDG-parser
 */
  private final int FDGPORT = 10102;

  private final int ANALYSISENDTAG = 13;
  private final int INITIALSTARTTAG = 123;
  
  private final int MAXFILESIZE = 100000;
  
  private final String ENCODING = "ISO-8859-1";
  //private final String ENCODING = "UTF-8";
  
  private String filename = null;
  private InputSource in;
  private Socket fdgSocket;
  
  private PrintWriter out;
  
  private String xmlOutput;

/**
 * Default constructor
 * 
 * @param fileUrl name of the file or url
 * @param isURI true, if given string is name of the file,
 *              false if the String is the actual text
 */
	public XFDGConnect(String fileUrl, boolean isURI) {
		if(isURI) {
			this.xmlOutput = this.readSource(fileUrl);
		} else {
			this.xmlOutput = fileUrl;
		}
	}
	
	public XFDGConnect() {}
	
	public void setInput(String text) {
		this.xmlOutput = text;
	}

/**
 * Method that reads textfile from the file or url.
 *
 * @param fileUrl name of the file or URL
 * @return lines of the text file as a String
 */
	public String readSource(String fileUrl) {
		this.filename = null;
		String temp = null;	
		URL myUrl;
		File myFile;
		try {
			filename = fileUrl;
			System.out.println("XFDGConnect /  luetaan dokumentti koodauksella "+ENCODING+".");
			System.out.print("XFDGConnect / luetaan tiedosto: '" +filename +"'...");
		} catch(Exception e) {
			System.out.println("Invalid or missing file name!");
			//System.exit(0);
		}

		StringBuffer textIn = new StringBuffer();
		
		try {
			BufferedReader in;
			myFile = new File(filename);
			if(myFile.exists()) {
				in = new BufferedReader(
						new InputStreamReader(
					  		new FileInputStream(myFile), ENCODING));
				textIn = new StringBuffer((int)myFile.length());
			} else {
				myUrl = new URL(filename);
				in = new BufferedReader(
						new InputStreamReader(
							myUrl.openStream(), ENCODING));
			}
//			int luku = 1;
			System.out.println();
			while((temp = in.readLine()) != null) {
				  textIn.append(temp + "\n");
				  System.out.print(".");
//				  System.out.println("Rivi "+luku);
//				  luku++;
			}
			System.out.println("\n...valmis!");
			in.close();
		} catch(ArrayIndexOutOfBoundsException e) {
				System.out.println("Array index out of bounds:: " + e);
		} catch(MalformedURLException e) {
				System.out.println("URL error: " + e);
				System.exit(0);
		} catch(Exception e) {
			System.out.println("Error: " + e);
			System.exit(0);
		}
		
		return textIn.toString();
	}
/**
 * This method returns Fdg-parser InputSource-object
 * of the text-document created by constructor
 * It connects to Fdg-server and writes 
 * lines from the server as a SAX-tree InputSource-object
 *
 * @return Sax InputSource-object of fdg-parsed document
 */
	public InputSource getFDG() {
		if(xmlOutput == null) {
			System.out.println("XFDGConnect / getFDG: syöttötekstiä ei asetettu!");
			//System.exit(0);
			return null;
		}
		
		String input = xmlOutput;
		int fileLen = input.length();
//		System.out.println("XFDGConnect / tiedoston koko: " +fileLen + " (max "+MAXFILESIZE+")");
		
		try {
//			System.out.print("XFDGConnect / vastaanotetaan fdg...");
  			if(fileLen > 140000) { // TODO: isojen tiedostojen lisäys
  				System.out.print("...(kyseessä pitkä tiedosto, kestää hetken)...");
  				
  				int begin = 0;
  				int end = MAXFILESIZE;
  				boolean left = true;
  				
  				String block, line;
  				StringBuffer text;
  				StringBuffer finalText = new StringBuffer(fileLen);
  				String startTags;
  				BufferedReader buf;
  				
  				do {
  					text = new StringBuffer(MAXFILESIZE);
  	  				if(end > fileLen) {
  	  					end = fileLen;
  	  					left = false;
  	  				}
  	  				
  					fdgSocket = new Socket(HOSTNAME, FDGPORT);
  		            out = new PrintWriter(new OutputStreamWriter(fdgSocket.getOutputStream(),"ISO-8859-1"), true);
  		            
  					block = input.substring(begin, end);
  					buf = new BufferedReader(new InputStreamReader(fdgSocket.getInputStream(), "ISO-8859-1"));
  					out.write(block);
  	  				out.flush();
  	  				fdgSocket.shutdownOutput();

  	  				while((line =buf.readLine())!= null) {
  	  					text.append(line+"\n");
  	  				}
  	  				startTags = text.substring(0,INITIALSTARTTAG);
  	  				finalText.append(text.substring(INITIALSTARTTAG, (text.length()-ANALYSISENDTAG)));
 	  				
  	  				begin = begin+MAXFILESIZE; //0+140000, 140000+140000...
  	  				end = begin+MAXFILESIZE;

  				} while(left);
  				finalText = finalText.insert(0, startTags); //lisätään alkutagi...
  				finalText = finalText.append("</analysis>");//...ja lopputagi

  				in = new InputSource(new StringReader(finalText.toString()));
    			} else { // else säilytetään koska se on nopeampi...
    				fdgSocket = new Socket(HOSTNAME, FDGPORT);
    	            out = new PrintWriter(new OutputStreamWriter(fdgSocket.getOutputStream(),"ISO-8859-1"), true);
    				in = new InputSource(new InputStreamReader(fdgSocket.getInputStream(), "ISO-8859-1"));
    				out.write(input);
    				out.flush();
    				fdgSocket.shutdownOutput();
    			}
		} catch(UnknownHostException he) {
            System.err.println("XFDGConnect / Unknown host: "+ HOSTNAME);
        } catch(IOException ioe) {
            System.err.println("XFDGConnect / IO Exception occurred: "+ ioe);
        } catch(Exception e) {
            System.err.println("XFDGConnect / Exception occurred: "+e);
        }

//-----------------------------------
//		System.out.println("...valmis!");
		return in;
	}
	
	
	public String sourceToString(InputSource is) throws IOException {
		BufferedReader bf = new BufferedReader(is.getCharacterStream());
		String temp  = null;
		StringBuffer result = new StringBuffer();
		while((temp = bf.readLine()) != null) {
			result.append(temp+"\n");
		}
		return result.toString();
	}

/**
 * This method check is the connection to Fdg-server
 * up.
 *
 * @return true if socket is connected, otherwise false
 */
    private  boolean checkConnection() {
        if(fdgSocket.isConnected())
            return true;
        else return false;
    }

/**
 * This method closes the connection to Fdg-server
 */
    public void closeConnection() {
        try {
            out.close();
            //in.close();
            fdgSocket.close();
        } catch(IOException e ) {
            System.err.println("Failed to close socket: "+ e);
        }

        out = null;
        in = null;
        fdgSocket = null;
    }
//----------------------------------
//Lisätty 21-06-2006, vanhaa QIZX:ää,  kivoja testaukseen
//----------------------------------
    

    
    
    
//---------------------------
    
    
    

  /** Main method to run from the command line.    */
  public static void main (String[] args) 
							throws Exception {

	try {
		XFDGConnect app = new XFDGConnect(args[0], true);	
		System.out.println(app.sourceToString(app.getFDG()));
	} catch (Exception e) {
      System.out.println("java XFDGConnect <URL>");
	  System.exit(0);
    }
    
  }	
  
}

