package fi.seco.semweb.service.poka.util.terms;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.FileManager;


/**
 * Tänne kasataan yleistavaraa termitiedostojen käsittelyyn
 * @author alm
 *
 */

public class TermHandler {
	private static final Log log = LogFactory.getLog(TermHandler.class);
	
	/**
	 *  
	 * Termimallia 1 verrataan 2:een ja päällekkäiset resurssit
	 * poistetaan jälkimmäisenä parametrina annetusta.
	 * Käytetään YSO-termistöjen päällekkäisyyksien poistamiseen, sillä
	 * osa termistöistä on tehty käsin ja Terms-all-tiedoston käsinsäätö olisi 
	 * raskasta...
	 * 
	 * @param m1 malli 1, säilyvä
 	 * @param m2 malli 2, josta poistetaan ne urit, jotka löytyvät myös m1:stä
 	 * 
 	 * 
	 */
	
	public static Model removeDuplicateResources(Model m1, Model m2) {
		StmtIterator si = m1.listStatements();
		while(si.hasNext()) {
			Statement s = (Statement)si.next();
			System.out.println(s.toString());
			if(m2.contains(s)) {
				System.out.println("\t--> sama löytyy!");
				m2.remove(s);
			}
		}
		return m2;
	}
	
	/**
	 * Tiedoston lukija, lukee ontologian sisään
	 * --> createSearchTrie käyttää tätä
	 */

	
	public static Model readOntology(String filename, String encoding)
							throws IOException {
			InputStream in = null;
			in = FileManager.get().openNoMap( filename );
	        if (in == null) {
	    		throw new IllegalArgumentException(
	            	"File '" + filename + "' not found");
			}
			Model terms = ModelFactory.createDefaultModel();
			terms.read(new InputStreamReader(in, encoding), "");
	        
	        return terms;
		}
	/**
	 * Tiedoston kirjoittaja, pukkaa ulos UTF-8-koodausta
	 */	
		public static void writeFile(String filename, Model m) 
						throws IOException {
			
			if(filename == null) { //jos null, käytä oletusta
				return;
			}
			File f = new File(filename);
			FileOutputStream fos = new FileOutputStream(f);
			m.write(fos);
			fos.flush();
			fos.close();
		}
/**
 * Geneerinen tiedostonlukija, lukee tiedoston ja palauttaa Stringin
 * @param filename
 * @param encoding
 * @return
 */
		public static String readFile(String filename, String encoding) 
					throws ArrayIndexOutOfBoundsException, 
					       MalformedURLException,
					       Exception {
			
			URL myUrl;
			StringBuffer textIn = new StringBuffer();

//			try {
				BufferedReader in;
				File myFile = new File(filename);
				if(myFile.exists()) {
					in = new BufferedReader(
							new InputStreamReader(
						  		new FileInputStream(myFile), encoding));
					textIn = new StringBuffer((int)myFile.length());
				} else {

					myUrl = new URL(filename);
					
					URLConnection urlconnection = myUrl.openConnection();
					
					//väärennetään user agent, jotta päästään myös wikiin
					urlconnection.setRequestProperty( "User-Agent", "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; H010818)" );
									
					in = new BufferedReader(
							new InputStreamReader(
								urlconnection.getInputStream(), encoding));
					
				}
//				int luku = 1;
				String temp;
				while((temp = in.readLine()) != null) {
					  textIn.append(temp + "\n");
					  System.out.print(".");
//					  System.out.println("Rivi "+luku);
//					  luku++;
				}
				System.out.println("\n...valmis!");
				in.close();
/*
			} catch(ArrayIndexOutOfBoundsException e) {
					log.error("Array index out of bounds:: " + e);
			} catch(MalformedURLException e) {
					log.error("URL error: " + e);
					//System.exit(0);
			} catch(Exception e) {
				System.out.println("Error: " + e);
				//System.exit(0);
			}
*/			
			return textIn.toString();
			
		}
		
		public static void main(String [] args) throws IOException {
			System.out.print("Luetaan malli 1..");
			Model m1 = TermHandler.readOntology("/home/alm/ontologies/ontologies/poka/frame/Terms-place-relativeFIXED.rdf","ISO-8859-1");
			System.out.println(".ok!");
			System.out.print("Luetaan malli 2..");
			Model m2 = TermHandler.readOntology("/home/alm/ontologies/ontologies/poka/frame/All-animals-bugs.rdf","UTF-8");
			System.out.println(".ok!");
			System.out.print("Poistetaan duplikaatit..");
			Model m3 = TermHandler.removeDuplicateResources(m1,m2);
			System.out.println(".ok!");
			System.out.print("Kirjoitetaan tiedostoon..");
			TermHandler.writeFile("/home/alm/ontologies/ontologies/poka/frame/All-animals-bugs-paikka.rdf", m3);
			System.out.println(".ok!");
			
		}
	
	
}
