package fi.seco.semweb.service.resourcelinking;

import java.util.HashMap;

import com.hp.hpl.jena.rdf.model.Resource;

import fi.seco.semweb.model.WeightedResourceObject;

/**
 * A wrapper for a resource linker that caches its results.
 * @author oisuomin
 *
 */

public class CachingResourceLinker implements IResourceLinker {

	private HashMap<Resource, Iterable<WeightedResourceObject>> cache;
	private IResourceLinker linker;

	/**
	 * Create a new CachingResourceLinker that caches the results of another resource linker.
	 * @param linker a resource linker to wrap
	 */
	
	public CachingResourceLinker(IResourceLinker linker) {
		cache = new HashMap<Resource,Iterable<WeightedResourceObject>>();
		this.linker = linker;
	}
	
	public Iterable<WeightedResourceObject> getLinkedResources(Resource resource) {
		if (cache.containsKey(resource))
			return cache.get(resource);

		Iterable<WeightedResourceObject> result = linker.getLinkedResources(resource);

		cache.put(resource, result);
		return result;
	}

}
