/*
 * Created on Jul 19, 2004
 *
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package fi.seco.semweb.service.resourcelinking;

import java.util.LinkedList;
import java.util.ListIterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

import fi.seco.semweb.infrastructure.SecoRegistry;
import fi.seco.semweb.infrastructure.jena.JenaService;
import fi.seco.semweb.model.WeightedResourceObject;
import fi.seco.semweb.util.JenaHelper;
import fi.seco.semweb.util.iterator.ALazyGeneratingIterator;
import fi.seco.semweb.util.iterator.AStatementMappingIterator;
import fi.seco.semweb.util.iterator.IIterableIterator;

public class RDFPathResourceLinker implements IResourceLinker {

	private Log log = LogFactory.getLog(RDFPathResourceLinker.class);
	protected final Pattern pattern  = Pattern.compile("\\s*([\\^\\.]?)(\\w*):(\\w+)(\\*?)|\\s*([\\^\\.]?)\\<(.+)\\>(\\*?)");
	private LinkedList<LinkStep> path = new LinkedList<LinkStep>();

	public RDFPathResourceLinker(String path) {
		Matcher matcher  = pattern.matcher(path);
		while (matcher.find()) {
			String property     = matcher.group(3);
			boolean inv         = false;
			if (property != null) {
				// name space and locaname given
				String ns      = matcher.group(2);

				// Inverse?
				String tmpstr  = matcher.group(1);
				if (tmpstr != null && tmpstr.equals("^")) inv = true;

				if (ns != null && !ns.equals("")) // name space defined?
				property = ns + ":" + property;
				else // no name space - use default
				property = "#" + property;
			} else {
				// uri given
				property = matcher.group(6);
				if (property != null) {
					// Inverse?
					String tmpstr  = matcher.group(5);
					if (tmpstr != null && tmpstr.equals("^")) inv = true;
				} else log.error("Malformed path expression: " + path);
			}
			this.path.add(new LinkStep(SecoRegistry.getService(JenaService.class).p(property), inv));
		}
	}

	public Iterable<WeightedResourceObject> getLinkedResources(final Resource resource) {
		return new ALazyGeneratingIterator<WeightedResourceObject>() {

			private LinkedList<IIterableIterator<Resource>> iterStack = new LinkedList<IIterableIterator<Resource>>();
			private ListIterator<LinkStep> stepIterator = path.listIterator();
			private IIterableIterator<Resource> curIterator=traverse(resource,stepIterator.next());

			@Override
			protected WeightedResourceObject generateNext() {
				while (stepIterator.hasNext() || !curIterator.hasNext())
					if (curIterator.hasNext()) {
						iterStack.addLast(curIterator);
						curIterator=traverse(curIterator.next(),stepIterator.next());
					} else {
						if (iterStack.isEmpty()) return null;
						curIterator=iterStack.removeLast();
						stepIterator.previous();
					}
				return new WeightedResourceObject(curIterator.next());
			}

		};
	}

	public IIterableIterator<Resource> traverse(Resource resource,LinkStep step) {
		if (!step.inv) return new AStatementMappingIterator<Resource>(resource.listProperties(step.property)) {
			@Override
			public Resource map(Statement o) {
				return o.getResource();
			}

		};
		return new AStatementMappingIterator<Resource>(JenaHelper.listInverseProperties(step.property,resource)) {
			@Override
			public Resource map(Statement o) {
				return o.getSubject();
			}

		};
	}

	public class LinkStep {
		/**
		 *  Description of the Field
		 */
		private boolean inv;
		/**
		 *  Description of the Field
		 */
		private Property property;

		/**
		 *  Constructor for the LinkImpl object
		 *
		 * @param  property    Description of the Parameter
		 * @param  inv         Description of the Parameter
		 * @param  transitive  Description of the Parameter
		 */
		public LinkStep(Property property, boolean inv) {
			this.property = property;
			this.inv = inv;
		}


		/**
		 *  Gets the property attribute of the LinkImpl object
		 *
		 * @return    The property value
		 */
		public Property getProperty() {
			return this.property;
		}


		/**
		 *  Gets the inv attribute of the LinkImpl object
		 *
		 * @return    The inv value
		 */
		public boolean isInv() {
			return inv;
		}


		/**
		 *  Description of the Method
		 *
		 * @return    Description of the Return Value
		 */
		@Override
		public String toString() {
			return (inv ? "^" + this.property : "." + this.property);
		}
	}

}
