package fi.seco.semweb.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFList;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDF;

import fi.seco.semweb.util.iterator.EmptyIterator;

/* use this as:

import static fi.seco.semweb.util.JenaModelHelper.p;
import static fi.seco.semweb.util.JenaModelHelper.r;

 */

public class JenaHelper {

	private static Log log = LogFactory.getLog(JenaHelper.class);

	public static Property p(String uri) {
		return ResourceFactory.createProperty(uri);
	}

	public static Resource r(String uri) {
		return ResourceFactory.createResource(uri);
	}

	public static StmtIterator listInverseProperties(Resource r) {
		return listInverseProperties(r, r.getModel());
	}

	public static StmtIterator listInverseProperties(RDFNode r, Model m) {
		return m.listStatements(null,null,r);
	}

	public static StmtIterator listInverseProperties(Property p, Resource r) {
		return listInverseProperties(p,r,r.getModel());
	}

	public static StmtIterator listInverseProperties(Property p, RDFNode r, Model m) {
		return m.listStatements(null,p,r);
	}

	public static Property p(Model model, String uri) {
		return model.createProperty(model.expandPrefix(uri));
	}

	public static OntProperty op(OntModel model, String uri) {
		return model.createOntProperty(model.expandPrefix(uri));
	}

	public static Resource r(Model model, String uri) {
		return model.createResource(model.expandPrefix(uri));
	}

	public static OntResource or(OntModel model, Resource r) {
		return model.getOntResource(r);
	}

	public static OntResource or(OntModel model, String uri) {
		return model.createOntResource(model.expandPrefix(uri));
	}

	public static Resource i(Model model, Resource classResource) {
		Resource r = model.createResource();
		r.addProperty(RDF.type,classResource);
		return r;
	}

	public static Resource i(Model model, String classUri) {
		return i(model,r(model,classUri));
	}

	public static Resource i(Model model, String uri, String classUri) {
		Resource r = r(model,uri);
		r.addProperty(RDF.type,r(model,classUri));
		return r;
	}

	@SuppressWarnings("unchecked")
	public static Iterator<RDFNode> getListItems(Resource resource, Property p) {
		try {
			return ((RDFList) resource.getProperty(p).getResource().as(RDFList.class)).iterator();
		} catch (Exception e) {
			log.error("JenaHelper.getListItems() called with non-list. This shouldn't happen, but returning an empty iterator anyway..");
			return EmptyIterator.SHARED_INSTANCE;
		}
	}

	@SuppressWarnings("unchecked")
	public static List<RDFList> getListsThatThisItemIsPartOf(RDFNode r, Model m) {
		List<RDFList> ret = new ArrayList<RDFList>();
		try {
			StmtIterator lists = listInverseProperties(RDF.first,r,m);
			while (lists.hasNext()) {
				Resource list = lists.nextStatement().getSubject();
				while (true) {
					StmtIterator sti=listInverseProperties(RDF.rest,list);
					if (sti.hasNext()) list=sti.nextStatement().getSubject();
					else break;
				}
				ret.add((RDFList) list.as(RDFList.class));
			}
		} catch (Exception e) {
			log.error("JenaHelper.getListsThatThisItemIsPartOf() called with non-list-item. This shouldn't happen, but returning an empty iterator anyway..");
		}
		return ret;
	}

	public static List<RDFList> getListsThatThisItemIsPartOf(Resource r) {
		return getListsThatThisItemIsPartOf(r, r.getModel());
	}

	public static Individual oi(OntModel model, Resource classResource) {
		return model.createIndividual(classResource);
	}

	public static Individual oi(OntModel model, String uri, Resource classResource) {
		return model.createIndividual(model.expandPrefix(uri),classResource);
	}

	public static Individual oi(OntModel model, String uri, String classUri) {
		return model.createIndividual(model.expandPrefix(uri),r(model,classUri));
	}

	public static OntClass oc(OntModel model, OntResource resource) {
		return resource.asClass();
	}

	public static OntClass oc(OntModel model, String uri) {
		return model.createClass(model.expandPrefix(uri));
	}

	public static String getLangProperty(Resource r, Property p, String lang) {
		StmtIterator sti = r.listProperties(p);
		while (sti.hasNext()) {
			Statement st = sti.nextStatement();
			if (lang.equalsIgnoreCase(st.getLanguage())) return st.getString();
		}
		return null;
	}

}
