package fi.seco.semweb.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDFS;

import fi.seco.semweb.model.BaseObject;

public class LString extends BaseObject {

	/**
	 * Literal label lang support
	 */
	private static final long serialVersionUID = 1L;
	private HashMap<String,String> strings = new HashMap<String,String>();

	public LString() {

	}

	public LString(String string) {
		strings.put(null,string);
	}

	public LString(String lang, String string) {
		strings.put(lang,string);
	}

	public LString(StmtIterator i) {
		addAll(i);
	}

	public LString(Resource r, Property p) {
		addAll(r,p);
	}

	public LString(Resource r, Iterable<Property> props) {
		for (Property p: props) addAll(r,p);
	}

	public LString(Resource r) {
		addAllLabels(r);
	}

	public String get(String lang) {
		return strings.get(lang);
	}

	public String getMatch(String lang) {
		if (get(lang)!=null) return get(lang);
		for (String slang : strings.keySet()) if (slang !=null && slang.startsWith(lang)) return get(slang);
		return null;
	}

	public String getMatchAny(String lang) {
		String ret = getMatch(lang);
		if (ret!=null) return ret;
		ret = get(null);
		if (ret!=null) return ret;
		ret = strings.values().iterator().next();
		return ret;
	}

	public Collection<String> getAll() {
		return strings.values();
	}

	public Map<String,String> getAllAsMap() {
		return strings;
	}

	public LString add(String string) {
		add(null,string);
		return this;
	}

	public LString add(Statement s) {
		strings.put(s.getLanguage(),s.getString());
		return this;
	}

	public LString addAll(StmtIterator i) {
		for (;i.hasNext();)
			add(i.nextStatement());
		return this;
	}

	public LString addAll(Resource r, Property p) {
		for (StmtIterator t = r.listProperties(p);t.hasNext();)
			add(t.nextStatement());
		return this;
	}

	public LString addAllLabels(Resource r) {
		addAll(r,RDFS.label);
		return this;
	}

	public LString add(String lang, String string) {
		strings.put(lang,string);
		return this;
	}

	public LString remove(String lang) {
		strings.remove(lang);
		return this;
	}

	@Override
	public String toString() {
		return strings.values().toString();
	}

	public void serializeToRDFAsValuesOfProperty(Resource r, Property p) {
		for (String lang : strings.keySet())
			if (lang == null) r.addProperty(p,get(null));
			else r.addProperty(p,r.getModel().createLiteral(get(lang),lang));
	}

	public void serializeToRDFAsLabelsOf(Resource r) {
		serializeToRDFAsValuesOfProperty(r,RDFS.label);
	}

	public int size() {
		return strings.size();
	}

	public LString addAll(LString other) {
		strings.putAll(other.getAllAsMap());
		return this;
	}

}
