package fi.seco.semweb.util.collection;

import java.util.Collection;
import java.util.Iterator;

import fi.seco.semweb.util.iterator.AFilteringIterator;

public abstract class AFilteringCollection<E> extends EncapsulatingCollection<E> {

	private boolean compacted=false;

	public AFilteringCollection(Collection<E> col) {
		super(col);
	}

	@Override
	public Iterator<E> iterator() {
		return new AFilteringIterator<E>(col) { @Override public boolean accept(E obj) { return AFilteringCollection.this.accept(obj); } };
	}

	@SuppressWarnings("unchecked")
	public void compact() {
		if (compacted==false) try {
			Collection<E> col2 = col.getClass().newInstance();
			for (E e : new AFilteringIterator<E>(col){ @Override public boolean accept(E obj) { return AFilteringCollection.this.accept(obj); } }) col2.add(e);
			col = col2;
			compacted=true;
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}

	@Override
	public int size() {
		compact();
		return super.size();
	}

	@Override
	public boolean isEmpty() {
		if (compacted) return col.isEmpty();
		return new AFilteringIterator<E>(col) { @Override public boolean accept(E obj) { return AFilteringCollection.this.accept(obj); } }.hasNext();
	}

	@Override
	public boolean contains(Object o) {
		compact();
		return super.contains(o);
	}

	@Override
	public Object[] toArray() {
		compact();
		return super.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		compact();
		return super.toArray(a);
	}

	@Override
	public boolean add(E o) {
		if (!compacted || accept(o)) return super.add(o);
		return false;
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		compact();
		return super.containsAll(c);
	}

	@Override
	public boolean addAll(Collection<? extends E> c) {
		if (!compacted) return super.addAll(c);
		boolean changed=false;
		for (E e : new AFilteringIterator<E>(c){ @Override public boolean accept(E obj) { return AFilteringCollection.this.accept(obj); } }) if (col.add(e)) changed=true;
		return changed;
	}

	@Override
	public void clear() {
		super.clear();
		compacted=true;
	}

	public abstract boolean accept(E object);

}
