package fi.seco.semweb.util.collection;

import java.util.Collection;
import java.util.Iterator;

import fi.seco.semweb.util.IFilter;
import fi.seco.semweb.util.iterator.FilteringIterator;

public class FilteringCollection<E> extends EncapsulatingCollection<E> {

	private static final long serialVersionUID = 1L;
	private IFilter<E> filter;
	private boolean compacted=false;

	public FilteringCollection(Collection<E> col,IFilter<E> filter) {
		super(col);
		this.filter=filter;
	}

	@Override
	public Iterator<E> iterator() {
		return new FilteringIterator<E>(col,filter);
	}

	@SuppressWarnings("unchecked")
	public void compact() {
		if (compacted==false) try {
			Collection<E> col2 = col.getClass().newInstance();
			for (E e : new FilteringIterator<E>(col,filter)) col2.add(e);
			col = col2;
			compacted=true;
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}

	@Override
	public int size() {
		compact();
		return super.size();
	}

	@Override
	public boolean isEmpty() {
		if (compacted) return col.isEmpty();
		return new FilteringIterator<E>(col,filter).hasNext();
	}

	@Override
	public boolean contains(Object o) {
		compact();
		return super.contains(o);
	}

	@Override
	public Object[] toArray() {
		compact();
		return super.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		compact();
		return super.toArray(a);
	}

	@Override
	public boolean add(E o) {
		if (!compacted || filter.accept(o)) return super.add(o);
		return false;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean containsAll(Collection c) {
		compact();
		return super.containsAll(c);
	}

	@Override
	public boolean addAll(Collection<? extends E> c) {
		if (!compacted) return super.addAll(c);
		boolean changed=false;
		for (E e : new FilteringIterator<E>(c,filter)) if (col.add(e)) changed=true;
		return changed;
	}

	@Override
	public void clear() {
		super.clear();
		compacted=true;
	}

}
