package fi.seco.semweb.util.iterator;

import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import fi.seco.semweb.util.collection.IteratorList;

public class SerializableIterator<E> implements ISerializableIterableIterator<E> {

	private static final long serialVersionUID = 1L;
	private transient Iterator<E> iter;
	
	public SerializableIterator(Iterator<E> iter) {
		this.iter=iter;
	}
	
	public Iterator<E> iterator() {
		return iter;
	}

	public boolean hasNext() {
		return iter.hasNext();
	}

	public E next() {
		return iter.next();
	}

	public void remove() {
		iter.remove();
	}

	private Object serializedForm;
	
	protected Object writeReplace() throws ObjectStreamException {
		if (iter instanceof Serializable) serializedForm=iter;
		else serializedForm=new IteratorList<E>(iter, new ArrayList<E>());
		return this;
	}
	
	@SuppressWarnings("unchecked")
	protected Object readResolve() throws ObjectStreamException {
		if (serializedForm instanceof Iterator) iter=(Iterator<E>) serializedForm;
		else iter=((Collection<E>)serializedForm).iterator();
		return this;
	}
	
}
