/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */
package fi.seco.semweb.util.iterator.mapiterator;

import java.util.NoSuchElementException;

import org.apache.commons.collections15.MapIterator;


public class SkipMaxMapIterator<T1,T2> implements MapIterator<T1,T2> {

	private int max;

	private MapIterator<T1,T2> iter;

	public SkipMaxMapIterator(MapIterator<T1,T2> iter, int skip, int max) {
		this.iter = iter;
		this.max = max;

		// roll over skip limit
		while (skip-- > 0) iter.next();
	}

	public void remove() {
		iter.remove();
	}

	public boolean hasNext() {
		return max != 0 && iter.hasNext();
	}

	public T1 next() {
		if (max-- != 0)
			return iter.next();
		throw new NoSuchElementException();
	}

	public T1 getKey() {
		return iter.getKey();
	}

	public T2 getValue() {
		return iter.getValue();
	}

	public T2 setValue(T2 arg0) {
		return iter.setValue(arg0);
	}

}
