package fi.seco.semweb.util.lucene;

import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;


public class LuceneDirectory implements ILuceneDirectory {

	private static Log log = LogFactory.getLog(LuceneDirectory.class);
	private Analyzer analyzer = new StandardAnalyzer();
	private Directory directory;
	private NonExceptingIndexWriter writer;
	private NonExceptingIndexReader reader;
	private NonExceptingIndexSearcher searcher;

	public LuceneDirectory() {
		System.err.println("LuceneDirectory: using parameterless constructor");
	}

	public LuceneDirectory(String path) {
		try {
			try {
				directory=FSDirectory.getDirectory(path,false);
			} catch (IOException e) {
				directory=FSDirectory.getDirectory(path, true);
			}
			IndexReader.unlock(directory);
			if (!directory.fileExists("segments")) getLuceneWriter();
		} catch (IOException e) {
			log.error("Error reading Lucene index",e);
		}
	}

	public NonExceptingIndexWriter getLuceneWriter() {
		try {
			if (writer==null) {
				if (searcher!=null) {
					searcher.close();
					searcher=null;
				}
				if (reader!=null) {
					reader.close();
					reader=null;
				}
				if (directory.fileExists("segments"))
					writer = new NonExceptingIndexWriter(new IndexWriter(directory,analyzer,false));
				else
					writer = new NonExceptingIndexWriter(new IndexWriter(directory,analyzer,true));
			}
		} catch (IOException e) {
			log.error("Error accessing lucene index",e);
		}
		return writer;
	}

	public NonExceptingIndexReader getLuceneReader() {
		if (reader==null) {
			if (writer!=null) {
				writer.close();
				writer=null;
			}
			reader = NonExceptingIndexReader.open(directory);
		}
		return reader;
	}

	public NonExceptingIndexSearcher getLuceneSearcher() {
		if (searcher==null) {
			getLuceneReader();
			searcher=new NonExceptingIndexSearcher(new IndexSearcher(reader.getReader()));
		}
		return searcher;
	}

	public long getLastModified() {
		try {
			return directory.fileModified("segments");
		} catch (IOException e) {
			log.error("Couldn't get last modified date for index",e);
			return -1;
		}
	}

	public void clearIndex() {
		try {
			if (writer!=null) {
				writer.close();
				writer=null;
			}
			if (searcher!=null) {
				searcher.close();
				searcher=null;
			}
			if (reader!=null) {
				reader.close();
				reader=null;
			}
			for (String s: directory.list()) directory.deleteFile(s);
		} catch (IOException e) {
			log.error("Error deleting lucene index",e);
		}
	}

	public Directory getDirectory() {
		return directory;
	}


	public void optimize() {
		getLuceneWriter().optimize();
	}

}
