package fi.seco.semweb.util.lucene;

import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.FieldSelector;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Explanation;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.HitCollector;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searchable;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.TopFieldDocs;
import org.apache.lucene.search.Weight;
import org.apache.lucene.store.Directory;

public final class NonExceptingIndexSearcher implements Searchable {

	private IndexSearcher searcher;
	private static Log log = LogFactory.getLog(NonExceptingIndexSearcher.class);

	public NonExceptingIndexSearcher(IndexSearcher searcher) {
		this.searcher=searcher;
	}

	public NonExceptingIndexSearcher(Directory directory) {
		try {
			this.searcher=new IndexSearcher(directory);
		} catch (IOException e) {
			log.error(e);
		}
	}

	public final void close() {
		try {
			searcher.close();
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final Document doc(int arg0) {
		try {
			return searcher.doc(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}
	
	public final int docFreq(Term arg0) {
		try {
			return searcher.docFreq(arg0);
		} catch (IOException e) {
			log.error("",e);
			return -1;
		}
	}

	public final int[] docFreqs(Term[] arg0) {
		try {
			return searcher.docFreqs(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	@Override
	public final boolean equals(Object o) {
		return searcher.equals(o);
	}

	public final Explanation explain(Query arg0, int arg1) {
		try {
			return searcher.explain(arg0, arg1);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final Explanation explain(Weight arg0, int arg1) {
		try {
			return searcher.explain(arg0, arg1);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final IndexReader getIndexReader() {
		return searcher.getIndexReader();
	}

	public final Similarity getSimilarity() {
		return searcher.getSimilarity();
	}

	@Override
	public final int hashCode() {
		return searcher.hashCode();
	}

	public final int maxDoc() {
		try {
			return searcher.maxDoc();
		} catch (IOException e) {
			log.error("",e);
			return -1;
		}
	}

	public final Query rewrite(Query arg0) {
		try {
			return searcher.rewrite(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final void search(Query arg0, Filter arg1, HitCollector arg2) {
		try {
			searcher.search(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final TopFieldDocs search(Query arg0, Filter arg1, int arg2, Sort arg3) {
		try {
			return searcher.search(arg0, arg1, arg2, arg3);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TopDocs search(Query arg0, Filter arg1, int arg2) {
		try {
			return searcher.search(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final NonExceptingHits search(Query arg0, Filter arg1, Sort arg2) {
		try {
			return new NonExceptingHits(searcher.search(arg0, arg1, arg2));
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final NonExceptingHits search(Query arg0, Filter arg1) {
		try {
			return new NonExceptingHits(searcher.search(arg0, arg1));
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final NonExceptingHits search(Query arg0) {
		try {
			return new NonExceptingHits(searcher.search(arg0));
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final void search(Query arg0, HitCollector arg1) {
		try {
			searcher.search(arg0, arg1);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final NonExceptingHits search(Query arg0, Sort arg1) {
		try {
			return new NonExceptingHits(searcher.search(arg0, arg1));
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final void search(Weight arg0, Filter arg1, HitCollector arg2) {
		try {
			searcher.search(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final TopFieldDocs search(Weight arg0, Filter arg1, int arg2, Sort arg3) {
		try {
			return searcher.search(arg0, arg1, arg2, arg3);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TopDocs search(Weight arg0, Filter arg1, int arg2) {
		try {
			return searcher.search(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final void setSimilarity(Similarity arg0) {
		searcher.setSimilarity(arg0);
	}

	@Override
	public final String toString() {
		return searcher.toString();
	}

	public IndexSearcher getSearcher() {
		return searcher;
	}
	
	public Document doc(int i, FieldSelector fieldSelector) {
		try {
			return searcher.doc(i, fieldSelector);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}
}
