package fi.seco.semweb.util.map.iterablemap;

import java.util.AbstractCollection;
import java.util.AbstractSet;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections15.IterableMap;
import org.apache.commons.collections15.MapIterator;
import org.apache.commons.collections15.map.HashedMap;

import fi.seco.semweb.util.iterator.ALazyInitializingIterator;
import fi.seco.semweb.util.iterator.IIterableIterator;
import fi.seco.semweb.util.iterator.mapiterator.ALazyInitializingMapIterator;

public class IteratorIterableMap<T1, T2> implements IterableMap<T1,T2> {

	private IterableMap<T1,T2> map;
	private Iterator<? extends T1> iter;
	private T1 notAddedKey;

	public IteratorIterableMap(IIterableIterator<? extends T1> iter)  {
		this(iter,new HashedMap<T1,T2>());
	}

	public IteratorIterableMap(Iterator<? extends T1> iter) {
		this(iter,new HashedMap<T1,T2>());
	}

	public IteratorIterableMap(Iterable<? extends T1> iterable) {
		this(iterable.iterator(),new HashedMap<T1,T2>());
	}

	public IteratorIterableMap(Iterator<? extends T1> iter, IterableMap<T1,T2> map) {
		this.map=map;
		this.iter=iter;
	}

	public void collapse() {
		if (notAddedKey!=null) map.put(notAddedKey, null);
		for (;iter.hasNext();) map.put(iter.next(), null);
		iter=null;
	}

	public void clear() {
		map.clear();
		iter=null;
	}

	public boolean containsKey(Object arg0) {
		if (iter!=null) collapse();
		return map.containsKey(arg0);
	}

	public boolean containsValue(Object arg0) {
		if (iter!=null) collapse();
		return map.containsValue(arg0);
	}

	public Set<Entry<T1, T2>> entrySet() {
		if (iter!=null && map.isEmpty())
			return new AbstractSet<Entry<T1,T2>>() {

				@Override
				public boolean add(Entry<T1,T2> arg0) {
					return map.put(arg0.getKey(), arg0.getValue()) != null;
				}

				@Override
				public void clear() {
					IteratorIterableMap.this.clear();
				}

				@Override
				public boolean contains(Object arg0) {
					return IteratorIterableMap.this.containsKey(arg0);
				}

				@Override
				public boolean isEmpty() {
					return IteratorIterableMap.this.isEmpty();
				}

				@Override
				public Iterator<Entry<T1,T2>> iterator() {
					if (iter!=null && map.isEmpty()) return new ALazyInitializingIterator<Entry<T1,T2>>() {

						@Override
						protected Iterator<? extends java.util.Map.Entry<T1, T2>> initialize() {
							if (IteratorIterableMap.this.iter==null) return map.entrySet().iterator();
							return new Iterator<Map.Entry<T1, T2>>() {
								public boolean hasNext() {
									boolean hasNext=IteratorIterableMap.this.iter.hasNext();
									if (!hasNext) {
										if (notAddedKey!=null) map.put(notAddedKey,null);
										IteratorIterableMap.this.iter=null;
									}
									return hasNext;
								}

								public Entry<T1,T2> next() {
									if (notAddedKey!=null) map.put(notAddedKey, null);
									notAddedKey = IteratorIterableMap.this.iter.next();
									return new MapEntry(notAddedKey,null);
								}

								class MapEntry implements Entry<T1,T2> {

									private T1 key;
									private T2 value;

									public MapEntry(T1 key, T2 value) {
										this.key=key;
										this.value=value;
									}

									public T1 getKey() {
										return key;
									}

									public T2 getValue() {
										return value;
									}

									public T2 setValue(T2 value) {
										notAddedKey=null;
										map.put(key,value);
										T2 tmp = this.value;
										this.value=value;
										return tmp;
									}

								}

								public void remove() {
									notAddedKey=null;
								}

							};
						}

					};
					else if (iter!=null) collapse();
					return map.entrySet().iterator();
				}

				@Override
				public boolean remove(Object arg0) {
					return IteratorIterableMap.this.remove(arg0) != null;
				}

				@Override
				public int size() {
					return IteratorIterableMap.this.size();
				}

			};
			else if (iter!=null) collapse();
			return map.entrySet();
	}

	@Override
	public boolean equals(Object arg0) {
		if (iter!=null) collapse();
		return map.equals(arg0);
	}

	public T2 get(Object arg0) {
		if (iter!=null) collapse();
		return map.get(arg0);
	}

	@Override
	public int hashCode() {
		if (iter!=null) collapse();
		return map.hashCode();
	}

	public boolean isEmpty() {
		if (iter!=null && iter.hasNext()) return false;
		return map.isEmpty();
	}

	public Set<T1> keySet() {
		if (iter!=null && map.isEmpty())
			return new AbstractSet<T1>() {

				@Override
				public boolean add(T1 arg0) {
					throw new UnsupportedOperationException("Can't add to set backed by a map");
				}

				@Override
				public boolean addAll(Collection<? extends T1> arg0) {
					throw new UnsupportedOperationException("Can't add to set backed by a map");
				}

				@Override
				public void clear() {
					IteratorIterableMap.this.clear();
				}

				@Override
				public boolean contains(Object arg0) {
					return IteratorIterableMap.this.containsKey(arg0);
				}

				@Override
				public boolean isEmpty() {
					return IteratorIterableMap.this.isEmpty();
				}

				@Override
				public Iterator<T1> iterator() {
					if (iter!=null && map.isEmpty()) return new ALazyInitializingIterator<T1>() {

						@Override
						public Iterator<? extends T1> initialize() {
							if (IteratorIterableMap.this.iter==null) return map.keySet().iterator();
							return new Iterator<T1>() {

								public boolean hasNext() {
									boolean hasNext=IteratorIterableMap.this.iter.hasNext();
									if (!hasNext) {
										if (notAddedKey!=null) map.put(notAddedKey, null);
										IteratorIterableMap.this.iter=null;
									}
									return hasNext;
								}

								public T1 next() {
									if (notAddedKey!=null) map.put(notAddedKey, null);
									notAddedKey = IteratorIterableMap.this.iter.next();
									return notAddedKey;
								}

								public void remove() {
									notAddedKey=null;
								}

							};
						}

					};
					else if (iter!=null) collapse();
					return map.keySet().iterator();
				}

				@Override
				public boolean remove(Object arg0) {
					return IteratorIterableMap.this.remove(arg0) != null;
				}

				@Override
				public int size() {
					return IteratorIterableMap.this.size();
				}

			};
			else if (iter!=null) collapse();
			return map.keySet();
	}

	public MapIterator<T1, T2> mapIterator() {
		if (iter!=null && map.isEmpty()) return new ALazyInitializingMapIterator<T1,T2>() {
			@Override protected MapIterator<T1,T2> initialize() {
				if (IteratorIterableMap.this.iter==null) return map.mapIterator();
				return new MapIterator<T1,T2>() {

					private T2 notAddedValue;

					public T1 getKey() {
						return notAddedKey;
					}

					public T2 getValue() {
						return notAddedValue;
					}

					public boolean hasNext() {
						boolean hasNext=IteratorIterableMap.this.iter.hasNext();
						if (!hasNext) {
							if (notAddedKey!=null) map.put(notAddedKey,notAddedValue);
							IteratorIterableMap.this.iter=null;
						}
						return hasNext;
					}

					public T1 next() {
						if (notAddedKey!=null) map.put(notAddedKey,notAddedValue);
						notAddedKey=IteratorIterableMap.this.iter.next();
						notAddedValue=null;
						return notAddedKey;
					}

					public void remove() {
						notAddedKey=null;
					}

					public T2 setValue(T2 arg0) {
						notAddedKey=null;
						map.put(notAddedKey, arg0);
						T2 tmp = notAddedValue;
						notAddedValue=arg0;
						return tmp;
					}
				};
			}
		}; else if (iter!=null) collapse();
		return map.mapIterator();
	}

	public T2 put(T1 arg0, T2 arg1) {
		if (iter!=null) collapse();
		return map.put(arg0, arg1);
	}

	public void putAll(Map<? extends T1, ? extends T2> arg0) {
		if (iter!=null) collapse();
		map.putAll(arg0);
	}

	public T2 remove(Object arg0) {
		if (iter!=null) collapse();
		return map.remove(arg0);
	}

	public int size() {
		if (iter!=null) collapse();
		return map.size();
	}

	public Collection<T2> values() {
		if (iter!=null && map.isEmpty()) return new AbstractCollection<T2>() {

			@Override
			public boolean add(T2 o) {
				throw new UnsupportedOperationException("Can't add to collection backed by a map");
			}

			@Override
			public boolean addAll(Collection<? extends T2> c) {
				throw new UnsupportedOperationException("Can't add to collection backed by a map");
			}

			@Override
			public void clear() {
				IteratorIterableMap.this.clear();
			}

			@Override
			public boolean contains(Object o) {
				return IteratorIterableMap.this.containsValue(o);
			}

			@Override
			public boolean isEmpty() {
				return IteratorIterableMap.this.isEmpty();
			}

			@Override
			public Iterator<T2> iterator() {
				if (iter!=null && map.isEmpty()) return new ALazyInitializingIterator<T2>() {

					@Override
					protected Iterator<? extends T2> initialize() {
						if (IteratorIterableMap.this.iter==null) return map.values().iterator();
						return new Iterator<T2>() {

							public boolean hasNext() {
								boolean hasNext=IteratorIterableMap.this.iter.hasNext();
								if (!hasNext) {
									if (notAddedKey!=null) map.put(notAddedKey,null);
									IteratorIterableMap.this.iter=null;
								}
								return hasNext;
							}

							public T2 next() {
								if (notAddedKey!=null) map.put(notAddedKey,null);
								notAddedKey = IteratorIterableMap.this.iter.next();
								return null;
							}

							public void remove() {
								notAddedKey=null;
							}
						};
					}
				};
				else if (iter!=null) collapse();
				return map.values().iterator();
			}

			@Override
			public int size() {
				return IteratorIterableMap.this.size();
			}

		};
		else if (iter!=null) collapse();
		return map.values();
	}

}
