package fi.seco.semweb.util.srcgen;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.adtech.jastor.JastorContext;
import com.ibm.adtech.jastor.JastorException;
import com.ibm.adtech.jastor.JastorGenerator;
import com.ibm.adtech.jastor.inference.Ontology;
import com.ibm.adtech.jastor.inference.OntologyClass;
import com.ibm.adtech.jastor.jet.OntologyClassFileProvider;
import com.ibm.adtech.jastor.jet.OntologyClassTemplate;
import com.ibm.adtech.jastor.jet.OntologyFileProvider;

import fi.seco.semweb.util.srcgen.templates.FactoryTemplate;
import fi.seco.semweb.util.srcgen.templates.ImplementationTemplate;
import fi.seco.semweb.util.srcgen.templates.InterfaceTemplate;
import fi.seco.semweb.util.srcgen.templates.SimpleImplementationTemplate;

public class SimplerJastorOntology2JavaConverter {

	private static Log log = LogFactory.getLog(SimplerJastorOntology2JavaConverter.class);

	public static String getTypeForClass(String className) {
		if (className.equals("java.lang.Boolean"))
			return "boolean";
		else if (className.equals("java.lang.Byte"))
			return "byte";
		else if (className.equals("java.lang.Character"))
			return "char";
		else if (className.equals("java.lang.Double"))
			return "double";
		else if (className.equals("java.lang.Float"))
			return "float";
		else if (className.equals("java.lang.Integer"))
			return "int";
		else if (className.equals("java.lang.Long"))
			return "long";
		else if (className.equals("java.lang.Short"))
			return "short";
		else if (className.equals("java.lang.String"))
			return "String";
		else if (className.equals("com.hp.hpl.jena.rdf.model.Literal")) return "Literal";
		return className;
	}

	public static String shortName(String className) {
		String[] tmp = className.split("\\.");
		return tmp[tmp.length-1];
	}

	public static void generateAll(String ontologyLocation,String ontologyUri, String srcDir, String packageName) {
		generateAll(ontologyLocation,ontologyUri,srcDir,packageName,"fi.seco.semweb.model.ResourceObject","fi.seco.semweb.model.IResourceObject","fi.seco.semweb.model.ResourceObjectFactory");
	}

	public static void generateAll(String ontologyLocation,String ontologyUri, String srcDir, String packageName, String customThingClass, String customThingInterface, String customThingFactory) {
		System.out.println("Converting "+ontologyLocation+" ("+ontologyUri+") to "+packageName);
		JastorContext ctx = new JastorContext();
		ctx.setGeneratePropertyCache(false);
		ctx.setGenerateCacheInFactory(false);
		ctx.setGenerateListeners(false);
		ctx.setGenerateStandardCode(false);
		ctx.addOntologyClassTemplate("Implementation",new ImplementationTemplate(new OntologyClassFileProvider() {
            public File getFile(OntologyClass oc, File outputDir) {
                return oc.getImplFile(outputDir);
            }
        }));
/*		ctx.addOntologyClassTemplate("Listener",new ListenerTemplate(new OntologyClassFileProvider() {
            public File getFile(OntologyClass oc, File outputDir) {
                return oc.getListenerFile(outputDir);
            }
        })); */
		ctx.addOntologyClassTemplate("Interface",new InterfaceTemplate(new OntologyClassFileProvider() {
            public File getFile(OntologyClass oc, File outputDir) {
                return oc.getInterfaceFile(outputDir);
            }
        }));
		ctx.addOntologyTemplate("Factory",new FactoryTemplate(new OntologyFileProvider() {
			public File getFile(Ontology ont, File outputDir) {
                return ont.getFactoryFile(outputDir);
			}
        }));

		try {
			ctx.setCustomThing(customThingInterface,customThingClass,customThingFactory);
		} catch (JastorException e) {
			log.debug(e);
		}
		try {
			if (ontologyLocation.endsWith("rdfs")) ctx.addOntologyToGenerate(new URL(ontologyLocation).openStream(),JastorContext.ONT_LANG_RDFS,null,ontologyUri,packageName);
			else ctx.addOntologyToGenerate(new URL(ontologyLocation).openStream(),ontologyUri,packageName);
		} catch (MalformedURLException e) {
			log.error("",e);
		} catch (IOException e) {
			log.error("",e);
		}
		JastorGenerator gen;
		try {
			gen = new JastorGenerator( new File(srcDir).getCanonicalFile(),ctx);
			gen.run();
		} catch (IOException e) {
			log.error("",e);
		} catch (JastorException e) {
			log.error("",e);
		}
		System.out.println("Done.");
	}

	public static void generateSimple(String ontologyLocation,String ontologyUri, String srcDir, String packageName) {
		generateSimple(ontologyLocation,ontologyUri,srcDir,packageName,"fi.seco.semweb.model.ResourceObject");
	}

	public static void generateSimple(String ontologyLocation,String ontologyUri, String srcDir, String packageName, String customThingClass) {
		System.out.println("Converting "+ontologyLocation+" ("+ontologyUri+") to "+packageName);
		JastorContext ctx = new JastorContext();
		ctx.setGeneratePropertyCache(false);
		ctx.setGenerateCacheInFactory(false);
		ctx.setGenerateListeners(false);
		ctx.setGenerateStandardCode(false);
		try {
			ctx.setCustomThing(customThingClass,customThingClass,customThingClass);
		} catch (JastorException e) {
			log.debug(e);
		}
		ctx.addOntologyClassTemplate("Simple implementation",new SimpleImplementationTemplate(new OntologyClassFileProvider() {
            public File getFile(OntologyClass oc, File outputDir) {
                return oc.getInterfaceFile(outputDir);
            }
        }));
		try {
			if (ontologyLocation.endsWith("rdfs")) ctx.addOntologyToGenerate(new URL(ontologyLocation).openStream(),JastorContext.ONT_LANG_RDFS,null,ontologyUri,packageName);
			else ctx.addOntologyToGenerate(new URL(ontologyLocation).openStream(),ontologyUri,packageName);
		} catch (MalformedURLException e) {
			log.error("",e);
		} catch (IOException e) {
			log.error("",e);
		}
        Map<?, ?> map = ctx.getOntologyClassTemplates();
        Iterator<?> it = map.keySet().iterator();
        while (it.hasNext()) {
            String name = (String) it.next();
            OntologyClassTemplate oct = (OntologyClassTemplate) map.get(name);
            Iterator<?> itr = ctx.listOntologyClassesToGenerate().iterator();
            while (itr.hasNext()) {
                OntologyClass ont = (OntologyClass) itr.next();
                System.err.println("Generating " + name + " : " + ont.getURI());
                String genstr = oct.generate(ont);
                File genfile;
				try {
					genfile = oct.getFileProvider().getFile(ont,new File(srcDir).getCanonicalFile());
	                genfile.getParentFile().mkdirs();
	                /* if (genfile.exists()) saveFile(genfile,mergeFile(genstr,genfile));
	                else */ saveFile(genfile,genstr);
				} catch (IOException e) {
					e.printStackTrace();
				}
            }
        }
		System.out.println("Done.");
	}

/*   private static String mergeFile(String generated, File targetFile) {
	   try {
		   JMerger jmerger = new JMerger();

		   String uri = "jet_templates/merge.xml";
		   JControlModel controlModel = new JControlModel( uri );
		   jmerger.setControlModel( controlModel );
		   // set source
		   merger.setSourceCompilationUnit(merger.createCompilationUnitForContents(generated));

		   // set target
		   merger.setTargetCompilationUnit(merger.createCompilationUnitForInputStream(new FileInputStream(targetFile)));

		   return merger.getTargetCompilationUnitContents();
	   } catch (IOException e) {
		   e.printStackTrace();
	   }
	   return "";
   } */

   private static void saveFile(File outFile,String data) {
       try {
           BufferedWriter writer = new BufferedWriter(new FileWriter(outFile));
           writer.write(data);
           writer.close();

       } catch (IOException e) {
           System.err.println(e);
       }
   }

}
